checkRequire("Lua_EventOwner");
Lua_DisplayObject = class("Lua_DisplayObject", Lua_EventOwner, 1);
function Lua_DisplayObject:ctor(p, skin, frameBuild)
	self:superCall("ctor");
	self.parent = p;
	self.content = UICreate.createGameObject(p, 0, 0, 0);
	self.content.name = self.classname;

	self.skinName = skin;
	self.frameBuild = (frameBuild ~= nil) and frameBuild or false;
	self.isActive = true;
	self.siblingIndex = 0;

	local screenOffsetH = GlobalVar.screenOffsetTopH + GlobalVar.screenOffsetBottomH;
	self.adaptHeight = MainWinDefine.V_height - MainWinDefine.DEFAULT_SHOW_HEIGHT - screenOffsetH;

	self.delayCallList = Array();
	self.exceptionDic = {};
	self:addExceptions();
	self:buildPanel();
end

function Lua_DisplayObject:addExceptions()
	self.exceptionDic["onBuildingItem"] = 1;
	self.exceptionDic["checkSaveParam"] = 1;
	self.exceptionDic["saveFunKey"] = 1;
	self.exceptionDic["saveParam"] = 1;
	self.exceptionDic["classname"] = 1;
	self.exceptionDic["super"] = 1;
	self.exceptionDic["setLocation"] = 1;
	self.exceptionDic["addExceptions"] = 1;
	self.exceptionDic["saveParamKey"] = 1;
	self.exceptionDic["superCall"] = 1;
	self.exceptionDic["dispose"] = 1;
end

function Lua_DisplayObject:saveParamKey(key, val)
	self.delayCallList:add({ funStr = key, param = val, isFun = false });
end

function Lua_DisplayObject:saveParam(...)
	if self.curKey == nil then
		return 0;
	end

	local params = {};
	params[1],params[2],params[3],params[4],params[5],params[6],params[7],params[8],params[9],params[10],params[11] = ...;
	self.delayCallList:add({ funStr = self.curKey, param = params, isFun = true});
	self.curKey = nil;
end

function Lua_DisplayObject:saveFunKey(key)
	self.curKey = key;
end

function Lua_DisplayObject:checkSaveParam(key)
	if self.skinBuilded == nil or self.exceptionDic[key] == 1 then
		return false;
	end
	return self.skinBuilded == false;
end

function Lua_DisplayObject:buildPanel()
	if self.frameBuild == true and self.skinName ~= nil then
		self.skinBuilded = false;
		self.skinTable = checkRequire(self.skinName);
		self.conTable = {};
		self.conTable["content"] = self["content"];
		self.itemIndex = 1;
		self.content:SetActive(false);
		Lua_CommonUtil.addTimer("BUILD_SKIN", 0.001, self, self.onBuildingItem);
	else
		self.skinBuilded = true;
		self:buildShowItem();
	end
end

function Lua_DisplayObject:onBuildingItem()
	if self.skinBuilded == true or self.isDispose_3 == true then
		Lua_CommonUtil.removeTimer(self, self.onBuildingItem);
		return;
	end

	local isEnd = false;
	local st = getTime();
	while getTime() - st < 0.005 do
		isEnd = Lua_WindowUtil.buildFromSkinEachFrame(self, self.skinTable, self.conTable, self.itemIndex);
		if isEnd == true then
			break;
		else
			self.itemIndex = 1 + self.itemIndex;
		end
	end

	if isEnd == true then
		self.conTable = nil;
		self.skinTable = nil;
		Lua_CommonUtil.removeTimer(self, self.onBuildingItem);
		self.skinBuilded = true;
		self.content:SetActive(true);
		self:buildShowItem();
		local len = self.delayCallList:getSize();
		for i = 1,len do
			local fitem = self.delayCallList[i];
			if fitem.isFun == true then
				local f = self[fitem.funStr];
				local params = fitem.param;
				f(self, params[1],params[2],params[3],params[4],params[5],params[6],params[7],params[8],params[9],params[10],params[11]);
			else
				self[fitem.funStr] = fitem.param;
			end
			
		end
		self.delayCallList:clear();
	end
end

function Lua_DisplayObject:buildFromSetSkin(skinName, itemName)
	local skinTable = checkRequire(skinName);
	local posState = nil;
	for i,v in ipairs(skinTable) do
		if v.type == 6 and v.name == itemName then
			posState = v;
			break;
		end
	end

	if posState ~= nil then
		Lua_WindowUtil.buildFromSkin(self, posState.target_item, self.content);
		self:setLocation(posState.x, posState.y, 0);
	end
end

function Lua_DisplayObject:buildFromSkin()
	if self.frameBuild ~= true and self.skinName ~= nil then
		Lua_WindowUtil.buildFromSkin(self, self.skinName, self.content);
	end
end

function Lua_DisplayObject:buildShowItem()
	self:buildFromSkin();
	self.rectTf = self.content:AddComponent(typeof(UnityEngine.RectTransform));
	self.siblingIndex = self.rectTf:GetSiblingIndex();
	self:setRectSize(0, 0);
end

function Lua_DisplayObject:setEnable(value)
	self.isActive = value;
	self.content:SetActive(value);
end

function Lua_DisplayObject:setParent(p)
	self.parent = p;
	self.content.transform:SetParent(p.transform, false);
end

function Lua_DisplayObject.Set:contentName(value)
	self.content.name = value;
end

function Lua_DisplayObject.Get:mainContent()
	return self.content;
end

-- 不要手动执行这个方法实现内部对象的适配，如果在该方法内有对DragContainer和DragList修改高度的话，会造成二次重绘
function Lua_DisplayObject:onScreenResize(adaptHeight)
	self.adaptHeight = adaptHeight;
end

function Lua_DisplayObject:setSiblingIndex(val)
	self.siblingIndex = val;
	UIUtils.setObjSiblingIndex(self.content, val);
end

function Lua_DisplayObject:setLocationVec(vec)
	if vec ~= nil then
		self:setLocation(vec.x, vec.y, vec.z);
	end
end

function Lua_DisplayObject:setLocation(posX, posY, posZ)
	if posX == nil then posX = 0; end
	if posY == nil then posY = 0; end
	if posZ == nil then posZ = 0; end
	self.posX = posX;
	self.posY = posY;
	self.posZ = posZ;
	LuaUtil.setObjPos(self.content, posX, posY, posZ);
end

function Lua_DisplayObject.Get:x()
	if self.posX ~= nil then
		return self.posX;
	end
	if self.rectTf ~= nil then
		return self.rectTf.anchoredPosition.x;
	end
	return self.content.transform.localPosition.x;
end

function Lua_DisplayObject.Get:y()
	if self.posY ~= nil then
		return self.posY;
	end
	if self.rectTf ~= nil then
		return self.rectTf.anchoredPosition.y;
	end
	return self.content.transform.localPosition.y;
end

function Lua_DisplayObject.Get:z()
	if self.posZ ~= nil then
		return self.posZ;
	end
	if self.rectTf ~= nil then
		return self.rectTf.anchoredPosition.z;
	end
	return self.content.transform.localPosition.z;
end

function Lua_DisplayObject.Get:w()
	if self._width ~= nil then
		return self._width;
	end
	return self.rectTf.sizeDelta.x;
end

function Lua_DisplayObject.Get:h()
	if self._height ~= nil then
		return self._height;
	end
	return self.rectTf.sizeDelta.y;
end

function Lua_DisplayObject:getLocation()
	if self.rectTf ~= nil then
		return self.rectTf.anchoredPosition;
	end
	if self.content ~= nil and self.content.transform ~= nil then
		return self.content.transform.localPosition;
	end
	return nil;
end

function Lua_DisplayObject:setRectSize(width, height)
	if self.rectTf == nil then
		return;
	end
	
	self._width = width == nil and 0 or width;
	self._height = height == nil and 0 or height;
	LuaUtil.setRectSize(self.rectTf, self._width, self._height);
end

function Lua_DisplayObject:setRectAnchor(value)
	if self.rectTf == nil then
		return;
	end
	if value == 0 then -- TopLeft
		LuaUtil.setRectAnchor(self.rectTf, 0, 1, 0, 1);
	elseif value == 1 then -- Top
		LuaUtil.setRectAnchor(self.rectTf, 0.5, 1, 0.5, 1);
	elseif value == 2 then -- TopRight
		LuaUtil.setRectAnchor(self.rectTf, 1, 1, 1, 1);
	elseif value == 3 then -- Left
		LuaUtil.setRectAnchor(self.rectTf, 0, 0.5, 0, 0.5);
	elseif value == 4 then -- Center
		LuaUtil.setRectAnchor(self.rectTf, 0.5, 0.5, 0.5, 0.5);
	elseif value == 5 then -- Right
		LuaUtil.setRectAnchor(self.rectTf, 1, 0.5, 1, 0.5);
	elseif value == 6 then -- BottomLeft
		LuaUtil.setRectAnchor(self.rectTf, 0, 0, 0, 0);
	elseif value == 7 then -- Bottom
		LuaUtil.setRectAnchor(self.rectTf, 0.5, 0, 0.5, 0);
	elseif value == 8 then -- BottomRight
		LuaUtil.setRectAnchor(self.rectTf, 1, 0, 1, 0);
	elseif value == 9 then
		LuaUtil.setRectAnchor(self.rectTf, 0, 0, 1, 1);
	else
		LuaUtil.setRectAnchor(self.rectTf, 0.5, 0.5, 0.5, 0.5);
	end
end

function Lua_DisplayObject:setAnchorMin(x, y)
	if self.rectTf == nil then
		return;
	end
	LuaUtil.setRectAnchorMin(self.rectTf, x, y);
end

function Lua_DisplayObject:setAnchorMax(x, y)
	if self.rectTf == nil then
		return;
	end
	LuaUtil.setRectAnchorMax(self.rectTf, x, y);
end

function Lua_DisplayObject:setRectPivot(value)
	if self.rectTf == nil then
		return;
	end
	UIUtils.setComponentPivot(self.rectTf, value);
end

function Lua_DisplayObject.Get:pivotX()
	return self.rectTf.pivot.x;
end

function Lua_DisplayObject.Get:pivotY()
	return self.rectTf.pivot.y;
end

-- 当前轴心x坐标转换为居中x坐标
function Lua_DisplayObject.Get:centerPosX()
	return self.x - (self.pivotX - 0.5) * self.w;
end

-- 当前轴心y坐标转换为居中y坐标
function Lua_DisplayObject.Get:centerPosY()
	return self.y - (self.pivotY - 0.5) * self.h;
end

function Lua_DisplayObject:setScale(scaleX, scaleY)
	self._scaleX = scaleX or 1;
	self._scaleY = scaleY or 1;
	LuaUtil.setObjScale(self.content, scaleX, scaleY, 1);
end

function Lua_DisplayObject.Get:scaleX()
	return self._scaleX or 1;
end

function Lua_DisplayObject.Get:scaleY()
	return self._scaleY or 1;
end

function Lua_DisplayObject:setRot(rotX, rotY, rotZ)
	if rotX == nil then rotX = 1; end
	if rotY == nil then rotY = 1; end
	if rotZ == nil then rotZ = 1; end
	LuaUtil.setObjRot(self.content, rotX, rotY, rotZ);
end

-- 判断是否范围内
function Lua_DisplayObject:checkInBound(touchX, touchY)
	return false
end

function Lua_DisplayObject:dispose()
	self.rectTf = nil;
	self.conTable = nil;
	if self.skinTable ~= nil then
		for i, v in ipairs(self.skinTable) do
			self[v.name] = nil;
		end
		self.skinTable = nil;
	end

	local str = self.tweenName or self.classname
	DataCenter.winTweenManager:removeWinTween(str)
	self.tweenName = nil
	Lua_CommonUtil.removeTimer(self, self.onBuildingItem);
	Lua_EventOwner.dispose(self);
	if self.isDispose_3 ~= true then
		self.isDispose_3 = true;
		if self.content ~= nil and IsNil(self.content) == false then
			WindowMananger.getInstance():returnToPool(self.content.transform);
			UnityEngine.GameObject.Destroy(self.content);
			self.content = nil;
		end
	end
end