checkRequire("Lua_DragContainerItem");
checkRequire("Lua_DragContainerData");
Lua_DragContainer = class("Lua_DragContainer", Lua_DisplayObject)
function Lua_DragContainer:ctor(p, rw, rh, dt, isMask)
    self._rectWidth = rw;
    self._rectHeight = rh;
    self._dragType = dt;
    self.isMask = isMask;
    self._canDrag = true;

    self.minX = -rw / 2;
    self.maxX = rw / 2;
    self.minY = -rh / 2;
    self.maxY = rh / 2;

    self.maskOffectW = 0;
    self.maskOffectH = 0;
    self.subOffectX = 0;
    self.subOffectY = 0;
    self.subRectW = 0;
    self.subRectH = 0;
    self.subMinPos = 0;
    self.subMaxPox = 0;

    self.cellPivot = 4;
    self.cellPivotX = 0.5;
    self.cellPivotY = 0.5;
    self.cellSpacingX = 0;
    self.cellSpacingY = 0;
    self.isAlphaTween = false;

    self.alpha = 1;
    self.moveTime = 2;
    self.cellsCount = 0;
    self.curSelectedIdx = nil;
    self.showDragDataList = Array();
    self.showList = Array();
    self.hideTab = {};

    self.longPressLimit = nil; -- 长按一定时间后移除
    self.bLongPress = false;

    self.callBackOwner = nil; -- 回调对象
    self.dragCallBack = nil; -- 拖拽回调（返回当前拖拽方向的位置）
    self.turnPageCallBack = nil; -- 翻页回调
    self.moveCallBack = nil; -- 移动完成回调

    self.isSubList = false; -- 是否子列表
    self.ownerList = nil; -- 所属父列表
    self.customParam = nil; -- 自定义参数

    self.showTween = true; -- 是否显示动画，播放一次后设置false，重新播放需要手动设置true
    self.tweenId = 0;
    self.tweenTime = 0;
    self.delayTween = false; -- 延后表现动画，moveToIndex定位后再播放(immediately=true)

    self.isPageEnd = false; -- 是否到最后一页
    self._clickMoveOffset = nil -- 点击后移动偏移值

    self:superCall("ctor", p);
end

function Lua_DragContainer:buildShowItem()
    self:superCall("buildShowItem");

    self.scrollContainer = UICreate.createScrollContainer(self.content, 0, 0, self._rectWidth, self._rectHeight);
    -- self.scrollContainer.bCreateContent = true;
    self.scrollContainer.horizontal = self._dragType == Lua_AxisDefine.Horizontal;
    self.scrollContainer.vertical = self._dragType == Lua_AxisDefine.Vertical;
    self.scrollContainer.movementTypeValue = 1;
    self.scrollContainer.elasticity = 0.3;
    self.scrollContainer.inertia = true;
    self.scrollContainer:setScrollPosition(0, 0, 0);
    self.scrollContainer:setScrollSize(0, 0);
    self.scrollContainer:setScrollAnchor(0.5, 0.5, 0.5, 0.5);
    self.scrollContainer:setScrollPivot(4);
    self.scrollContainer:setNormalizedPosition(0, 1);
    self.scrollContainer:addValueChangedFun(self, self.onValueChanged);
    self.scrollContainer:addMousDownFuntion(self, self.onMouseDownFun);
    self.scrollContainer:addMouseUpFuntion(self, self.onMouseUpFun);
    self.scrollContainer:addDragFun(self, nil, self.onDragFun, self.onEndDragFun);
    self.scrollContainer:addAlphaChangedFun(self, self.onAlphaChanged);
    if self.isMask == true then
        self.scrollContainer:addMask();
    else
        self.scrollContainer:addRectMask2D();
    end

    self.sub = self.scrollContainer.scrollContent;
    self.subRect = self.scrollContainer.scrollRect;

    self:setRectSize(self._rectWidth, self._rectHeight);
end

-- 是否可长按
function Lua_DragContainer.Get:canLongPress()
    return self.scrollContainer ~= nil and self.scrollContainer.canLongPress or false;
end

-- 是否可长按
function Lua_DragContainer.Set:canLongPress(val)
    if val == nil then val = false; end
    if self.scrollContainer ~= nil then self.scrollContainer.canLongPress = val; end
end

function Lua_DragContainer:setScale(scaleX, scaleY)
    LuaUtil.setObjScale(self.content, 1, 1, 1);
end

function Lua_DragContainer.Set:movementTypeValue(value)
    value = value < 0 and 0 or value;
    value = value > 2 and 2 or value;
    self.scrollContainer.movementTypeValue = value;
end

function Lua_DragContainer.Set:inertia(value)
    self.scrollContainer.inertia = value;
end

function Lua_DragContainer.Set:canDrag(value)
    self._canDrag = value;
    if self.isSubList then
        self.scrollContainer.raycastTarget = value;
    else
        self.scrollContainer.canDrag = value;
    end
end

function Lua_DragContainer.Set:clickMoveOffset(value)
    self._clickMoveOffset = value;
end

-- 慎用
function Lua_DragContainer.Get:horizontalNormalizedPosition()
    return self.scrollContainer.horizontalNormalizedPosition;
end

-- 慎用
function Lua_DragContainer.Get:verticalNormalizedPosition()
    return self.scrollContainer.verticalNormalizedPosition;
end

function Lua_DragContainer:onValueChanged()
    if self.callBackOwner ~= nil and self.dragCallBack ~= nil then
        self.dragCallBack(self.callBackOwner, self.curSubPos);
    end

    self:updateDragItem();
end

function Lua_DragContainer:onMouseDownFun()
    self:removeMoveTimer();
    self:removeLongPressTimer();

    self.startMousePos = self.mousePosition;
    self.startClickTime = getTime();
    self.pressItem = self:checkClick();
    if self.pressItem ~= nil then
        self.pressItem:onMouseDownHandle();
    end

    if self.canLongPress then
        self:startLongPressTimer();
    end
end

function Lua_DragContainer:onMouseUpFun()
    self:removeLongPressTimer();
    self:onTurnPage(false);

    if self.pressItem ~= nil then
        self.pressItem:onMouseUpHandle();
        if not self.bLongPress then
            local winPos = LuaUtil.getTouchPosInverseObj(self.sub);
            local val = self.mousePosition - self.startMousePos;
            if self.pressItem:checkInBound(winPos.x, winPos.y) and (not self._canDrag or Mathf.Abs(val) < 4) then
                if self._clickMoveOffset ~= nil then
                    self:moveToIndex(self.pressItem.cellSeqIndex - self._clickMoveOffset, false)
                end

                local bSelected, bCheckOwner = self.pressItem:onPressClick();
                if bSelected == true then
                    self:setSelected(self.pressItem);
                end

                -- 检测父列表点击
                if bCheckOwner == true and self.ownerList ~= nil and self.ownerList.isActive then
                    local pressItem = self.ownerList:checkClick();
                    if pressItem ~= nil then
                        if pressItem:onPressClick() == true then
                            self.ownerList:setSelected(pressItem);
                        end
                    end
                end

                -- 作为子列表不可拖动时处理
                local subList = self:checkSubClick();
                if subList ~= nil then
                    local pressItem = subList:checkClick();
                    if pressItem ~= nil then
                        if pressItem:onPressClick() == true then
                            subList:setSelected(pressItem);
                        end
                    end
                end
            end
        end
    end
    self.pressItem = nil;
    self.bLongPress = false;
end

function Lua_DragContainer:onDragFun()
    if not self.isPageEnd and self.bDragEnd and self.turnPageCallBack then
        self.loadLabel:setEnable(true);
    end
    -- if self.bDragEnd then
    --     self:onTurnPage(true);
    -- end
end

function Lua_DragContainer:onEndDragFun()
    if not self.isPageEnd and self.bDragEnd then
        self:onTurnPage(true);
        if self._loadLabel then
            self.loadLabel:setEnable(false);
        end
    end
end

function Lua_DragContainer:onAlphaChanged(val)
    self.alpha = val;
    LuaUtil.setObjAlpha(self.sub, val);
end

function Lua_DragContainer:checkClick()
    local winPos = LuaUtil.getTouchPosInverseObj(self.sub);
    for i = 1, self.showList:getSize() do
        if self.showList[i]:checkInBound(winPos.x, winPos.y) then
            return self.showList[i];
        end
    end
    return nil;
end

function Lua_DragContainer:startLongPressTimer()
    if self.bLongPressTimer ~= true then
        self.bLongPressTimer = true;
        self:addTimer("Lua_DragContainer_LongPress", 0.01, self, self.onLongPress, nil, -1);
    end
end

function Lua_DragContainer:removeLongPressTimer()
    if self.bLongPressTimer == true then
        self.bLongPressTimer = false;
        self:removeTimer(self, self.onLongPress);
    end
end

function Lua_DragContainer:onLongPress()
    local val = self.mousePosition - self.startMousePos;
    local t = getTime() - self.startClickTime;
    if t > 0.3 and Mathf.Abs(val) < 2 then
        self.bLongPress = true;
        if self.longPressLimit ~= nil and t >= self.longPressLimit then
            self:removeLongPressTimer();
            return;
        end
        if self.pressItem ~= nil then
            if self.pressItem:onLongPressClick() == true then
                self:setSelected(self.pressItem);
            end
        end
    end
end

function Lua_DragContainer.Get:mousePosition()
    if self._dragType == Lua_AxisDefine.Horizontal then
        return CommonUtil.mousePositionX;
    end
    return CommonUtil.mousePositionY;
end

-- 设置宽高
function Lua_DragContainer:setContentRect(w, h, bUpdate)
    if self._rectWidth == w and self._rectHeight == h then
        return;
    end
    
    w = w or 0;
    h = h or 0;
    self._rectWidth = w;
    self._rectHeight = h;
    self.minX = -w / 2;
    self.maxX = w / 2;
    self.minY = -h / 2;
    self.maxY = h / 2;
    self.scrollContainer:SetDimensions(w + self.maskOffectW, h + self.maskOffectH);
    self:setRectSize(w, h);
    if bUpdate ~= false then
        self:updateChildPos();
    end
end

-- 设置mask偏移值
function Lua_DragContainer:setMaskOffect(w, h)
    self.maskOffectW = w == nil and 0 or w;
    self.maskOffectH = h == nil and 0 or h;
    self.scrollContainer:SetDimensions(self._rectWidth + w, self._rectHeight + h);
end

-- 设置sub偏移值
function Lua_DragContainer:setSubOffect(x, y)
    self.subOffectX = x == nil and 0 or x;
    self.subOffectY = y == nil and 0 or y;
end

-- 设置cell对象pivot
function Lua_DragContainer:setCellPivot(cellPivot)
    self.cellPivot = cellPivot == nil and 4 or cellPivot;
    self.cellPivotX, self.cellPivotY = self:getPivotValue(cellPivot);
end

-- 设置cell对象之间的间距
function Lua_DragContainer:setCellSpacing(spacingX, spacingY)
    self.cellSpacingX = spacingX;
    self.cellSpacingY = spacingY;
end

function Lua_DragContainer:getPivotValue(v)
    if v == 0 then
        return 0, 1;
    elseif v == 1 then
        return 0.5, 1;
    elseif v == 2 then
        return 1, 1;
    elseif v == 3 then
        return 0, 0.5;
    elseif v == 4 then
        return 0.5, 0.5;
    elseif v == 5 then
        return 1, 0.5;
    elseif v == 6 then
        return 0, 0;
    elseif v == 7 then
        return 0.5, 0;
    elseif v == 8 then
        return 1, 0;
    end
    return 0.5, 0.5;
end

function Lua_DragContainer:addSubContainer(cellClass, data, bUpdatePos, offsetVal)
    if cellClass == nil then 
        return;
    end

    offsetVal = offsetVal == nil and 0 or offsetVal;
    local vo = Lua_DragContainerData.new(self, self.cellPivot);
    vo:init(cellClass, data, offsetVal);
    vo:returnCellToPool();
    self.showDragDataList:add(vo);

    if bUpdatePos == true then
        self:updateChildPos();
    end
end

function Lua_DragContainer:updateChildPos()
    -- horizontal 0-1 vertical 1-0
    local horizontalNormalizedPosition = 0;
    local verticalNormalizedPosition = 1;
    if self.isFirstEnter == false then
        horizontalNormalizedPosition = self.horizontalNormalizedPosition;
        verticalNormalizedPosition = self.verticalNormalizedPosition;
    end
    self.scrollContainer:setNormalizedPosition(horizontalNormalizedPosition, verticalNormalizedPosition);
    self.isFirstEnter = false;

    self.cellsCount = self.showDragDataList:getSize();
    self:updateSubSize();

    local data;
    local itemX;
    local itemY;
    local blockSizeX;
    local blockSizeY;
    local posX;
    local posY;
    local maxSize = 0;
    for i = 1, self.cellsCount do
        data = self.showDragDataList[i];
        posX = -self.subRectW / 2 + self.cellPivotX * data.cellWidth;
        posY = self.subRectH / 2 - (1 - self.cellPivotY) * data.cellHeight;
        if self._dragType == Lua_AxisDefine.Horizontal then
            if i < self.cellsCount then
                blockSizeX = data.cellWidth + self.cellSpacingX;
            else
                blockSizeX = data.cellWidth;
            end
            itemX = posX + maxSize;
            itemY = posY + data.offsetVal;
            maxSize = maxSize + blockSizeX;
        else
            if i < self.cellsCount then
                blockSizeY = data.cellHeight + self.cellSpacingY;
            else
                blockSizeY = data.cellHeight;
            end
            itemX = posX + data.offsetVal;
            itemY = posY - maxSize;
            maxSize = maxSize + blockSizeY;
        end
        data:updateCellPos(i, itemX, itemY);
    end

    self:updateDragItem();
    if not self.delayTween then
        self:playTween();
    end
end

function Lua_DragContainer:updateSubSize()
    self.subRectW = 0;
    self.subRectH = 0;
    local data;
    if self._dragType == Lua_AxisDefine.Horizontal then
        for i = 1, self.cellsCount do
            data = self.showDragDataList[i];
            self.subRectW = self.subRectW + data.cellWidth;
        end
        self.subRectW = self.subRectW + (self.cellsCount - 1) * self.cellSpacingX;
        self.subRectW = self.subRectW < self._rectWidth and self._rectWidth or self.subRectW;
        self.subRectH = self._rectHeight;
        LuaUtil.setRectSize(self.subRect, self.subRectW, self.subRectH);

        self.subMinPos = -(self.subRectW - self._rectWidth) / 2;
        self.subMaxPox = -self.subMinPos;
        if self.bUpdateSubSize ~= true then
            self.bUpdateSubSize = true;
            self:setSubPosition(self.subMaxPox); -- checkItemInBound 时可以减少item的创建回收
        end
    else
        self.subRectW = self._rectWidth;
        for i = 1, self.cellsCount do
            data = self.showDragDataList[i];
            self.subRectH = self.subRectH + data.cellHeight;
        end
        self.subRectH = self.subRectH + (self.cellsCount - 1) * self.cellSpacingY;
        self.subRectH = self.subRectH < self._rectHeight and self._rectHeight or self.subRectH;
        LuaUtil.setRectSize(self.subRect, self.subRectW, self.subRectH);

        self.subMinPos = -(self.subRectH - self._rectHeight) / 2;
        self.subMaxPox = -self.subMinPos;
        if self.bUpdateSubSize ~= true then
            self.bUpdateSubSize = true;
            self:setSubPosition(self.subMinPos); -- checkItemInBound 时可以减少item的创建回收
        end
    end
end

function Lua_DragContainer:updateDragItem()
    local subRectX = self.subRect.anchoredPosition.x;
    local subRectY = self.subRect.anchoredPosition.y;
    local centerOffX; -- cell当前轴心的x坐标转换为居中x坐标偏移值
    local centerOffY; -- cell当前轴心的x坐标转换为居中x坐标偏移值
    local posX; -- 把对象X转化为父对象sub X坐标
    local posY; -- 把对象Y转化为父对象sub Y坐标
    local blockSizeX;
    local blockSizeY;
    local data;
    local idx = 0;
    for i = 1, self.cellsCount do
        data = self.showDragDataList[i];

        centerOffX = (self.cellPivotX - 0.5) * data.cellWidth;
        centerOffY = (self.cellPivotY - 0.5) * data.cellHeight;
        posX = data.itemX - centerOffX + subRectX;
        posY = data.itemY - centerOffY + subRectY;

        blockSizeX = (data.cellWidth + self.cellSpacingX) / 2;
        blockSizeY = (data.cellHeight + self.cellSpacingY) / 2;

        if self:checkItemInBound(posX, posY, blockSizeX, blockSizeY) then
            data:getCellFromPool(self);
            data.cell:setSiblingIndex(idx);
            idx = idx + 1;
        else
            data:returnCellToPool();
        end
    end
end

function Lua_DragContainer:checkItemInBound(posX, posY, blockSizeX, blockSizeY)
    if self._dragType == Lua_AxisDefine.Horizontal then
        if posX > self.minX - blockSizeX and posX < self.maxX + blockSizeX then
            return true;
        end
    else
        if posY > self.minY - blockSizeY and posY < self.maxY + blockSizeY then
            return true;
        end
    end
    return false;
end

function Lua_DragContainer:playTween()
    self:removeTweenTimer();
    if self.showTween == true then
        self.showTween = false;
        self.isAlphaTween = false;
        local tweenData = Lua_UITweenConfigData.getConfigData(self.tweenId);
        if tweenData == nil then
            return;
        end

        self:updateShowListSort();
        local len = self.showList:getSize();
        local item;
        local startPosX;
        local startPosY;
        for i = 1, len do
            item = self.showList[i];
            TweenUtil.getInstance():stop(item.content, self.isAlphaTween);

            if tweenData.startRot ~= nil then
                LuaUtil.setObjRot(item.content, tweenData.startRot.x, tweenData.startRot.y, tweenData.startRot.z);
            end

            startPosX = nil;
            startPosY = nil;
            if tweenData.startPos ~= nil then
                if tweenData.startPos.x ~= 0 then
                    startPosX = tweenData.startPos.x * self.subRectW;
                    startPosY = self.showDragDataList[item.cellSeqIndex].itemY;
                elseif tweenData.startPos.y ~= 0 then
                    startPosX = self.showDragDataList[item.cellSeqIndex].itemX;
                    startPosY = tweenData.startPos.y * self.subRectH;
                end
            end
            startPosX = startPosX == nil and self.showDragDataList[item.cellSeqIndex].itemX or startPosX;
            startPosY = startPosY == nil and self.showDragDataList[item.cellSeqIndex].itemY or startPosY;
            if tweenData.offsetPosTab ~= nil then
                startPosX = startPosX + tweenData.offsetPosTab[1];
                startPosY = startPosY + tweenData.offsetPosTab[2];
            end
            LuaUtil.setObjPos(item.content, startPosX, startPosY, 0);

            if tweenData.startScale ~= nil then
                LuaUtil.setObjScale(item.content, tweenData.startScale.x, tweenData.startScale.y, tweenData.startScale.z);
            end

            local delay = (i - 1) * tweenData.delayTime + tweenData.firstDelayTime;
            local offect = tweenData.offect;
            local targetPosX = self.showDragDataList[item.cellSeqIndex].itemX;
            local targetPosY = self.showDragDataList[item.cellSeqIndex].itemY;
            local targetRot = tweenData.targetRotTab;
            local targetScale = tweenData.targetScaleTab;
            item:setEnable(not tweenData.delayShow);

            if tweenData.bFade then
                self.isAlphaTween = true;
                TweenUtil.getInstance():addTweenTargetAlpha(item.content, tweenData.keepTime, delay, tweenData.fadeValue, tweenData.isFadeIn);
            end

            TweenUtil.getInstance():addTween(item.content, tweenData.keepTime, delay, 
                targetPosX, targetPosY, 0, 
                targetRot[1], targetRot[2], targetRot[3], 
                targetScale[1], targetScale[2], targetScale[3], 
                nil, nil, offect, tweenData.delayShow);

            if self.tweenTime < (tweenData.keepTime + delay) then
                self.tweenTime = tweenData.keepTime + delay;
            end
        end

        self.bTweenTimer = true;
        Lua_CommonUtil.delayCall(self.tweenTime, self, self.onTweenCompleted);
    end
end

function Lua_DragContainer:onTweenCompleted()
    self:removeTweenTimer();
    if self.moveIndex ~= nil then
        self:moveToIndex(self.moveIndex, self.immediately, self.isBottom);
        self.moveIndex = nil;
        self.immediately = nil;
        self.isBottom = nil;
    end
end

function Lua_DragContainer:removeTweenTimer()
    if self.bTweenTimer == true then
        self.bTweenTimer = false;
        Lua_CommonUtil.removeTimer(self, self.onTweenCompleted);
    end
    self.tweenTime = 0;
end

function Lua_DragContainer:setChildData(data, isSpecial)
    isSpecial = isSpecial ~= nil and isSpecial or false;
    local vo;
    if type(data) == "table" and data.typeof ~= nil and data:typeof() == "Array" and isSpecial == false then -- 数组
        local cell
        for i = 1, data:getSize() do
            vo = self.showDragDataList[i];
            if vo then
                if vo.cell == nil then
                    cell = self:createCellItem(vo.cellClass);
                    cell:setData(data[i]);
                    vo.cellWidth = cell:getWidth();
                    vo.cellHeight = cell:getHeight();
                    self:returnCell(cell);
                else
                    vo:updateCellData(data[i]);
                end
            end
        end
    else
        vo = self.showDragDataList[1];
        if vo then
            if vo.cell == nil then
                cell = self:createCellItem(vo.cellClass);
                cell:setData(data);
                vo.cellWidth = cell:getWidth();
                vo.cellHeight = cell:getHeight();
                self:returnCell(cell);
            else
                vo:updateCellData(data);
            end
        end
    end
    self:updateChildPos();
end

function Lua_DragContainer:updateChildData()
    local len = self.showDragDataList:getSize();
    local vo;
    local width;
    local height;
    for i = 1, len do
        vo = self.showDragDataList[i];
        if vo.cell ~= nil then
            width = vo.cell:getWidth();
            height = vo.cell:getHeight();
            vo.cellWidth = width;
            vo.cellHeight = height;
            vo.cell:setRectSize(width, height);
        end
    end
    self:updateChildPos();
end

-- immediately 立即到达指定索引
-- isBottom == false 选中某项在显示区域顶部或左侧  isBottom == true选中某项在显示区域底部或右侧
function Lua_DragContainer:moveToIndex(idx, immediately, isBottom)
    if self.cellsCount <= 0 then
        return;
    end

    immediately = immediately ~= nil and immediately or false;
    isBottom = isBottom ~= nil and isBottom or false;
    if not self.delayTween and self.bTweenTimer then
        self.moveIndex = idx;
        self.immediately = immediately;
        self.isBottom = isBottom;
        return;
    end

    idx = idx <= 0 and 1 or idx;
    idx = idx > self.cellsCount and self.cellsCount or idx;

    local data = self.showDragDataList[idx];
    if data == nil then
        return;
    end

    local targetPos;
    if self._dragType == Lua_AxisDefine.Horizontal then
        targetPos = self.subMaxPox;
        for i = 1, idx - 1 do
            data = self.showDragDataList[i];
            targetPos = targetPos - data.cellWidth - self.cellSpacingX;
        end
        if isBottom then
            data = self.showDragDataList[idx];
            targetPos = targetPos - data.cellWidth + self._rectWidth;
        end 
    else
        targetPos = self.subMinPos;
        for i = 1, idx - 1 do
            data = self.showDragDataList[i];
            targetPos = targetPos + data.cellHeight + self.cellSpacingY;
        end
        if isBottom then
            data = self.showDragDataList[idx];
            targetPos = targetPos + data.cellHeight - self._rectHeight;
        end       
    end

    if targetPos > self.subMaxPox then
        targetPos = self.subMaxPox;
    elseif targetPos < self.subMinPos then
        targetPos = self.subMinPos;
    end
    
    if immediately then
        MsgUtil.callFunc("FPS_INTERMISSION_HIGHT");
        self:setSubPosition(targetPos);
        self:updateDragItem(); -- 为了setSelected获得showshowDragDataList中的cell，直接更新而不采用onValueChanged更新
        if self.delayTween then
            self:playTween();
        end
    else
        self:moveToTargetPos(targetPos);
    end
end

function Lua_DragContainer:setSubPosition(val)
    if self._dragType == Lua_AxisDefine.Horizontal then
        self.scrollContainer:setScrollPosition(val + self.subOffectX, self.subOffectY, 0);
    else
        self.scrollContainer:setScrollPosition(self.subOffectX, val + self.subOffectY, 0);
    end
end

function Lua_DragContainer:moveToTargetPos(val)
    self.targetPos = val;
    self:removeMoveTimer();
    self:startMoveTimer();
end

function Lua_DragContainer:startMoveTimer()
    if self.bMoveTimer ~= true then
        self.bMoveTimer = true;
        self.startMoveTime = getTime()
        Lua_CommonUtil.addTimer("Lua_DragContainer_Move", 0.01, self, self.onMoveTimer);
    end
end

function Lua_DragContainer:removeMoveTimer()
    if self.bMoveTimer == true then
        self.bMoveTimer = false;
        Lua_CommonUtil.removeTimer(self, self.onMoveTimer);
    end
    self.moveIndex = nil;
    self.immediately = nil;
    self.isBottom = nil;
end

function Lua_DragContainer:onMoveTimer()
    local dt = getTime() - self.startMoveTime
    if dt >= self.moveTime then
        self:removeMoveTimer()
        self:setSubPosition(self.targetPos)
        if self.callBackOwner ~= nil and self.moveCallBack ~= nil then
            self.moveCallBack(self.callBackOwner)
        end
    else
        self:setSubPosition(Mathf.Lerp(self.curSubPos, self.targetPos, dt / self.moveTime))
    end
    -- self:updateDragItem(); -- 采用onValueChanged更新
end

function Lua_DragContainer.Get:curSubPos()
    if self._dragType == Lua_AxisDefine.Horizontal then
        return self.subRect.anchoredPosition.x - self.subOffectX;
    else
        return self.subRect.anchoredPosition.y - self.subOffectY;
    end
end

function Lua_DragContainer:setSelectClick(index)
    index = index <= 0 and 1 or index;
    index = index > self.cellsCount and self.cellsCount or index;
    local data = self.showDragDataList[index];
    if data ~= nil and data.cell ~= nil then
        data.cell:onPressClick();
    end
end

function Lua_DragContainer:setSelected(val)
    self.curSelectedIdx = nil;
    if val == nil or self.cellsCount <= 0 then
        return;
    end

    local item = nil
    if type(val) == "number" then
        val = val <= 0 and 1 or val;
        val = val > self.cellsCount and self.cellsCount or val;
        item = self.showDragDataList:getItem(val).cell;
    elseif type(val) == "table" then
        item = val;
    end
    
    if item ~= nil then
        local rs;
        local showItem;
        local len = self.showList:getSize()
        for i = 1, len do
            showItem = self.showList:getItem(i);
            rs = (showItem == item)
            if showItem.selected ~= rs and showItem.onSelectionChange ~= nil then
                showItem:onSelectionChange(rs);
            end
            if showItem.selected == true then
                self.curSelectedIdx = showItem.cellSeqIndex;
            end
        end
    end
end

-- 是否拖动到底
function Lua_DragContainer.Get:bDragEnd()
    if self._dragType == Lua_AxisDefine.Horizontal then
        if self.curSubPos <= self.subMinPos then
            return true;
        end
    else
        if self.curSubPos >= self.subMaxPox then
            return true;
        end
    end
    return false;
end

-- 是否到底
function Lua_DragContainer.Get:isTheEnd()
    if self._dragType == Lua_AxisDefine.Horizontal then
        if self.subRectW <= self._rectWidth then
            return true;
        end
    else
        if self.subRectH <= self._rectHeight then
            return true;
        end
    end
    return false;
end

-- 翻页
function Lua_DragContainer:onTurnPage(value)
    if self.isPageEnd or not value then
        if self._loadLabel then
            self.loadLabel:setEnable(false);
        end
        return;
    end
    if self.turnPageCallBack == nil or self.bTurnPage == value then
        return;
    end
    self.bTurnPage = value;

    if self.bTurnPage then
        self.turnPageCallBack(self.callBackOwner);
    --     self.loadLabel:setEnable(true);
    -- else
    --     if self._loadLabel then
    --         self.loadLabel:setEnable(false);
    --     end
    end
end

function Lua_DragContainer.Get:loadLabel()
    if self._loadLabel == nil then
        self._loadLabel = UICreate.createLabel(self.content, "Load Next", 0, 0, 100, 31, 22);
        self._loadLabel.pivotValue = 4
        self._loadLabel.alignmentValue = 4;
        self._loadLabel.horizontalOverflowValue = 1;
        self._loadLabel:setLocation(0, -self._rectHeight / 2 + 15, 0);
        self._loadLabel:setEnable(false);
    end
    return self._loadLabel;
end

-- 设置子列表的父列表对象 处理子列表与父列表同时拖动
function Lua_DragContainer:setOwnerList(ownerList)
    self.ownerList = ownerList;
    if ownerList == nil then
        self.scrollContainer.ownerConteiner = nil;
    else
        self.scrollContainer.ownerConteiner = ownerList.scrollContainer;
    end
end

function Lua_DragContainer:createCellItem(cell)
    if self.hideTab[cell.classname] == nil then
        self.hideTab[cell.classname] = Array()
    end

    local item;
    local arr = self.hideTab[cell.classname];
    if arr:getSize() > 0 then
        item = arr:remove(1);
        item:setEnable(true);
    else
        item = cell.new(self.sub, nil, nil);
    end

    if self.alpha ~= 1 then
        LuaUtil.setObjAlpha(item.content, self.alpha);
    end
    self.showList:add(item);
    return item;
end

function Lua_DragContainer:returnCell(item)
    if item == nil then
        return;
    end

    local len = self.showList:getSize();
    for i = 1, len do
        if self.showList[i] == item then
            TweenUtil.getInstance():stop(item.content, self.isAlphaTween);
            item:setEnable(false);
            self.showList:remove(i);
            self:returnCellToHide(item)
            if self.alpha ~= 1 then
                LuaUtil.setObjAlpha(item.content, 1);
            end
            break;
        end
    end
end

function Lua_DragContainer:returnCellToHide(item)
    if item == nil then 
        return; 
    end

    if item.clean ~= nil then
        item:clean();
    end
    if self.hideTab[item.classname] == nil then
        self.hideTab[item.classname] = Array()
    end
    self.hideTab[item.classname]:add(item)
end

-- 外部调用
function Lua_DragContainer:clearList()
    self:removeMoveTimer();
    self:removeTweenTimer();
    self:removeLongPressTimer();
    self.cellsCount = 0;
    local item;
    local len = self.showList:getSize();
    while (len > 0) do
        item = self.showList:remove(len);
        if item ~= nil then
            TweenUtil.getInstance():stop(item.content, self.isAlphaTween);
            item:setEnable(false);
            self:returnCellToHide(item)
            item = nil
        end
        len = len - 1;
    end

    len = self.showDragDataList:getSize();
    while (len > 0) do
        item = self.showDragDataList:remove(len);
        if item ~= nil then
            item:clean();
            item:dispose();
            item = nil
        end
        len = len - 1;
    end
end

function Lua_DragContainer:updateShowList()
    local len = self.showList:getSize();
    for i = 1, len do
        if self.showList[i].updateData ~= nil then
            self.showList[i]:updateData();
        end
    end
end

function Lua_DragContainer:updateShowListSort()
    quickSort(self.showList, "cellSeqIndex", true);
end

function Lua_DragContainer:dispose()
    self:removeMoveTimer();
    self:removeTweenTimer();
    self:removeLongPressTimer();

    self.callBackOwner = nil;
    self.dragCallBack = nil;
    self.turnPageCallBack = nil;

    if self.content ~= nil then
        TweenUtil.getInstance():stop(self.content);
    end

    local item;
    if self.showDragDataList ~= nil then
        local len = self.showDragDataList:getSize();
        while len > 0 do
            item = self.showDragDataList:remove(len);
            item:clean();
            item:dispose();
            item = nil;
            len = len - 1;
        end
        self.showDragDataList:dispose();
        self.showDragDataList:clear();
        self.showDragDataList = nil;
    end

    if self.showList ~= nil then
        while self.showList:getSize() > 0 do
            item = self.showList:remove();
            TweenUtil.getInstance():stop(item.content, self.isAlphaTween);
            item:dispose();
            item = nil;
        end
        self.showList:dispose();
        self.showList:clear();
        self.showList = nil;
    end

    if self.hideTab then
        for _, arr in pairs(self.hideTab) do
            while arr:getSize() > 0 do
                item = arr:remove();
                TweenUtil.getInstance():stop(item.content, self.isAlphaTween);
                item:dispose();
                item = nil;
            end
            arr:dispose();
            arr:clear();
            arr = nil;
        end
    end

    if self.hideList ~= nil then
        while self.hideList:getSize() > 0 do
            item = self.hideList:remove();
            TweenUtil.getInstance():stop(item.content, self.isAlphaTween);
            item:dispose();
            item = nil;
        end
        self.hideList:dispose();
        self.hideList:clear();
        self.hideList = nil;
    end

    self.ownerList = nil;
    self.pressItem = nil;
    self.sub = nil;
    self.subRect = nil;
    if self.scrollContainer ~= nil then
        self.scrollContainer:dispose();
        self.scrollContainer = nil;
    end

    self:superCall("dispose");
end





--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
-- 子列表处理
function Lua_DragContainer:checkSubClick()
    if self.pressItem == nil or self.pressItem.subList == nil or not self.pressItem.subList.isActive then
        return nil;
    end

    local winPos = LuaUtil.getTouchPosInverseObj(self.sub);
    local subList = self.pressItem.subList;
    local subX = subList.centerPosX + self.pressItem.centerPosX;
    local subY = subList.centerPosY + self.pressItem.centerPosY;
    local minX = subX - subList.w / 2;
    local maxX = subX + subList.w / 2;
    local minY = subY - subList.h / 2;
    local maxY = subY + subList.h / 2;
    if self:checkSubInBound(winPos.x, winPos.y, minX, maxX, minY, maxY) then
        return subList;
    end
    return nil;
end

function Lua_DragContainer:checkSubInBound(posX, posY, minX, maxX, minY, maxY)
    if posX >= minX and posX <= maxX and posY >= minY and posY <= maxY then
        return true;
    end
    return false
end
