Lua_DragContainerData = class("Lua_DragContainerData");
function Lua_DragContainerData:ctor(ownerList, pivot)
	self.ownerList = ownerList;
	self.pivot = pivot == nil and 4 or pivot;

	self.itemX = 0;
	self.itemY = 0;
	self.data = nil;
	self.cell = nil;
	self.index = nil;
	self.cellWidth = 0;
	self.cellHeight = 0;
end

function Lua_DragContainerData:init(cellClass, data, offsetVal)
	self.cellClass = cellClass;
	self.data = data;
	self.offsetVal = offsetVal;

	self.cell = self.ownerList:createCellItem(self.cellClass);
	if data ~= nil then
		self:updateCellData(data);
	end
end

function Lua_DragContainerData:updateCellData(data)
	self.data = data;
	if self.cell ~= nil then
		self.cell:setData(data);
		self.cellWidth = self.cell:getWidth();
		self.cellHeight = self.cell:getHeight();
	end
	self:returnCellToPool();
end

function Lua_DragContainerData:updateCellPos(index, posX, posY)
	self.index = index;
	self.itemX = posX;
	self.itemY = posY;
	if self.cell ~= nil then
		self.cell:setLocation(self.itemX, self.itemY);
	end
end

function Lua_DragContainerData:returnCellToPool()
	if self.cell ~= nil then
		self.ownerList:returnCell(self.cell);
		self.cell = nil;
		self.ownerList = nil;
	end
end

function Lua_DragContainerData:getCellFromPool(ownerList)
	self.ownerList = ownerList;
	if self.cell == nil then
		self.cell = self.ownerList:createCellItem(self.cellClass);
        self.cell:setRectPivot(self.pivot);
        self.cell:setLocation(self.itemX, self.itemY);
        self.cell:setScale(1, 1, 1);
		self.cell:setData(self.data);
		self.cell:setRectSize(self.cellWidth, self.cellHeight);
		self.cell:setCellSeqIndex(self.index);

		if self.cell.subList ~= nil then
			self.cell.subList:setOwnerList(self.ownerList);
		end
	end

	if self.cell.onSelectionChange ~= nil then
		self.cell:onSelectionChange(self.ownerList.curSelectedIdx == self.index);
	end
end

function Lua_DragContainerData:clean()
	self:returnCellToPool();
	self.data = nil;
end

function Lua_DragContainerData:dispose()
	self.cell = nil;
	self.data = nil;
	self.itemX = nil;
	self.itemY = nil;
	self.index = nil;
	self.cellClass = nil;
	self.ownerList = nil;
	self.cellWidth = nil;
	self.cellHeight = nil;
end