checkRequire("Lua_ListCell");
checkRequire("Lua_DragListData");
Lua_DragList = class("Lua_DragList", Lua_DragContainer)
function Lua_DragList:ctor(p, rw, rh, dt, isMask)
    self:superCall("ctor", p, rw, rh, dt, isMask);
    self.hideList = Array();
    self.hCount = 0; -- 一行个数，即几列
    self.startIdx = 1;
end

function Lua_DragList:setListCell(cellClass, cellWidth, cellHeight, hvNum)
    self.cellClass = cellClass;
    self.cellWidth = cellWidth;
    self.cellHeight = cellHeight;
    self.hvNum = hvNum; -- 几行或几列

    self.blockSizeX = cellWidth + self.cellSpacingX;
    self.blockSizeY = cellHeight + self.cellSpacingY;
end

function Lua_DragList:setCellSpacing(spacingX, spacingY)
    self:superCall("setCellSpacing", spacingX, spacingY);
    self.blockSizeX = self.cellWidth + spacingX;
    self.blockSizeY = self.cellHeight + spacingY;
end

function Lua_DragList:setArrayData(data, startIdx)
    -- horizontal 0-1 vertical 1-0
    local horizontalNormalizedPosition = 0;
    local verticalNormalizedPosition = 1;
    if self.isFirstEnter == false then
        horizontalNormalizedPosition = self.horizontalNormalizedPosition;
        verticalNormalizedPosition = self.verticalNormalizedPosition;
    end

    self:removeMoveTimer();
    self:removeTweenTimer();
    self:removeLongPressTimer();
    self:clearList();
    self.cellsCount = 0;
    -- self.curSelectedIdx = nil; -- 重新赋值需要保留当前选择，这里不能置空

    self.data = data;
    if data == nil then
        return;
    end

    if startIdx then
        self.startIdx = startIdx;
    end
    if data.typeof ~= nil and data:typeof() == type_str_array then
        self.cellsCount = data:getSize();
    else
        self.cellsCount = #data;
    end
    self.cellsCount = self.cellsCount - (self.startIdx - 1);
    if self.cellsCount <= 0 then self.cellsCount = 0 end

    local vo;
    local num = self.showDragDataList:getSize() - self.cellsCount;
    while (num > 0) do
        vo = self.showDragDataList:remove();
        vo:clean();
        num = num - 1;
    end
    while (num < 0) do
        vo = Lua_DragListData.new(0, 0, nil, self, self.cellPivot);
        self.showDragDataList:add(vo);
        num = num + 1;
    end

    self.tempSubRectH = self.subRectH
    self:updateChildPos();

    if self.isFirstEnter == false then
        if self._dragType == Lua_AxisDefine.Horizontal then

        else
            -- 这个计算结果跟self.verticalNormalizedPosition的值差不多
            -- local value = self.subRectH - self._rectHeight
            -- if self.curSubPos > 0 then
            --     verticalNormalizedPosition = (value / 2 - self.curSubPos) / value
            -- else
            --     verticalNormalizedPosition = (value / 2 + Mathf.Abs(self.curSubPos)) / value
            -- end
            
            -- 分页计算
            if self.turnPageCallBack and self.tempSubRectH > 0 and self.subRectH > self.tempSubRectH then
                local value = self.subRectH / self.tempSubRectH
                verticalNormalizedPosition = self.verticalNormalizedPosition * value
                verticalNormalizedPosition = tonumber(string.format("%.1f", verticalNormalizedPosition))
            end
        end
    end
    self.scrollContainer:setNormalizedPosition(horizontalNormalizedPosition, verticalNormalizedPosition);
    self.isFirstEnter = false;
end

function Lua_DragList:updateChildPos()
    self:updateSubSize();

    local itemX;
    local itemY;
    local hIdx;
    local vIdx;
    local posX = -self.subRectW / 2 + self.cellPivotX * self.cellWidth;
    local posY = self.subRectH / 2 - (1 - self.cellPivotY) * self.cellHeight;
    local dataIdx = self.startIdx;
    local data;
    for i = 1, self.cellsCount do
        hIdx = Mathf.Floor((i - 1) / self.hCount);
        vIdx = Mathf.Floor((i - 1) % self.hCount);
        
        itemX = vIdx * self.blockSizeX + posX;
        itemY = -hIdx * self.blockSizeY + posY;

        data = self.showDragDataList[i];
        data:init(itemX, itemY, self.data[dataIdx]);
        data.index = i;
        data.cellWidth = self.cellWidth;
        data.cellHeight = self.cellHeight;
        
        dataIdx = dataIdx + 1;
    end

    self:updateDragItem();
    if not self.delayTween then
        self:playTween();
    end
end

function Lua_DragList:updateSubSize()
    self.subRectW = 0;
    self.subRectH = 0;
    self.hCount = self._dragType == Lua_AxisDefine.Horizontal and Mathf.Ceil(self.cellsCount / self.hvNum) or self.hvNum; -- 一行个数
    self.vCount = Mathf.Ceil(self.cellsCount / self.hCount); -- 一列个数
    if self._dragType == Lua_AxisDefine.Horizontal then
        self.subRectW = self.hCount * self.cellWidth + (self.hCount - 1) * self.cellSpacingX;
        self.subRectW = self.subRectW < self._rectWidth and self._rectWidth or self.subRectW;
        self.subRectH = self._rectHeight
        LuaUtil.setRectSize(self.subRect, self.subRectW, self.subRectH);

        self.subMinPos = -(self.subRectW - self._rectWidth) / 2;
        self.subMaxPox = -self.subMinPos;
        if self.bUpdateSubSize ~= true then
            self.bUpdateSubSize = true;
            self:setSubPosition(self.subMaxPox); -- checkItemInBound 时可以减少item的创建回收
        end
    else
        self.subRectW = self._rectWidth
        self.subRectH = self.vCount * self.cellHeight + (self.vCount - 1) * self.cellSpacingY;
        self.subRectH = self.subRectH < self._rectHeight and self._rectHeight or self.subRectH;
        LuaUtil.setRectSize(self.subRect, self.subRectW, self.subRectH);

        self.subMinPos = -(self.subRectH - self._rectHeight) / 2;
        self.subMaxPox = -self.subMinPos;
        if self.bUpdateSubSize ~= true then
            self.bUpdateSubSize = true;
            self:setSubPosition(self.subMinPos); -- checkItemInBound 时可以减少item的创建回收
        end
    end
end

function Lua_DragList:updateDragItem()
    local subRectX = self.subRect.anchoredPosition.x;
    local subRectY = self.subRect.anchoredPosition.y;
    local posX;
    local posY;
    local centerOffX = (self.cellPivotX - 0.5) * self.cellWidth;
    local centerOffY = (self.cellPivotY - 0.5) * self.cellHeight;
    local data;
    local idx = 0;
    for i = 1, self.cellsCount do
        data = self.showDragDataList[i];
        posX = data.itemX - centerOffX + subRectX;
        posY = data.itemY - centerOffY + subRectY;

        if self:checkItemInBound(posX, posY, self.blockSizeX / 2, self.blockSizeY / 2) then
            data:getCellFromPool(self);
            data.cell:setSiblingIndex(idx);
            idx = idx + 1;
        else
            data:returnCellToPool();
        end
    end
end

function Lua_DragList:moveToIndex(idx, immediately, isBottom)
    if self.cellsCount <= 0 then
        return;
    end

    if not self.delayTween and self.bTweenTimer then
        self.moveIndex = idx;
        self.immediately = immediately;
        self.isBottom = isBottom;
        return;
    end

    immediately = immediately ~= nil and immediately or false;
    isBottom = isBottom ~= nil and isBottom or false;
    idx = idx <= 0 and 1 or idx;
    idx = idx > self.cellsCount and self.cellsCount or idx;

    local toIdx;
    if self._dragType == Lua_AxisDefine.Horizontal then
        local line = Mathf.Ceil(idx / self.hCount);
        if line > self.hvNum then
            toIdx = self.hCount;
        else
            toIdx = idx - (line - 1) * self.hCount;
        end
    else
        toIdx = Mathf.Ceil(idx / self.hvNum);
    end

    local targetPos;
    if self._dragType == Lua_AxisDefine.Horizontal then
        targetPos = self.subMaxPox;
        targetPos = targetPos - (toIdx - 1) * self.cellWidth - (toIdx - 1) * self.cellSpacingX;
        if isBottom then
            targetPos = targetPos - self.cellWidth + self._rectWidth;
        end
    else
        targetPos = self.subMinPos;
        targetPos = targetPos + (toIdx - 1) * self.cellHeight + (toIdx - 1) * self.cellSpacingY;
        if isBottom then
            targetPos = targetPos + self.cellHeight - self._rectHeight;
        end
    end

    if targetPos > self.subMaxPox then
        targetPos = self.subMaxPox;
    elseif targetPos < self.subMinPos then
        targetPos = self.subMinPos;
    end

    if immediately then
        MsgUtil.callFunc("FPS_INTERMISSION_HIGHT");
        self:setSubPosition(targetPos);
        self:updateDragItem();
        if self.delayTween then
            self:playTween();
        end
    else
        self:moveToTargetPos(targetPos);
    end
end

function Lua_DragList:createCellItem()
    local item;
    if self.hideList:getSize() > 0 then
        item = self.hideList:remove(1);
        item:setEnable(true);
    else
        item = self.cellClass.new(self.sub);
    end

    if self.alpha ~= 1 then
        LuaUtil.setObjAlpha(item.content, self.alpha);
    end
    self.showList:add(item);
    return item;
end

function Lua_DragList:returnCell(item)
    if item == nil then
        return;
    end

    local len = self.showList:getSize();
    for i = 1, len do
        if self.showList[i] == item then
            TweenUtil.getInstance():stop(item.content);
            item:setEnable(false);
            self.showList:remove(i);
            self.hideList:add(item);
            if self.alpha ~= 1 then
                LuaUtil.setObjAlpha(item.content, 1);
            end
            break;
        end
    end
end

function Lua_DragList:clearList()
    local len = self.showDragDataList:getSize();
    for i = 1, len do
        self.showDragDataList[i]:clean();
    end
end