Lua_DragListData = class("Lua_DragListData");
function Lua_DragListData:ctor(posX, posY, data, ownerList, pivot)
	self.itemX = posX;
	self.itemY = posY;
	self.data = data;
	self.ownerList = ownerList;
	self.cell = nil;
	self.index = nil;
	self.cellWidth = 0;
	self.cellHeight = 0;
	self.pivot = pivot == nil and 4 or pivot;
end

function Lua_DragListData:init(posX, posY, data)
	self.itemX = posX;
	self.itemY = posY;
	self.data = data;

	if self.cell ~= nil and self.ownerList ~= nil then
		self.cell:setLocation(self.itemX, self.itemY);
		if self.ownerList.customParam ~= nil then
			self.cell:setCustomParam(self.ownerList.customParam);
		end
		self.cell:setData(self.data);

		if self.cell.onSelectionChange ~= nil then
			self.cell:onSelectionChange(self.ownerList.curSelectedIdx == self.index);
		end

		if self.cell.subList ~= nil then
			self.cell.subList:setOwnerList(self.ownerList);
		end
	end
end

function Lua_DragListData:returnCellToPool()
	if self.cell ~= nil then
		self.ownerList:returnCell(self.cell);		
		self.cell = nil;
		self.ownerList = nil;
	end
end

function Lua_DragListData:getCellFromPool(ownerList)
	self.ownerList = ownerList;
	if self.cell == nil then
		self.cell = self.ownerList:createCellItem();
		self.cell:setEnable(true);
		self.cell:setRectPivot(self.pivot);
		self.cell:setLocation(self.itemX, self.itemY);
		self.cell:setScale(1, 1, 1);
		self.cell:setRectSize(self.cellWidth, self.cellHeight);

		if self.ownerList.customParam ~= nil then
			self.cell:setCustomParam(self.ownerList.customParam);
		end
		self.cell:setData(self.data);
		self.cell:setCellSeqIndex(self.index);

		if self.cell.subList ~= nil then
			self.cell.subList:setOwnerList(self.ownerList);
		end
	end

	if self.cell.onSelectionChange ~= nil then
		self.cell:onSelectionChange(self.ownerList.curSelectedIdx == self.index);
	end
end

function Lua_DragListData:clean()
	self:returnCellToPool();
	self.data = nil;
end

function Lua_DragListData:dispose()
	self.cell = nil;
	self.data = nil;
	self.itemX = nil;
	self.itemY = nil;
	self.index = nil;
	self.pivot = nil;
	self.ownerList = nil;
	self.cellWidth = nil;
	self.cellHeight = nil;
end