-- socket管理
Lua_Connection = {};
Lua_Connection.MAIN_SOCKET = 1;
Lua_Connection.CHAT_SOCKET = 2;
Lua_Connection.KUAFU_SOCKET = 3;
Lua_Connection.VALIDATION_SOCKET = 4;
Lua_Connection.LOCALSERVER_SOCKET = 5;
Lua_ConnectionManager = class("Lua_ConnectionManager");
function Lua_ConnectionManager:ctor()
	self:superCall("ctor");
	
	self.bInit = false;
	self.socketTypeList = Array();
	self.socketDic = {};
	self.stopReConnectTime = 0;
	self.canConnectTime = 0;
	self.connectTime = 0;
	self.tempMsg = nil;
	self.reconnectSpitTime = 3; -- 每轮socket重连间隔
	self.reconnectTimes = 6; -- 每轮socket重连次数
	self.bShowReconnectingWin = true;
	self:stopReconnect();
end

function Lua_ConnectionManager:dispose()
	self.stopReConnectTime = 0;
	self.canConnectTime = 0;
	self.connectTime = 0;
	self.tempMsg = nil;
	self.reconnectSpitTime = 3;
	self.reconnectTimes = 0;
	self.bShowReconnectingWin = true;
	self:closeAllSocket();
	self:stopReconnect();
	if self.bInit then
		self.bInit = false;
		Lua_CommonUtil.removeTimer(self, self.onBagTimer);
	end
end

function Lua_ConnectionManager:init()
	if self.bInit then return; end
	self.bInit = true;
	Lua_CommonUtil.addTimer("connectionTimer", 0.01, self, self.onBagTimer);
end

-- 每次重连间隔
function Lua_ConnectionManager.Get:reconnectInterval()
	if self._reconnectInterval == nil then
		self._reconnectInterval = Lua_CommonUtil.getCommonConfig("reconnect_interval", 1, 0) / 1000; 
		if self._reconnectInterval <= 0 then self._reconnectInterval = 1; end
	end
	return self._reconnectInterval;
end

-- 重连提示窗口弹出需重连次数
function Lua_ConnectionManager.Get:reconnectTipswinTimes()
	if self._reconnectTipswinTimes == nil then
		self._reconnectTipswinTimes = Lua_CommonUtil.getCommonConfig("reconnect_tipswin_times", 1, 0);
		if self._reconnectTipswinTimes <= 0 then self._reconnectTipswinTimes = 10; end
	end
	return self._reconnectTipswinTimes;
end

-- 重连提示窗口弹出次数
function Lua_ConnectionManager.Get:reconnectTipswinShowTimes()
	if self._reconnectTipswinShowTimes == nil then
		self._reconnectTipswinShowTimes = Lua_CommonUtil.getCommonConfig("reconnect_tipswin_showTimes", 1, 0);
	end
	return self._reconnectTipswinShowTimes;
end

function Lua_ConnectionManager:onBagTimer()
	if self.checkCompleteConnectTime > 0 then
		if self.checkCompleteConnectTime <= getTime() then -- 超长未获取到玩家信息，认为连接失败
			self:closeAllSocket();
			self.checkCompleteConnectTime = 0;
			if self.bReconnecting then -- 重连
				self.curReconnectTipswinShowTimes = self.curReconnectTipswinShowTimes - 1;
				print("1self.curReconnectTipswinShowTimes = ", self.curReconnectTipswinShowTimes)
				if self.curReconnectTipswinShowTimes <= 0 then -- 结束，回到登录界面
					Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_OPEN, true);
				else -- 继续重连
					Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_OPEN);
				end
			else
				self.tempMsg = LanUtil.getValue(10003);
				Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_OPEN, true);
			end
		end
	elseif self.curReconnectInterval > 0 then
		local curTime = getTime();
		local dt = curTime - self.curReconnectTime;
		self.curReconnectTime = curTime;
		self.curReconnectInterval = self.curReconnectInterval - dt;
		if self.curReconnectInterval <= 0 then
			self:closeAllSocket();
			self.curReconnectTipswinTimes = self.curReconnectTipswinTimes - 1;
			if self.curReconnectTipswinTimes <= 0 then -- 弹出重连提示窗口
				self.curReconnectTipswinShowTimes = self.curReconnectTipswinShowTimes - 1;
				print("2self.curReconnectTipswinShowTimes = ", self.curReconnectTipswinShowTimes)
				if self.curReconnectTipswinShowTimes <= 0 then -- 结束，回到登录界面
					Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_OPEN, true);
				else -- 继续重连
					Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_OPEN);
				end
			else
				self.curReconnectInterval = self.reconnectInterval;
				self:connectionSocket(Lua_Connection.VALIDATION_SOCKET, ServerConfig.gatewayIP, ServerConfig.gatewayPort, true, true);
			end
		end
	end

	
	local isConnecting = true;
	local len = self.socketTypeList:getSize();
	while len > 0 do 
		local st = self.socketTypeList[len];
		if self.socketDic[st].isConnecting then
			self.socketDic[st]:onBagTimer();
		else
			isConnecting = false;
			self.socketTypeList:remove(len);
		end
		len = len - 1;
	end
end

function Lua_ConnectionManager:getConnection(id)
	local con = self.socketDic[id];
	if con ~= nil and self.socketTypeList:indexOf(id) ~= -1 then
		return con;
	end
	return nil;
end

function Lua_ConnectionManager:closeConnenction(id)
	if self.socketTypeList:removeItem(id) then
		local con = self.socketDic[id];
		if con ~= nil then
			con:close();
			return true;
		end
	end
	return false;
end

function Lua_ConnectionManager:isConnection(id)
	local con = self.socketDic[id];
	if con ~= nil then
		return con:isConnect();
	end
	return false;
end

function Lua_ConnectionManager:setConnectionHandleMaxNum(id, num)
	local con = self:getConnection(id);
	if con == nil then return; end
	if num == nil or num <= 0 then num = 30; end
	con.bagMax = num;
end

function Lua_ConnectionManager:connectionSocket(id, server, port, bShowTips, isReConnect)
	if id == Lua_Connection.VALIDATION_SOCKET then
		DataCenter.gameReported:loginReportedData(Lua_GameReported.REPORT_LINK_VALIDATION_SOCKET);
	elseif id == Lua_Connection.MAIN_SOCKET then
		DataCenter.gameReported:loginReportedData(Lua_GameReported.REPORT_LINK_MAIN_SOCKET);
	end
	
	if isReConnect then self:closeConnenction(id); end
	if port == nil or strIsNullOrEmpty(server) then return; end
	if bShowTips == nil then bShowTips = true; end
	local con = self.socketDic[id];
	if con == nil then
		con = Connection(id);
		self.socketDic[id] = con;
		self.socketTypeList:add(id);
	else
		if self.socketTypeList:indexOf(id) == -1 then
			self.socketTypeList:add(id);
		end
	end
	con:connect(server, port, bShowTips);
	self:init();
end

function Lua_ConnectionManager:errorAlert(msg, needReconnect, showReconnect)
    LogManager.showLog(
        "断线重连 msg: "..tostring(msg)..", 重连中: "..tostring(self.bReconnecting)..", needReconnect: "..tostring(needReconnect)..", showReconnect: "..tostring(showReconnect).."\n"..getTraceback()
        );
	if self.bReconnecting then -- 重连中
        return;
    end

    if GlobalVar.canReconnect and needReconnect then
        if DataCenter.bConfigLoaded then
        	self:setCollectInfo(200);
        	clearLua(true);
        end

        self.bInit = false;
        self.tempMsg = msg;
        self:init(); -- clearLua 会把计时器清除
        self.curReconnectTipswinShowTimes = self.reconnectTipswinShowTimes;
        print("3self.curReconnectTipswinShowTimes = ", self.curReconnectTipswinShowTimes)
        if showReconnect == nil then showReconnect = true; end
        self.bShowReconnectingWin = showReconnect;
        self:restartReconnect();
    else
    	self.tempMsg = msg;
        self:closeAllSocket();
        self:stopReconnect();
        ContainerUtil.clearAll();
        PlayerData.instance:dispose();
        TweenUtil.getInstance():dispose();
        Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_OPEN, true); -- 结束返回登录界面
    end
end

-- 开始重连，一直到服务器重新下发玩家信息才判断完成，先连接验证服，再连接主socket
function Lua_ConnectionManager:restartReconnect()
    print("断线重连 restartReconnect");
	self:closeAllSocket();
    self.bReconnecting = true;
    self.curReconnectTime = getTime();
    self.curReconnectInterval = self.reconnectInterval;
    self.curReconnectTipswinTimes = self.reconnectTipswinTimes;
    if self.bShowReconnectingWin then Lua_WindowUtil.openWin("Lua_ReconnectingWindow", Lua_WinParam.ACTION_OPEN); end
    if DataCenter.loginData.bLocalServer then -- 本地服务器
    	self:connectionSocket(Lua_Connection.MAIN_SOCKET, ServerConfig.single_server, ServerConfig.single_port, true, true);
    else
		self:connectionSocket(Lua_Connection.VALIDATION_SOCKET, ServerConfig.gatewayIP, ServerConfig.gatewayPort, true, true);
    end
end

-- 暂停重连
function Lua_ConnectionManager:stopReconnect()
	self.bReconnecting = false;
	self.curReconnectTime = 0;
	self.curReconnectInterval = 0;
	self.curReconnectTipswinTimes = 0;
	self.curReconnectTipswinShowTimes = 0;
	self.checkCompleteConnectTime = 0;
	Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_CLOSE);
    Lua_WindowUtil.openWin("Lua_ReconnectingWindow", Lua_WinParam.ACTION_CLOSE);
end

function Lua_ConnectionManager:setCollectInfo(setpauseval)
	DataCenter.gcSetPauseVal = setpauseval;
    LanUtil.systemGcCollect();
	collectgarbage("collect");
    collectgarbage("setpause", setpauseval);
    collectgarbage("setstepmul", 5000);
    LoadManager.getInstance():clearResource();
end

-- 退出
function Lua_ConnectionManager:loginOut()
    self:setCollectInfo(200);
	clearLua(); -- core
	self:stopReconnect();
	self:closeAllSocket();
	ContainerUtil.clearAll();
    PlayerData.instance:dispose();
    TweenUtil.getInstance():dispose();
    PlayerPrefsUtil.instance:dispose();
    DataCenter.audioManager:removeAllAudio();
    GameFindObjMananger.instance:dispose();
    WindowMananger.getInstance():removeAllWindow();
    CommonUtil.clearMemory();
	GlobalVar.bHandleServerMsg = true;
    GlobalVar.bConfigLoaded = false;
    DataCenter.bConfigLoaded = false;
    DataCenter.audioManager:play(Lua_AudioDefine.LOGIN_WIN_BG);
    Lua_WindowUtil.openWin("LoginBgWindow", Lua_WinParam.ACTION_OPEN);
    Lua_WindowUtil.openWin("Lua_StartWindow", Lua_WinParam.ACTION_OPEN);
end

-- 移除所有socket
function Lua_ConnectionManager:closeAllSocket()
	local len = self.socketTypeList:getSize();
	while (len > 0) do
		local st = self.socketTypeList[len];
		self.socketDic[st]:close();
		self.socketTypeList:remove(len);
		len = len - 1;
	end
end

-- 退出，如果是sdk的，先退出sdk，sdk退出成功后返回登录界面
function Lua_ConnectionManager:relogin()
	if not Lua_PlatformUtil.IsNoSdk() then
        DataCenter.sdkInterface:callSDKInterface(Lua_SDKMessageType.LoginOut); -- sdk退出
        return;
   	end
    self:loginOut();
end

-- 同步连接socket状态
function Lua_ConnectionManager:syncSocketConnectState(socketType, state)
	print("syncSocketConnectState", socketType, state);
	if state then -- 连接成功
		if DataCenter.loginData.bLocalServer then -- 本地服务器
			if socketType == Lua_Connection.MAIN_SOCKET then
				self.curReconnectInterval = 0;
				self.checkCompleteConnectTime = getTime() + self.reconnectInterval * self.reconnectTipswinTimes;
				Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_CLOSE);
				if self.bReconnecting and self.bShowReconnectingWin then 
					Lua_WindowUtil.openWin("Lua_ReconnectingWindow", Lua_WinParam.ACTION_OPEN);
				else
					 Lua_WindowUtil.openWin("Lua_ReconnectingWindow", Lua_WinParam.ACTION_CLOSE);
				end
			end
	    else
	    	if socketType == Lua_Connection.VALIDATION_SOCKET then
	    		self.curReconnectInterval = 0;
	    		self.checkCompleteConnectTime = getTime() + self.reconnectInterval * self.reconnectTipswinTimes;
	    		print("self.checkCompleteConnectTime = ", self.checkCompleteConnectTime, getTime());
	    		Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_CLOSE);
	    		if self.bReconnecting and self.bShowReconnectingWin then 
					Lua_WindowUtil.openWin("Lua_ReconnectingWindow", Lua_WinParam.ACTION_OPEN); 
				else
					Lua_WindowUtil.openWin("Lua_ReconnectingWindow", Lua_WinParam.ACTION_CLOSE);
				end
			end
	    end
	else -- 连接失败
		if self.bReconnecting then -- 重连中
			if bc == Lua_Connection.VALIDATION_SOCKET then -- 验证服socket连接失败
				-- 重连机制会每个一段时间重新连接
				-- self:connectionSocket(Lua_Connection.VALIDATION_SOCKET, ServerConfig.gatewayIP, ServerConfig.gatewayPort, true, true);
			elseif bc == Lua_Connection.MAIN_SOCKET then -- 主socket连接失败
				self:connectionSocket(Lua_Connection.MAIN_SOCKET, ServerConfig.single_server, ServerConfig.single_port, true, true);
			end
		else
			if bc == Lua_Connection.VALIDATION_SOCKET then -- 验证服socket连接失败
				self.tempMsg = LanUtil.getValue(10003)
		        Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_OPEN, true); -- 结束返回登录界面
			elseif bc == Lua_Connection.MAIN_SOCKET then -- 主socket连接失败
				self.tempMsg = LanUtil.getValue(10003)
		        Lua_WindowUtil.openWin("Lua_ReconnectWindow", Lua_WinParam.ACTION_OPEN, true); -- 结束返回登录界面
		    elseif bc == Lua_Connection.LOCALSERVER_SOCKET then
		        OperateAlert.show("获取本地服务器列表失败");
		    end
		end
	end
end

-- 消息上行
function Lua_ConnectionManager:sendSocket(msgid, msg, socketType)
	if socketType == nil then socketType = Lua_Connection.MAIN_SOCKET; end
    local con = self:getConnection(socketType);
    if con == nil or not con.isConnecting then -- 未链接则判断是否需要重连
    	if DataCenter.loginData.isEnterGameWorld then -- 游戏内重连
    		GlobalVar.canReconnect = true;
	        self:errorAlert(LanUtil.getValue(10003), true);
	    else -- 还未进入游戏，提示
	        GlobalVar.canReconnect = false;
	        self:errorAlert(LanUtil.getValue(10003));
	        OperateAlert.showLangValue(10003);
	    end
    	return; 
    end

	if msg ~= nil then
        local pb_data = msg:SerializeToString();
        con:sendBytes(msgid, pb_data);
    else
    	con:sendMsg(msgid);
    end
end