-- 引导同步数据集合，用于功能与引导数据逻辑交集
Lua_GuideCommonData = class("Lua_GuideCommonData");
function Lua_GuideCommonData:ctor()
	self:superCall("ctor");

	self.curGuideStepVo = nil;
	self._bAllowPopupWins = false

	self.curStep = 0;
	self.curGuideId = 0;
	self.curGuideWinName = nil;
	self.curGuideWinData = nil;
	self._guideClickObjPath = nil;
	self._guideClickObjPosX = nil;
	self._guideClickObjPosY = nil;
	self._guideClickObjWidth = nil;
	self._guideClickObjHeight = nil;
	self._guideClickObjMaskParams = nil;
	self._guideClickObjTarget = nil;
	self.bOpenGuideAllowPopupWin = false;
	self.antiRepeatClick = nil; -- 英雄上阵后防点击过快下阵
	self.bStopGuide = nil;
	self.hideGuideEffect = nil;
end

function Lua_GuideCommonData:dispose()
	self.curGuideStepVo = nil;
	self._bAllowPopupWins = false

	self.curStep = 0;
	self.curGuideId = 0;
	self._winGuideResult = nil;
	self.curGuideWinName = nil;
	self.curGuideWinData = nil;
	self._guideClickObjPath = nil;
	self._guideClickObjPosX = nil;
	self._guideClickObjPosY = nil;
	self._guideClickObjWidth = nil;
	self._guideClickObjHeight = nil;
	self._guideClickObjMaskParams = nil;
	self._guideClickObjTarget = nil;
	self.bOpenGuideAllowPopupWin = false;
	self.antiRepeatClick = nil;
	self.hideGuideEffect = nil;
	self.bStopGuide = nil;
	if self.allowPopipWinsDic ~= nil then
		table.clear(self.allowPopipWinsDic);
	end
end

function Lua_GuideCommonData:syncGuideStep(stepVo, checkPopup)
	self.curGuideStepVo = stepVo; -- 当前引导步骤的vo
	self.curStep = stepVo ~= nil and stepVo.step or 0;
	self.curGuideId = stepVo ~= nil and stepVo.guideId or 0;
	local popupWins = (stepVo ~= nil and not stepVo.isEmpty) and stepVo.config.popupWins or nil;
	self:syncPopupWins(popupWins); -- 同步步骤允许弹框
	self._bAllowPopupWins = false;-- 引导步骤开始的时候同步过来，设置 self.bAllowPopupWins 开关
	local unCheckPopupWins = (stepVo ~= nil and not stepVo.isEmpty) and stepVo.config.unCheckPopupWins or 0;
	-- 配置该步骤不检测非引导相关的弹框，则为false
	if unCheckPopupWins ~= 1 and self.curGuideStepVo ~= nil and self.curGuideStepVo.canPopupWin then 
		if self._guidingAllowPopupWinsArr:getSize() > 0 then
			self._bAllowPopupWins = true;
		elseif self._guideStepPopupWinsArr ~= nil and self._guideStepPopupWinsArr:getSize() > 0 then
			self._bAllowPopupWins = true;
		end
	end

	-- 检测当前已经弹出的弹框界面
	if checkPopup then
		local dic = DataCenter.popupWinManager.popupWinNameDic;
		if dic ~= nil then
			-- print("检测当前已经弹出的弹框界面")
			for k, vo in pairs(dic) do
				-- print("1检测当前已经弹出的弹框界面", vo.winName);
				if Lua_WindowUtil.isOpenWin(vo.winName) then
					-- print("2检测当前已经弹出的弹框界面", vo.winName);
					self:syncPopupWinState(vo.winName, 1, true);
				end
			end
		end
	end
end

-- 引导过程中允许的弹框列表
function Lua_GuideCommonData.Get:guidingAllowPopupWinsArr()
	self:initGuidingAllowPopupWis();
	return self._guidingAllowPopupWinsArr;
end

-- 引导过程中允许的弹框集合
function Lua_GuideCommonData.Get:guidingAllowPopupWinsDic()
	self:initGuidingAllowPopupWis();
	return self._guidingAllowPopupWinsDic;
end

-- 初始化引导过程中允许的弹框
function Lua_GuideCommonData:initGuidingAllowPopupWis()
	if self._guidingAllowPopupWinsArr == nil then
		self._guidingAllowPopupWinsDic = {};
		self._guidingAllowPopupWinsArr = Array();
		local list = getConfig("guide_const", "guiding_allow_popupWins", "arr_value");
		if list ~= nil and #list > 0 then
			for i = 1, #list do
				local winName = list[i];
				self._guidingAllowPopupWinsDic[winName] = true;
				self._guidingAllowPopupWinsArr:add(winName);
			end
		end
	end
end

-- 引导步骤执行后设置 self._bAllowPopupWins 为false
function Lua_GuideCommonData:resetAllowPopupWinsState()
	self._bAllowPopupWins = false;
end

-- 同步可以弹出的弹框窗口
function Lua_GuideCommonData:syncPopupWins(list)
	self:initGuidingAllowPopupWis();
	if list ~= nil and #list > 0 then
		for i = 1, #list do
			local winName = list[i];
			-- print("syncPopupWins", winName);
			if not strIsNullOrEmpty(winName) and self._guidingAllowPopupWinsDic[winName] == nil then
				if self._guideStepPopupWinsArr == nil then
					self._guideStepPopupWinsArr = Array()
				end
				self._guideStepPopupWinsArr:add(winName);
				self._guidingAllowPopupWinsDic[winName] = true;
			end
		end
	else
		if self._guideStepPopupWinsArr ~= nil then
			local len = self._guideStepPopupWinsArr:getSize();
			while (len > 0) do
				local winName = self._guideStepPopupWinsArr[len];
				self._guideStepPopupWinsArr:remove(len);
				self._guidingAllowPopupWinsDic[winName] = nil;
				len = len - 1;
			end
		end
	end
end

-- 检测是否是引导用到的弹框
function Lua_GuideCommonData:checkGuideUsePopupWin(winName)
	if self.curGuideStepVo == nil then
		return false;
	end

	if self.curGuideStepVo.isEmpty then
		return false;
	end

	if strIsNullOrEmpty(winName) then
		return false;
	end

	-- 挖空区域判断面板
	local config = self.curGuideStepVo.config;
	if config.clickObjWin ~= nil and config.clickObjWin == winName then
		return true;
	end

	-- 步骤允许弹框
	if self._guidingAllowPopupWinsDic ~= nil and self._guidingAllowPopupWinsDic[winName] ~= nil then
		if self._bAllowPopupWins then
			return true;
		end

		if self._guideStepPopupWinsArr ~= nil and self._guideStepPopupWinsArr:indexOf(winName) > 0 then
			return true;
		end
	end

	-- 条件的弹框
	if self.curGuideStepVo:isConditionByWinName(winName) then
		return true;
	end

	return false;
end

-- 同步弹框界面打开, state 1 显示， 2 关闭或隐藏  force 强制加入，用于引导开始判断
function Lua_GuideCommonData:syncPopupWinState(winName, state, force)
	-- print("syncPopupWinState", winName, state, DataCenter.guideData.isGuiding);
	if not DataCenter.guideData.isGuiding then
		self.bOpenGuideAllowPopupWin = false;
		return;
	end

	if state == 1 then -- 显示
		if not force then
			if self._guidingAllowPopupWinsDic == nil then
				self.bOpenGuideAllowPopupWin = false;
				return;
			end

			if self._guidingAllowPopupWinsDic[winName] == nil then
				return;
			end
		end

		self.bOpenGuideAllowPopupWin = true;
		if self.guidePopupWinsArr == nil then
			self.guidePopupWinsArr = Array();
			self.guidePopupWinsArr:add(winName);
		else
			if self.guidePopupWinsArr:indexOf(winName) == -1 then
				self.guidePopupWinsArr:add(winName);
			end
		end
	elseif state == 2 then -- 关闭或隐藏
		if self.guidePopupWinsArr ~= nil and self.guidePopupWinsArr:getSize() > 0 then
			if self.guidePopupWinsArr:removeItem(winName) then
				self.bOpenGuideAllowPopupWin = self.guidePopupWinsArr:getSize() > 0;
			end
		else
			self.bOpenGuideAllowPopupWin = false;
		end
	end
	-- print("syncPopupWinState result", winName, state, self.bOpenGuideAllowPopupWin);
end

-- 检测弹框中是否有引导用到的弹框界面
function Lua_GuideCommonData:checkPopupWinGuiding()
	if not self.bOpenGuideAllowPopupWin then
		-- print("1checkPopupWinGuiding", self.bOpenGuideAllowPopupWin)
		return true;
	end 

	if self.guidePopupWinsArr == nil then -- 正常逻辑不会进
		-- print("2checkPopupWinGuiding self.guidePopupWinsArr == nil")
		return true;
	end

	local len = self.guidePopupWinsArr:getSize();
	if len <= 0 then -- 正常逻辑不会进
		-- print("3checkPopupWinGuiding len <= 0")
		return true;
	end

	if self.curGuideStepVo == nil or self.curGuideStepVo.isEmpty then
		-- print("4checkPopupWinGuiding")
		return true;
	end

	local syncGuideWinName = nil;
	local config = self.curGuideStepVo.config;
	if config.syncWinInfo ~= nil then
		syncGuideWinName = self.inheritWindowName or config.syncWinInfo.winName;
	end
	local clickObjWin = self.curGuideStepVo.config.clickObjWin;
	if not strIsNullOrEmpty(clickObjWin) or not strIsNullOrEmpty(syncGuideWinName) then
		for i = 1, len do
			local str =  self.guidePopupWinsArr[i];
			if str == clickObjWin then
				-- print("5checkPopupWinGuiding")
				return true;
			elseif str == syncGuideWinName then
				-- print("6checkPopupWinGuiding")
				return true;
			end
		end
	end

	-- print("7checkPopupWinGuiding")
	return false;
end

-- 同步界面引导数据
function Lua_GuideCommonData:syncWinGuideData(winName, info)
	self.curGuideWinName = winName;
	self.curGuideWinData = info;

	-- 重置
	self._guideSceneId = 0;
	self._guideClickObjTarget = nil;
	self._guideClickObjPath = nil;
	self._guideClickObjPosX = 0;
	self._guideClickObjPosY = 0;
	self._guideClickObjWidth = 0;
	self._guideClickObjHeight = 0;
	self._guideClickObjMaskParams = nil;
	self._winGuideResult = nil;
	self.antiRepeatClick = nil;
	self.hideGuideEffect = nil;
end

-- 继承后的窗口名
function Lua_GuideCommonData:setInheritWindowName(winName)
	self.inheritWindowName = winName;
end

-- 获取界面引导数据
function Lua_GuideCommonData:getWinGuideData(winName)
	if strIsNullOrEmpty(winName) then
		return 0, nil;
	end

	if self.curGuideWinName == winName then
		return self.curGuideId, self.curGuideWinData;
	end
	return 0, nil;
end

-- 界面同步引导结果, 1 完成； -1 失败
function Lua_GuideCommonData.Get:winGuideResult()
	return self._winGuideResult;
end

-- 界面同步引导结果, 1 完成； -1 失败
function Lua_GuideCommonData.Set:winGuideResult(val)
	self._winGuideResult = val;
end

-- 查找对象所在场景
function Lua_GuideCommonData.Set:guideSceneId(val)
	self._guideSceneId = val;
end

-- 查找对象所在场景
function Lua_GuideCommonData.Get:guideSceneId()
	return self._guideSceneId;
end

-- 挖空对象目标
function Lua_GuideCommonData.Set:guideClickObjTarget(val)
	self._guideClickObjTarget = val;
end

-- 挖空对象目标
function Lua_GuideCommonData.Get:guideClickObjTarget()
	return self._guideClickObjTarget;
end

-- 挖空对象获取路径
function Lua_GuideCommonData.Set:guideClickObjPath(val)
	self._guideClickObjPath = val;
end

-- 挖空对象获取路径
function Lua_GuideCommonData.Get:guideClickObjPath()
	return self._guideClickObjPath;
end

-- 挖空位置x，未转换坐标
function Lua_GuideCommonData.Set:guideClickObjPosX(val)
	self._guideClickObjPosX = val;
end

-- 挖空位置x，未转换坐标
function Lua_GuideCommonData.Get:guideClickObjPosX()
	return self._guideClickObjPosX;
end

-- 挖空位置y，未转换坐标
function Lua_GuideCommonData.Set:guideClickObjPosY(val)
	self._guideClickObjPosY = val;
end

-- 挖空位置y，未转换坐标
function Lua_GuideCommonData.Get:guideClickObjPosY()
	return self._guideClickObjPosY;
end

function Lua_GuideCommonData.Set:guideClickObjMaskParams(val)
	self._guideClickObjMaskParams = val;
end

function Lua_GuideCommonData.Get:guideClickObjMaskParams()
	return self._guideClickObjMaskParams;
end

-- 挖空width
function Lua_GuideCommonData.Set:guideClickObjWidth(val)
	self._guideClickObjWidth = val;
end

-- 挖空width
function Lua_GuideCommonData.Get:guideClickObjWidth()
	return self._guideClickObjWidth;
end

-- 挖空height
function Lua_GuideCommonData.Set:guideClickObjHeight(val)
	self._guideClickObjHeight = val;
end

-- 挖空height
function Lua_GuideCommonData.Get:guideClickObjHeight()
	return self._guideClickObjHeight;
end

-- 是否处于暂停引导，进行弹窗的状态（防止点击弹窗中的某些功能按钮打开或跳转）
function Lua_GuideCommonData.Set:bStopGuide(val)
	self._bStopGuide = val;
end

-- 是否处于暂停引导，进行弹窗的状态（防止点击弹窗中的某些功能按钮打开或跳转）
function Lua_GuideCommonData.Get:bStopGuide()
	return self._bStopGuide;
end

-- 引导过程中需要预加载的窗口
function Lua_GuideCommonData:initGuidingPreLoadWins()
	if self._guidingPreLoadWinsDir == nil then
		self._guidingPreLoadWinsDir = {};
		local list = getConfig("guide_const", "preLoad_WinNames", "arr_value");
		if list ~= nil and #list > 0 then
			for i = 1, #list do
				local winName = list[i];
				self._guidingPreLoadWinsDir[winName] = true;
			end
		end
	end
end

function Lua_GuideCommonData:isLoadingWin(windowName)
	if self._guidingPreLoadWinsDir == nil then
		self:initGuidingPreLoadWins();
	end
	return self._guidingPreLoadWinsDir[windowName];
end
