-- 引导触发条件
Lua_GuideConditionVo = class("Lua_GuideConditionVo");
function Lua_GuideConditionVo:ctor()
	self:superCall("ctor");
	self.conditionType = 0;
	self.guideId = 0;
end

function Lua_GuideConditionVo:setInfo(list, gid)
	if list == nil or #list <= 0 then
		self.conditionType = 0;
		self.guideId = 0;
		return;
	end

	self.guideId = gid;
	self.conditionType = list[1];
	self.param1 = list[2];
	self.param2 = list[3];
	self.param3 = list[4];
end

function Lua_GuideConditionVo.Get:isComplete()
	if self.conditionType <= 0 then
		return false;
	end

	-- print("Lua_GuideConditionVo isComplete", self.conditionType, DataCenter.loginData.isCreateRole);
	if self.conditionType == Lua_GuideDefine.Trigger_Condition_CreateRole then -- 创角
		if self.param1 ~= nil and self.param1 == 2 then -- 判断创角成功
			return DataCenter.loginData.isCreateRole;
		else 
			return not DataCenter.loginData.isCreateRole;
		end
	elseif not DataCenter.loginData.isCreateRole and 
		(self.conditionType ~= Lua_GuideDefine.Trigger_Condition_OpenWin and self.param1 ~= "Lua_CreateRoleWindow") then -- 未创角都未触发(创角窗口例外)
		return false;
	end

	if self.conditionType == Lua_GuideDefine.Trigger_Condition_MainLineTask then -- 主线冒险任务
		return self:checkMainLineTask();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_Level then -- 等级
		return self:checkLevel();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_FunOpen then -- 功能开启
		return self:checkFunOpen();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_MainLine then -- 主线推图
		return self:checkMainLine();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_OpenWin then -- 界面打开
		return self:checkWinState();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_SceneShow then -- 场景显示
		return self:checkSceneShow();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_Item then -- 获取道具
		return self:checkItem();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_Hero then -- 获取英雄
		return self:checkHero();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_LineUp then -- 布阵
		return self:checkLineUp();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_MainLineStage then -- 主线冒险阶段奖励
		return self:checkMainLineTaskStage();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_Artifact then -- 神器 
		return self:checkArtifact();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_CollectionItem then -- 藏品柜
		return self:checkCollectionItem();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_FightState then -- 战斗状态
		return self:checkFightState();
	elseif self.conditionType == Lua_GuideDefine.Trigger_Condition_FightTypeResultWin then -- 战斗类型弹窗
		return self:checkFightTypeResultWin()
	end
	return false;
end

-- 主线冒险任务
function Lua_GuideConditionVo:checkMainLineTask()
	local taskId = self.param1;
	if taskId == nil then
		return false;
	end

	taskId = tonumber(taskId);
	if taskId <= 0 then
		return false;
	end

	local state = self.param2 ~= nil and tonumber(self.param2) or 2;
	return DataCenter.taskData:getExploreTaskState(taskId) == state;
end

-- 等级
function Lua_GuideConditionVo:checkLevel()
	local lv = self.param1;
	if lv == nil then
		return false;
	end

	lv = tonumber(lv);
	-- print("Lua_GuideConditionVo checkLevel", lv, DataCenter.roleData.level);
	return lv <= DataCenter.roleData.level;
end

-- 功能开启
function Lua_GuideConditionVo:checkFunOpen()
	local funId = self.param1;
	if funId == nil then
		return false;
	end
	
	-- print( funId, DataCenter.funcOpenData:isOpened(funId, false));
	funId = tonumber(funId);
	local funcState = self.param2 or 1;
	return DataCenter.funcOpenData:isOpened(funId, false, funcState);
end

-- 主线推图
function Lua_GuideConditionVo:checkMainLine()
	local pointId = self.param1;
	local bShowResultWin = self.param2 or 0;
	if pointId == nil then
		return false;
	end

	local result = DataCenter.mainLineData:getCheckpointIsCompleted(pointId);
	--print("checkMainLine", self.guideId, pointId, result, DataCenter.fightData:getRecordFightResult(Lua_FightDefine.FIGHT_TYPE_MAINLINE, pointId) == true)
	if result and bShowResultWin == 0 then
		result = DataCenter.fightData:getRecordFightResult(Lua_FightDefine.FIGHT_TYPE_MAINLINE, pointId) == true;
	end
	return result;
end

-- 界面打开
function Lua_GuideConditionVo:checkWinState()
	local winName = self.param1;
	-- print("Lua_WindowUtil.isActiveWin(winName)", winName, Lua_WindowUtil.isOpenWin(winName), Lua_WindowUtil.isWindowShowing(winName), Lua_WindowUtil.isActiveWin(winName));
	if strIsNullOrEmpty(winName) then
		return false;
	end

	if self.param2 ~= nil then
		if self.param2 == 1 then -- open
			--return Lua_WindowUtil.isOpenWin(winName);
			return Lua_WindowUtil.isWinInitComplete(winName);
		elseif self.param2 == 2 then -- hide
			return not Lua_WindowUtil.isOpenWin(winName);
		elseif self.param2 == 3 then -- show
			return Lua_WindowUtil.isWindowShowing(winName);
		elseif self.param2 == 4 then -- show and noActive
			return Lua_WindowUtil.isNoActiveWin(winName)
		end
	end
	return Lua_WindowUtil.isActiveWin(winName);
end

-- 场景显示
function Lua_GuideConditionVo:checkSceneShow()
	local sceneId = self.param1;
	if sceneId == nil then
		return false;
	end

	sceneId = tonumber(sceneId);
	if sceneId <= 0 then
		return false;
	end
	if self.param2 ~= nil then
		local fid = DataCenter.fightData.curPlayFightRecordId;
		if self.param2 == 1 then
			if fid == nil or fid <= 0 then
				return false;
			end
		end
		-- print("checkSceneShow fight", sceneId, fid, Scene3DManager.instance:isShowingScene(sceneId, fid))
		return Scene3DManager.instance:isShowingScene(sceneId, fid);
	end
	-- print("checkSceneShow", sceneId, Scene3DManager.instance:isShowingScene(sceneId, 0))
	return Scene3DManager.instance:isShowingScene(sceneId, 0);
end

-- 道具
function Lua_GuideConditionVo:checkItem()
	local itemId = self.param1;
	if itemId == nil then
		return false;
	end

	itemId = tonumber(itemId);
	if itemId < 0 then
		return false;
	elseif itemId == 0 then -- 英雄直升资源
		return self:checkSatisfyUpHeroGradeRes();
	end

	local num = self.param2 ~= nil and tonumber(self.param2) or 1;
	return DataCenter.packData:getItemCountById(itemId) >= num;
end

-- 英雄
function Lua_GuideConditionVo:checkHero()
	local heroId = self.param1;
	if heroId == nil then
		return false;
	end

	heroId = tonumber(heroId);
	if heroId <= 0 then
		return false;
	end

	local arr = DataCenter.heroData:getHeroUidArrById(heroId);
	local num = self.param2 ~= nil and tonumber(self.param2) or 1;
	local level = self.param3 ~= nil and tonumber(self.param3) or 1;
	local count = arr ~= nil and arr:getSize() or 0;
	if count >= num then 
		if level > 1 then
			for i = 1, count do
				local heroVo = DataCenter.heroData:getItemByUniqueId(arr[i]);
				if heroVo.level >= level then 
					return true;
				end
			end
		else
			return true;
		end
	end
	return false;
end

-- 布阵
function Lua_GuideConditionVo:checkLineUp()
	local lineupType = self.param1;
	if lineupType == nil then
		return false;
	end

	local val = self.param2 ~= nil and tonumber(self.param2) or 1;
	local cfg = Lua_HeroConfigData.getConfig(val);
	if cfg ~= nil then -- 英雄
		local lineupIdx, standIdx = DataCenter.heroLineupData:getLineupHeroInfo(lineupType, val);
		-- print("Lua_GuideConditionVo checkLineUp", val, lineupIdx, standIdx, self.param3)
		if self.param3 ~= nil then -- 站位要求
			return lineupIdx ~= nil and standIdx ~= nil and lineupIdx > 0 and standIdx > 0 and standIdx == tonumber(self.param3);
		end
		return lineupIdx ~= nil and lineupIdx > 0;
	else -- 上阵数量
		return DataCenter.heroLineupData:getLineupHeroNum(lineupType) >= val;
	end
end

-- 主线冒险任务阶段奖励
function Lua_GuideConditionVo:checkMainLineTaskStage()
	local stage = self.param1;
	if stage == nil then
		return false;
	end

	stage = tonumber(stage);
	if stage <= 0 then
		return false;
	end
	
	local state = self.param2 ~= nil and tonumber(self.param2) or 3;
	return DataCenter.taskData:getExploreStageState(stage) == state;
end

-- 神器
function Lua_GuideConditionVo:checkArtifact()
	local lineupType = self.param1;
	if lineupType == nil then
		return false;
	end

	local teamIdx = self.param2 ~= nil and tonumber(self.param2) or 1;
	local artifactPos = self.param2 ~= nil and tonumber(self.param2) or 1;
	local artifactId = DataCenter.heroLineupData:getLineupArtifact(lineupType, teamIdx, artifactPos);
	return artifactId ~= 0;
end

-- 藏品柜
function Lua_GuideConditionVo:checkCollectionItem()
	local collectionId = self.param1;
	local activeState = self.param2;
	
	local bSatisfy;
	if collectionId == 0 then
		bSatisfy = DataCenter.collectionData:getCollectionId(1, activeState) ~= nil;
	else
		bSatisfy = DataCenter.collectionData:getCollectionVoStateById(collectionId) == activeState;
	end

	return bSatisfy;
end

-- 战斗状态
function Lua_GuideConditionVo:checkFightState()
	local templateId = self.param1;
	local state = self.param2;
	if state == nil then
		state = 1;
	end

	local rid = DataCenter.fightData:getRecordIdByTemplateId(templateId);
	-- print("checkFightState", templateId, rid, state);
	if rid > 0 then return state == 1; end
	return state == 2;
end

-- 是否英雄升级资源
function Lua_GuideConditionVo:checkSatisfyUpHeroGradeRes()
	local bHeroRes = self.param1 == 0;
	if not bHeroRes then
		return;
	end

	local heroId = self.param2;
	local maxLvl = 0;
    local arr = DataCenter.heroData:getHeroUidArrById(heroId);
    local unique_id;

    if arr ~= nil then
	    for i = 1, arr:getSize() do
	        local heroVo = DataCenter.heroData:getItemByUniqueId(arr[i]);
	        if heroVo ~= nil and heroVo.level > maxLvl and heroVo.is_league_transaction == false and not heroVo.trial_state then
	            unique_id = arr[i];
	            maxLvl = heroVo.level;
	        end
	    end

	    if unique_id ~= nil then
		    local upgradeNum = self.param3 or 1;
		    -- print("Lua_GuideConditionVo:checkSatisfyUpHeroGradeRes", DataCenter.heroData:bHeroUpLevelByUidAndLevel(unique_id, upgradeNum))
		    return DataCenter.heroData:bHeroUpLevelByUidAndLevel(unique_id, upgradeNum);
		end
	end
	
	return false;
end

-- 战斗结算
function Lua_GuideConditionVo:checkFightTypeResultWin()
	local type = self.param1;
	local pointId = self.param2;
	local bShowResultWin = self.param3 or 0;
	if type == nil or pointId == nil then
		return false;
	end

	local result;
	--print("Lua_GuideConditionVo:checkFightTypeResultWin", type, pointId, bShowResultWin == 0, "!!!",DataCenter.newHeroActData:getStoryLevelResult(pointId, bShowResultWin == 0), "@@@@")
	if type == Lua_FightDefine.FIGHT_TYPE_STORYLEVEL then -- 剧情关卡
		result = DataCenter.newHeroActData:getStoryLevelResult(pointId, bShowResultWin == 0);
	end
	return result;
end