-- 引导数据
checkRequire("Lua_GuideVo");
checkRequire("Lua_GuideStepVo");
Lua_GuideData = class("Lua_GuideData");
function Lua_GuideData:ctor()
	self:superCall("ctor");
	
	self.guideStepPreLoadSign = "guideStepPreLoad";
	self.bOpenGuide = true; -- 默认开启引导
	self.curGuideId = 0; -- 当前引导id
	self.curGuideStep = 0;	-- 当前引导的步骤索引
	self.bSortWaitGuides = true; -- 排序等待引导队列
	self.waitGuideIds = Array(); -- 等待播放的引导队列，头部先播
	self.playedGuidesTab = {}; -- 服务器记录的引导id对应的步骤
	self._bInitServerInfo = false; -- 是否初始化服务器数据
	self._bInitConfig = false; -- 是否初始化配置表数据
	self._triggerConditionDic = {}; -- 触发条件列表
	self._guideVoDic = {};
	self._checkGuidesArr = Array();
	self.temporaryGuideIdsArr = Array();
	self.lastGuideIdDick = nil;
	self.ignoreGuideLimit = false; -- 谨慎使用
end

function Lua_GuideData:dispose()
	self.bOpenGuide = true;
	self.curGuideId = 0;
	self.curGuideStep = 0;
	self.bSortWaitGuides = true;
	self.waitGuideIds:clear();
	self.playedGuidesTab = {};
	self._bInitServerInfo = false;
	self._bInitConfig = false;
	self._triggerConditionDic = {};
	self._guideVoDic = {};
	self._checkGuidesArr:clear();
	self.temporaryGuideIdsArr:clear();
	self.lastGuideIdDick = nil;
	self._clickHeroDeltaTime = nil;
	self.preLoadWinDir = nil;
	self.ignoreGuideLimit = false;
	self._bAutoPlay = nil;
end

function Lua_GuideData:disposeTest()
	self.curGuideId = 0;
	self.curGuideStep = 0;
	self.bSortWaitGuides = true;
	self.waitGuideIds:clear();
	self._bInitConfig = false;
	self._triggerConditionDic = {};
	self._guideVoDic = {};
	self._checkGuidesArr:clear();
	self.temporaryGuideIdsArr:clear();
	self.lastGuideIdDick = nil;
end

-- 测试调整本地服务端数据
function Lua_GuideData:testUpdateServerInfo(gid, step)
	if self.playedGuidesTab == nil then
		self.playedGuidesTab = {};
	end
	
	local guideUtil = DataCenter.guideUtil;
	local lastGid = self:getLastGuideId(gid);
	while (lastGid > 0) do
		self.playedGuidesTab[lastGid] = guideUtil:getMaxStepByGuideId(lastGid);
		lastGid = self:getLastGuideId(lastGid);
	end

	local nextGid = guideUtil:getGuideNextId(gid);
	while (nextGid > 0) do
		self.playedGuidesTab[lastGid] = nil;
		nextGid = guideUtil:getGuideNextId(nextGid);
	end

	self.playedGuidesTab[gid] = step;
	self._bInitConfig = false;
	self._checkGuidesArr:clear();
	self.waitGuideIds:clear();
	self:initConfigs();
	if self.bOpenGuide then
		self:checkGuide();
		self:checkPlayGuide();
	end
end

function Lua_GuideData.Get:bInitServerInfo()
	return self._bInitServerInfo;
end

-- 根据引导id获取它上一个引导
function Lua_GuideData:getLastGuideId(gid)
	self:initConfigs();
	if self.lastGuideIdDick == nil or self.lastGuideIdDick[gid] == nil then
		return 0;
	end
	return self.lastGuideIdDick[gid];
end

function Lua_GuideData.Get:isGuiding()
	return self.curGuideId > 0;
end

-- 当前引导vo
function Lua_GuideData.Get:curGuideVo()
	if self.curGuideId <= 0 then
		return nil;
	end
	return self._guideVoDic[self.curGuideId];
end

-- 是否播放过该引导
function Lua_GuideData.Get:hasPlayedGuide(gid)
	return self.playedGuidesTab[gid] ~= nil and self.playedGuidesTab[gid] > 0;
end

-- 获取引导播放步骤
function Lua_GuideData:getGuidePlayedStep(gid)
	if self.playedGuidesTab[gid] ~= nil then
		return self.playedGuidesTab[gid];
	end
	return 0;
end

-- 检测引导是否完成某个步骤
function Lua_GuideData:checkGuideComplete(gid, step)
	if self.playedGuidesTab[gid] ~= nil then
		if step == nil then 
			step = DataCenter.guideUtil:getGuidePassStep(gid);
		end
		return self.playedGuidesTab[gid] >= step;
	end
	return false;
end

-- 检测引导是否完成某个步骤
function Lua_GuideData:checkGuideStepComplete(stepId)
	if stepId == nil or stepId <= 0 then
		return false;
	end

	local step = stepId % 1000;
	local gid = stepId - step;
	if self.playedGuidesTab[gid] ~= nil then
		return self.playedGuidesTab[gid] >= step;
	end
	return false;
end

-- 检查引导是否已完成
function Lua_GuideData:checkGuideEnd(gid)
	local config = DataCenter.guideUtil:getGuideCfg(gid);
	if config == nil then
		return true;
	end

	-- 下一阶段服务端数据完成，则当前阶段默认已完成
	local nextGuideId = config.nextGuideId;
	local judgeGuideId;
	if nextGuideId ~= nil and self.playedGuidesTab[nextGuideId] ~= nil then
		judgeGuideId = nextGuideId;
	elseif nextGuideId == nil and self.playedGuidesTab[gid] ~= nil then
		judgeGuideId = gid;
	end

	if judgeGuideId ~= nil then
		local nextConfig = DataCenter.guideUtil:getGuideCfg(judgeGuideId);
		local passStep = nextConfig.passStep or DataCenter.guideUtil:getMaxStepByGuideId(judgeGuideId, 1);
		local isComplete = self.playedGuidesTab[judgeGuideId] >= passStep;
		if isComplete then
			return true;
		end
	end

	-- 完成的判断条件,如有条件，所有成立就完成，无需再判断步骤	
	if config.completeConditions ~= nil and #config.completeConditions > 0 then
		local bCompleted = true;
		local conditionVo = Lua_GuideConditionVo.new();
		for i = 1, #config.completeConditions do
			conditionVo:setInfo(config.completeConditions[i], gid);
			if not conditionVo.isComplete then
				bCompleted = false;
				break;
			end
		end

		if bCompleted then
			return true;
		end
	end

	local step = self:getGuidePlayedStep(gid);
	local passStep = config.passStep;
	local maxStep = DataCenter.guideUtil:getMaxStepByGuideId(gid, 1);
	if passStep == nil then
		passStep = maxStep;
	elseif passStep > maxStep then
		passStep = maxStep;
	end
	if passStep == -1 then
		local nextStepCfg = DataCenter.guideUtil:getGuideStepCfg(gid, step + 1);
		if nextStepCfg == nil then
			return true;
		end
		
		-- 如果是只有创角条件，已创角则已完成
		if DataCenter.loginData.isCreateRole then
			while (true) do
				config = DataCenter.guideUtil:getGuideCfg(gid);
				if config ~= nil then
					local arr1 = config.triggerList1;
					local arr1Len = arr1 ~= nil and #arr1 or 0; 
					local arr2 = config.triggerList2;
					local arr2Len = arr2 ~= nil and #arr2 or 0; 
					-- print("checkGuideEnd isCreateRole gid", gid, arr1Len, arr2Len);
					if arr1Len == 1 and arr2Len == 0 then
						if arr1[1][1] == Lua_GuideDefine.Trigger_Condition_CreateRole then
							return true;
						else
							return false;
						end
					elseif arr2Len == 1 and arr1Len == 0 then
						if arr2[1][1] == Lua_GuideDefine.Trigger_Condition_CreateRole then
							return true;
						else
							return false;
						end
					else
						if arr1Len > 0 and arr2Len > 0 then
							return false;
						else
							local lastGid = self:getLastGuideId(gid);
							if lastGid ~= nil and lastGid > 0 then
								gid = lastGid;
							else
								return false;
							end
						end
					end
				else
					return false;
				end
			end
		end
		return false;
	end
	
	if step <= 0 then
		return false;
	end
	
	return step >= passStep;
end

-- 更新服务器数据
function Lua_GuideData:updateServerInfo(msg)
	if msg == nil or msg.guide_info == nil then
		return;
	end

	-- local str = "";
	for i = 1, #msg.guide_info do
		local info = msg.guide_info[i];
		self.playedGuidesTab[info.key] = info.value;
		-- str = str .. "(" .. info.key .. "、" .. info.value .. "),";
	end
	-- print("updateServerInfo", str);

	-- 检测触发引导
	self:checkGuide();
	self:checkPlayGuide();
	if not self._bInitServerInfo then
		self._bInitServerInfo = true;
		if not self.isGuiding then
			DataCenter.popupWinManager:checkPopupWin();
		end
	end
end

-- 同步引导步骤信息
function Lua_GuideData:syncGuideStep(gid, step, isTest, bRepeatStep)
	if gid == nil or gid <= 0 then 
		return;
	end

	if step <= 0 then
		return;
	end

	-- if self.curGuideId == gid then
	-- 	self.curGuideStep = step + 1;
	-- end
	
	if not isTest then
		self.playedGuidesTab[gid] = step;
		-- if self:checkGuideEnd(gid) then -- 引导结束移除触发条件，并检测它关联的下一个引导
		local stepCfg = DataCenter.guideUtil:getGuideStepCfg(gid, step + 1);
		-- print("Lua_GuideData, guideId", gid, "step", step + 1, stepCfg, self._guideVoDic[gid]);
		if stepCfg == nil then -- 无下一步引导
			local guideVo = self._guideVoDic[gid];
			if guideVo ~= nil then
				self:removeTrigger(gid);
				self._guideVoDic[gid] = nil;
				if not self.waitGuideIds:removeItem(gid) then
					self._checkGuidesArr:removeItem(gid);
				end

				local nextGuideId = guideVo.nextGuideId;
				if nextGuideId ~= nil and nextGuideId > 0 then 
					guideVo:reset(nextGuideId);
					-- print("syncGuideStep nextGuideId", nextGuideId);
					self._guideVoDic[nextGuideId] = guideVo;
					if guideVo:checkTrigger() then
						self.bSortWaitGuides = true;
						-- print("self.waitGuideIds:add(nextGuideId);", nextGuideId);
						self.waitGuideIds:add(nextGuideId);
					else
						-- print("self._checkGuidesArr:add(nextGuideId);", nextGuideId);
						self._checkGuidesArr:add(nextGuideId);
					end

					self:addTriggerByGuideId(nextGuideId);
				end
			end 
		end 

		-- 同步到服务器
		Lua_ProxyManager.getProxy("Lua_GuideProxy"):sSaveGuide(gid, step); 
		self:displayAssociationByType(gid, step);
	end

	if not bRepeatStep then
	-- 执行步骤结束后的操作
		self:checkGuideStepOverHandle(gid, step);
	end
end

-- 引导步骤结束后的处理
function Lua_GuideData:checkGuideStepOverHandle(gid, step)
	local cfg = DataCenter.guideUtil:getGuideStepCfg(gid, step);
	if cfg == nil then
		return;
	end

	local closeWins = cfg.closeWins;
	if closeWins ~= nil and #closeWins > 0 then
		for i = 1, #closeWins do
			local winName = closeWins[i];
			if not strIsNullOrEmpty(winName) then
				Lua_WindowUtil.openWin(winName, Lua_WinParam.ACTION_CLOSE);
			end
		end
	end

	local openWins = cfg.openWins;
	-- print("checkGuideStepOverHandle", gid, step, openWins ~= nil and #openWins or 0);
	if openWins ~= nil and #openWins > 0 then
		for i = 1, #openWins do
			local list = openWins[i];
			if list ~= nil and #list > 0 then
				local winName = list[1];
				if not strIsNullOrEmpty(winName) then
					if DataCenter.guideCommonData:isLoadingWin(winName) then
						self:showScene(winName, list[2]);
					else
						Lua_WindowUtil.openWin(winName, Lua_WinParam.ACTION_OPEN, list[2]);
					end
					-- print("checkGuideStepOverHandle openWin", winName, list[2]);
				end
			end
		end
	end

	local preLoadIds = cfg.preLoadIds;
	if preLoadIds ~= nil and #preLoadIds > 0 then
		local arr;
		for i = 1, #preLoadIds do
			local obj = DataCenter.preLoadUtil:getPreLoadInfo(preLoadIds[i]);
			if obj ~= nil then
				local vo = Lua_PreLoadResInfo.new();
		        vo.resType = obj.resType; -- 资源类型
		        vo.resName = obj.path; -- 资源名
		        vo.keepTime = obj.keepTime;
		        if arr == nil then arr = Array(); end
		        -- print("checkGuideStepOverHandle preLoad", obj.resType, obj.path, obj.keepTime);
		        arr:add(vo);
		    end
		end

		if arr ~= nil then
			local id = gid * 1000 + step;
			DataCenter.preLoadData:preLoad(self.guideStepPreLoadSign..id, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, nil, nil, nil, arr);
		end
	end
end

-- 播放对应引导id的引导
function Lua_GuideData:play(gid, step)
	if gid == nil or gid <= 0 then -- 播放下一个
		self:checkPlayGuide(); 
		return;
	end

	if gid == self.curGuideId then
		if step == nil or step == self:getGuidePlayedStep(gid) then
			OperateAlert.show("正在播放当前引导："..gid);
			return;
		end
		self.playedGuidesTab[gid] = step;
	else
		if self:addPlayGuide(gid, step) then
			return;
		end
	end

	self.curGuideId = gid;
	Lua_WindowUtil.openWin("Lua_GuideWindow", Lua_WinParam.ACTION_OPEN, gid);
end

-- 添加到播放列表
function Lua_GuideData:addPlayGuide(gid, step)
	if gid == nil or gid <= 0 then
		return true;
	end

	local lastId = self:getLastGuideId(gid);
	if lastId > 0 and not self:checkGuideEnd(lastId) then -- 上一个未播放完成
		return true;
	end

	if step == nil then
		step = self:getGuidePlayedStep(gid);
	end

	local vo = self._guideVoDic[gid];
	if vo == nil then
		local cfg = DataCenter.guideUtil:getGuideCfg(gid);
		if cfg == nil then
			return true;
		end

		self.playedGuidesTab[gid] = step;
		local guideUtil = DataCenter.guideUtil
		while (gid > 0 and self:checkGuideEnd(gid)) do
			gid = guideUtil:getGuideNextId(gid);
		end

		if gid <= 0 then
			return true;
		end

		vo = Lua_GuideVo.new(gid);
		self._guideVoDic[gid] = vo;
		if vo:checkTrigger() or vo.isConditionEmpty then
			self.waitGuideIds:add(gid);
			if self.curGuideId <= 0 then
				return false;
			else
				self.bSortWaitGuides = true;
			end
		else
			self._checkGuidesArr:add(gid);
		end

		self:addTriggerByGuideId(gid);
	else
		self.playedGuidesTab[gid] = step;
		local nextGuideId = gid;
		local guideUtil = DataCenter.guideUtil
		while (nextGuideId > 0 and self:checkGuideEnd(nextGuideId)) do
			nextGuideId = guideUtil:getGuideNextId(nextGuideId);
		end
		if nextGuideId <= 0 then
			self:removeTrigger(gid);
			self._guideVoDic[gid] = nil;
			if not self.waitGuideIds:removeItem(gid) then
				self._checkGuidesArr:removeItem(gid)
			end
		else
			if nextGuideId ~= gid then
				self:removeTrigger(gid);
				self._guideVoDic[gid] = nil;
				if not self.waitGuideIds:removeItem(gid) then
					self._checkGuidesArr:removeItem(gid);
				end
				gid = nextGuideId;
				vo:reset(gid);
				self._guideVoDic[gid] = vo;
				if vo:checkTrigger() or vo.isConditionEmpty then -- 可播放
					self.waitGuideIds:add(gid);
					if self.curGuideId <= 0 then
						return false;
					else
						self.bSortWaitGuides = true;
					end
				else
					self._checkGuidesArr:add(gid);
				end

				self:addTriggerByGuideId(gid);
			else
				if self.waitGuideIds:indexOf(gid) ~= -1 then
					if self.curGuideId <= 0 then
						return false;
					end
				elseif vo.isConditionEmpty then
					if self._checkGuidesArr:removeItem(gid) then
						self.waitGuideIds:add(gid);
						if self.curGuideId <= 0 then
							return false;
						else
							self.bSortWaitGuides = true;
						end
					end
				end
			end
		end
	end

	return true;	
end

-- 关闭播放引导
function Lua_GuideData:stop(gid)
	if gid == nil or gid == self.curGuideId then
		Lua_WindowUtil.openWin("Lua_GuideWindow", Lua_WinParam.ACTION_CLOSE);
		return;
	end
end

-- 检测触发的引导
function Lua_GuideData:checkGuide()
	if not self.bOpenGuide then -- 关闭引导
		return;
	end

	if self:initConfigs() then -- 初始化
		return;
	end

	self.temporaryGuideIdsArr:clear();
	local len = self.waitGuideIds:getSize();
	-- print("self.waitGuideIds len", len);
	while (len > 0) do
		local gid = self.waitGuideIds[len];
		if gid ~= self.curGuideId then -- 当前引导不做判断
			local vo = self._guideVoDic[gid];
			while self:checkGuideEnd(gid) do
				self:removeTrigger(gid);
				info = DataCenter.guideUtil:getGuideCfg(gid);
				local nextGuideId = info.nextGuideId;
				if nextGuideId == nil or nextGuideId <= 0 then -- 无
					gid = -1;
					break;
				else
					gid = nextGuideId;
				end
			end

			if gid ~= -1 then
				if vo == nil then
					vo = Lua_GuideVo.new(gid);
					self._guideVoDic[gid] = vo;
					self.waitGuideIds:remove(len);
					self:addTriggerByGuideId(gid);
					if not vo:checkTrigger() then
						self.temporaryGuideIdsArr:add(gid);
					else
						self.bSortWaitGuides = true;
						self.waitGuideIds:add(gid);
					end
				else
					local bChange = gid ~= vo.guideId;
					if bChange then
						self._guideVoDic[vo.guideId] = nil;
						vo:reset(gid);
						self._guideVoDic[gid] = vo;
						self.waitGuideIds:remove(len);
						self:addTriggerByGuideId(gid);
					end
					
					if not vo:checkTrigger() then
						if not bChange then
							self.waitGuideIds:remove(len);
						end
						self.temporaryGuideIdsArr:add(gid);
					else
						if bChange then
							self.bSortWaitGuides = true;
							self.waitGuideIds:add(gid);
						end
					end
				end
			else
				local param = self.waitGuideIds:remove(len);
				self:removeTrigger(param);
			end
		end
		len = len - 1;
	end

	len = self._checkGuidesArr:getSize();
	-- print("self._checkGuidesArr len", len);
	while (len > 0) do
		local gid = self._checkGuidesArr[len];
		local vo = self._guideVoDic[gid];
		-- print("checkGuide", gid, self:checkGuideEnd(gid));
		while self:checkGuideEnd(gid) do
			self:removeTrigger(gid);
			-- print("self:checkGuideEnd(gid) true", gid);
			info = DataCenter.guideUtil:getGuideCfg(gid);
			local nextGuideId = info.nextGuideId;
			if nextGuideId == nil or nextGuideId <= 0 then -- 无
				gid = -1;
				break;
			else
				gid = nextGuideId;
			end
		end

		-- print("checkGuide checkTrigger ", gid);
		if gid ~= -1 then 
			if vo == nil then
				vo = Lua_GuideVo.new();
				self._guideVoDic[gid] = vo;
				self._checkGuidesArr:remove(len);
				self:addTriggerByGuideId(gid);
				if vo:checkTrigger() then 
					self.bSortWaitGuides = true;
					self.waitGuideIds:add(gid);
				else
					self._checkGuidesArr:add(gid);
				end
			else
				local bChange = gid ~= vo.guideId;
				if bChange then
					self._guideVoDic[vo.guideId] = nil;
					vo:reset(gid);
					self._guideVoDic[gid] = vo;
					self._checkGuidesArr:remove(len);
					self:addTriggerByGuideId(gid);
				end

				if vo:checkTrigger() then 
					self.bSortWaitGuides = true;
					self.waitGuideIds:add(gid);
					if not bChange then
						self._checkGuidesArr:remove(len);
					end
				else
					if bChange then
						self._checkGuidesArr:add(gid);
					end
				end
			end
		else
			local param = self._checkGuidesArr:remove(len);
			self:removeTrigger(param);
		end
		len = len - 1;
	end

	len = self.temporaryGuideIdsArr:getSize();
	while (len > 0) do
		local gid = self.temporaryGuideIdsArr[len];
		self.temporaryGuideIdsArr.remove(len);
		self._checkGuidesArr:add(gid);
		len = len - 1;
	end
end

function Lua_GuideData:findHead(dic, headArr, lastDic)
	if dic == nil then
		-- print("findHead dic == nil");
		return;
	end

	local bChange = false;
	for k, v in pairs(dic) do
		local lastGuideId = v.lastGuideId;
		local nextGuideId = v.nextGuideId;
		-- print("findHead", k, lastGuideId, nextGuideId);
		if lastGuideId == -1 then -- 没有上一个，那就是头
			headArr:add(k);
		else
			lastDic[k] = lastGuideId;
			-- print(k, " lastGuideId ", lastGuideId);
			while (lastGuideId ~= -1) do
				v = dic[lastGuideId];
				if v == nil then
					break;
				end

				dic[lastGuideId] = nil;
				if v.lastGuideId == -1 then
					headArr:add(lastGuideId);
					break;
				else
					lastDic[lastGuideId] = v.lastGuideId;
					-- print(lastGuideId, " lastGuideId ", v.lastGuideId);
					lastGuideId = v.lastGuideId;
				end
			end
		end

		while (nextGuideId ~= -1) do -- 有下一个，后续都不可能是头
			v = dic[nextGuideId];
			if v ~= nil then
				lastDic[v.nextGuideId] = nextGuideId;
				-- print(v.nextGuideId, " lastGuideId3 ", nextGuideId);
				nextGuideId = v.nextGuideId;
				dic[v.nextGuideId] = nil;
			else
				break;
			end
		end

		bChange = true;
		dic[k] = nil;
		break;
	end

	-- print("findHead dic ~= nil", bChange);
	if bChange then -- 继续找下一个
		self:findHead(dic, headArr, lastDic);
	end
end

-- 初始化引导配置数据，主要提取触发检测条件
function Lua_GuideData:initConfigs()
	if self._bInitConfig then
		return false;
	end

	self._bInitConfig = true;
	local cfg = DataCenter.guideUtil.guideConfig;
	if cfg == nil then
		return true;
	end

	self.lastGuideIdDick= {};
	local linkDic = {}; -- {lastGuideId, nextGuideId}
	for k, v in pairs(cfg) do
		local nextGuideId = v.nextGuideId;
		if nextGuideId ~= nil then
			if linkDic[k] == nil then
				linkDic[k] = {lastGuideId = -1, nextGuideId = nextGuideId};
			else
				linkDic[k].nextGuideId = nextGuideId;
			end

			if linkDic[nextGuideId] == nil then
				linkDic[nextGuideId] = {lastGuideId = k, nextGuideId = -1};
			else
				linkDic[nextGuideId].lastGuideId = k;
			end
		else
			if linkDic[k] == nil then
				linkDic[k] = {lastGuideId = -1, nextGuideId = -1};
			else
				linkDic[k].nextGuideId = -1;
			end
		end
	end

	local info;
	local headArr = Array();
	self:findHead(linkDic, headArr, self.lastGuideIdDick);
	local len = headArr:getSize();
	for i = 1, len do
		local gid = headArr[i];
		-- print(gid, "checkGuideEnd", self:checkGuideEnd(gid))
		while self:checkGuideEnd(gid) do
			info = DataCenter.guideUtil:getGuideCfg(gid);
			local nextGuideId = info.nextGuideId;
			if nextGuideId == nil or nextGuideId <= 0 then -- 无
				gid = -1;
				break;
			else
				gid = nextGuideId;
			end
		end

		-- print("gid = ", gid);
		if gid ~= -1 then 
			local guideVo = Lua_GuideVo.new(gid);
			self._guideVoDic[gid] = guideVo;
			if guideVo:checkTrigger() then -- 触发
				self.bSortWaitGuides = true;
				-- print("self.waitGuideIds:add(gid)", gid);
				self.waitGuideIds:add(gid);
			else
				-- print("self._checkGuidesArr:add(gid)", gid);
				self._checkGuidesArr:add(gid);
			end

			-- 不管是等待的引导还是独立引导，都要加入条件判断，以免条件状态变更
			self:addTriggerByGuideId(gid);
		end
	end

	return true;
end

-- 根据引导id添加处发条件
function Lua_GuideData:addTriggerByGuideId(guideId)
	local guideVo = self._guideVoDic[guideId];
	if guideVo ~= nil then
		local arr = guideVo.triggerConditionsYu;
		if arr ~= nil then
			for j = 1, arr:getSize() do
				self:addTrigger(arr[j]);
			end
		end

		arr = guideVo.triggerConditionsHuo;
		if arr ~= nil then
			for j = 1, arr:getSize() do
				self:addTrigger(arr[j]);
			end
		end
	end
end

-- 加入触发条件
function Lua_GuideData:addTrigger(conditionVo)
	if conditionVo == nil then
		return;
	end

	local t = conditionVo.conditionType;
	local param = conditionVo.param1;
	if self._triggerConditionDic[t] == nil then 
		self._triggerConditionDic[t] = {};
	end
	if self._triggerConditionDic[t][param] == nil then
		self._triggerConditionDic[t][param] = Array();
	end
	self._triggerConditionDic[t][param]:add(conditionVo);
end

-- 移除触发条件
function Lua_GuideData:removeTrigger(gid)
	if gid == nil or gid <= 0 then
		return;
	end

	local guideVo = self._guideVoDic[gid];
	if guideVo == nil then
		return;
	end

	local triggerList = guideVo.triggerConditionsYu;
	if triggerList ~= nil then
		for i = 1, triggerList:getSize() do
			local conditionVo = triggerList[i];
			local t = conditionVo.conditionType;
			local param = conditionVo.param1;
			if self._triggerConditionDic[t] ~= nil then
				local arr = self._triggerConditionDic[t][param];
				if arr == nil then
					self._triggerConditionDic[t][param] = nil;
				else
					arr:removeItem(conditionVo);
					if arr:getSize() <= 0 then
						self._triggerConditionDic[t][param] = nil;
					end
				end
			end
		end
	end

	triggerList = guideVo.triggerConditionsHuo;
	if triggerList ~= nil then
		for i = 1, triggerList:getSize() do
			local conditionVo = triggerList[i];
			local t = conditionVo.conditionType;
			local param = conditionVo.param1;
			if self._triggerConditionDic[t] ~= nil then
				local arr = self._triggerConditionDic[t][param];
				if arr == nil then
					self._triggerConditionDic[t][param] = nil;
				else
					arr:removeItem(conditionVo);
					if arr:getSize() <= 0 then
						self._triggerConditionDic[t][param] = nil;
					end
				end
			end
		end
	end
end

-- 检测是否有可播放的引导
function Lua_GuideData:checkPlayGuide(gid)
	-- print("checkPlayGuide", gid, self.curGuideId, self.waitGuideIds:getSize());
	if self.curGuideId <= 0 and self.waitGuideIds:getSize() > 0 then
		self:sortWaitGuideIds();
		if gid ~= nil and gid > 0 then
			if self.waitGuideIds:indexOf(gid) ~= -1 then
				self:play(gid);
				return;
			end
		end

		gid = self.waitGuideIds[1];
		self:play(gid);
	end
end

function Lua_GuideData:sortWaitGuideIds()
	if not self.bSortWaitGuides then
		return;
	end

	self.bSortWaitGuides = false;
	quickSortf(self.waitGuideIds, self, self.sortWaitsFun);
end

function Lua_GuideData:sortWaitsFun(gidA, gidB)
	local cfgA = DataCenter.guideUtil:getGuideCfg(gidA);
	local sortA = (cfgA ~= nil and cfgA.sort ~= nil) and cfgA.sort or 99999999;
	local cfgB = DataCenter.guideUtil:getGuideCfg(gidB);
	local sortB = (cfgB ~= nil and cfgB.sort ~= nil) and cfgB.sort or 99999999;
	return sortA >= sortB;
end

function Lua_GuideData:checkConditionsByType(conditionType)
	if self._triggerConditionDic ~= nil then
		local tab = self._triggerConditionDic[conditionType];
		if tab ~= nil then
			for k, arr in pairs(tab) do
				self:checkConditions(arr);
			end
		end
	end
end

function Lua_GuideData:checkConditions(conditionsArr)
	if conditionsArr == nil then
		return;
	end

	local vo;
	local gid;
	local conditionVo;
	local len = conditionsArr:getSize();
	for i = 1, len do
		conditionVo = conditionsArr[i];
		if conditionVo.isComplete then
			gid = conditionVo.guideId;
			vo = self._guideVoDic[gid];
			if vo ~= nil and vo:checkTrigger(conditionVo) then
				if self.waitGuideIds:indexOf(gid) == -1 then
					self.bSortWaitGuides = true;
					self.waitGuideIds:add(gid);
				end
				self._checkGuidesArr:removeItem(gid);
			end
		end
	end
end

-- 等级提升检测
function Lua_GuideData:levelUpCheck()
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	if self._triggerConditionDic ~= nil then
		local tab = self._triggerConditionDic[Lua_GuideDefine.Trigger_Condition_Level];
		if tab ~= nil then
			local lv = DataCenter.roleData.level;
			for k, arr in pairs(tab) do
				if k <= lv then
					self:checkConditions(arr);
				end
			end
		end
	end

	-- 播放引导
	self:checkPlayGuide();
end

-- 主线冒险任务变化检测
function Lua_GuideData:mainLineTaskCheck(taskType)
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_MainLineTask);	
	self:checkPlayGuide();-- 播放引导
end

-- 主线冒险阶段奖励检测
function Lua_GuideData:mainLineTaskStageCheck()
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_MainLineStage);	
	self:checkPlayGuide();-- 播放引导
end

-- 主线推图检测
function Lua_GuideData:mainlineCheck()
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_MainLine);	
	self:checkPlayGuide();-- 播放引导
end

-- 功能开启检测
function Lua_GuideData:funOpenCheck(openDic)
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_FunOpen);	
	self:checkPlayGuide();-- 播放引导
end

-- 创角引导检测，需要创角引导则会调用该方法
function Lua_GuideData:createRoleCheck()
	if not self.bOpenGuide then -- 关闭引导
		return;
	end

	self:initConfigs(); -- 服务端数据还未下发，需检测创角引导
	--self:checkPlayGuide(); -- 播放引导，创角引导代码控制，不在这主动触发
end

-- 场景显示检测
function Lua_GuideData:sceneShowCheck(sceneId)
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_SceneShow);
	self:checkPlayGuide();-- 播放引导
end

-- 界面显示检测
function Lua_GuideData:winShowCheck(winName)
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_OpenWin);	
	self:checkPlayGuide();-- 播放引导
end

-- 道具检测
function Lua_GuideData:itemCheck()
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_Item);	
	self:checkPlayGuide();-- 播放引导
end

-- 英雄检测
function Lua_GuideData:heroCheck()
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_Hero);	
	self:checkPlayGuide();-- 播放引导
end

-- 上阵检测
function Lua_GuideData:lineUpCheck()
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_LineUp);	
	self:checkPlayGuide();-- 播放引导
end

-- 神器检测
function Lua_GuideData:lineUpArtifactCheck()
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_Artifact);	
	self:checkPlayGuide();-- 播放引导
end

-- 藏品柜
function Lua_GuideData:collectionItemCheck()
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_CollectionItem);	
	self:checkPlayGuide();-- 播放引导
end

-- 战斗状态
function Lua_GuideData:onFightState(templateId)
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_FightState);	
	self:checkPlayGuide();-- 播放引导
end

-- 战斗类型结算窗口
function Lua_GuideData:fightTypeResultWinCheck()
	if not self.bOpenGuide or not self._bInitServerInfo then -- 关闭引导
		return;
	end

	self:checkConditionsByType(Lua_GuideDefine.Trigger_Condition_FightTypeResultWin);	
	self:checkPlayGuide();-- 播放引导
end










-- 获取预加载的资源
function Lua_GuideData:getPreLoadListByGuideId(gid)
	if gid == nil and self.waitGuideIds:getSize() > 0 then
		self:sortWaitGuideIds();
		gid = self.waitGuideIds[1];
	end

	if gid == nil then
		return nil;
	end


end

--英雄场景
function Lua_GuideData:showScene(winName, heroId)
	if self.preLoadWinDir == nil then
		self.preLoadWinDir = {};
	end
	if winName ~= nil and heroId ~= nil then
		self.preLoadWinDir[winName] = true;
		local heroVo = Lua_HeroVo.new(heroId);
		heroVo.winName = winName;
		DataCenter.preLoadData:preShowModelScene(winName, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onShowModel, heroVo, heroVo);
	end
end

function Lua_GuideData:onShowModel(param, modelScenePanel)
    param.modelScenePanel = modelScenePanel;
    Lua_WindowUtil.openWin(param.winName, Lua_WinParam.ACTION_OPEN, param);
end

function Lua_GuideData:removeShowModel(winName)
	if self.preLoadWinDir ~= nil and self.preLoadWinDir[winName] then
		self.preLoadWinDir[winName] = nil;
    	DataCenter.preLoadData:removeBySign(winName, self, self.onShowModel);
    end
end

-- GM- 引导全部完成
function Lua_GuideData:completeAllGuide()
	local config = DataCenter.guideUtil.guideConfig;
	local passStep;
	local bPass;
	for k, v in pairs(config) do
		bPass = v.type == 5;--Mathf.Floor(k / 10000) == 5;
		if not bPass then
			passStep = v.passStep;
			if passStep ~= nil and passStep == -1 then
				Lua_ProxyManager.getProxy("Lua_GuideProxy"):sSaveGuide(k, passStep); 
			else
				passStep = DataCenter.guideUtil:getMaxStepByGuideId(k); 
				Lua_ProxyManager.getProxy("Lua_GuideProxy"):sSaveGuide(k, passStep); 
			end
			self.playedGuidesTab[k] = passStep;
		end
	end
end


-- 关联其他功能
function Lua_GuideData:displayAssociationByType(gid, step, bCheckPlayEnd)
	local bPlayEnd = true;
	if bCheckPlayEnd then
		local cfg = DataCenter.guideUtil:getGuideCfg(gid);
		local passStep = cfg.passStep or DataCenter.guideUtil:getMaxStepByGuideId(gid, 1);
		if step < passStep then
			bPlayEnd = false;
		end
	end
	if not bPlayEnd then 
		return;
	end

	local associateType = DataCenter.guideUtil:getAssociationType(gid);
	if associateType == 1 then
		DataCenter.newHeroActData:displayWhenStroyEnd();
	end
end

-- 自动播放
function Lua_GuideData.Get:bAutoPlay()
	if self._bAutoPlay == nil then
		return;
	end

	local type = DataCenter.guideUtil:getAutoPlayTypeByGuideId(self.curGuideId);
	if type == nil then
		return;
	else
		return self._bAutoPlay[type];
	end
end

function Lua_GuideData.Set:bAutoPlay(value)
	if self._bAutoPlay == nil then
		self._bAutoPlay = {};
	end

	local type = DataCenter.guideUtil:getAutoPlayTypeByGuideId(self.curGuideId);
	if type == nil then
		return;
	else
		self._bAutoPlay[type] = value;
	end
end