-- 具体引导步骤
checkRequire("Lua_GuideConditionVo");
Lua_GuideStepVo = class("Lua_GuideStepVo");
function Lua_GuideStepVo:ctor(gid, step)
	self:superCall("ctor");
	self:reset(gid, step);
end

-- 重置数据
function Lua_GuideStepVo:reset(gid, step)
	self.guideId = gid;
	self.step = step;
	self.id = gid * 1000 + step;
	self.config = DataCenter.guideUtil:getGuideStepCfg(gid, step);
	self:returnConditonVos(self._completeConditionsArr);
	self:returnConditonVos(self._operateConditionsArr);
	self:returnConditonVos(self._repeatConditionsArr);
	self.canOperate = false;
	self.isCompleteEvent = false;
	self._completeConditionTypesDic = nil;
	self._operateConditionTypesDic = nil;
	self._repeatConditionsTypeDic = nil;
	self._completeConditionsArr = nil;
	self._operateConditionsArr = nil;
	self._repeatConditionsArr = nil;
	self.winNameDir = nil; -- 该引导步骤中条件所需的窗口名Dir
end

function Lua_GuideStepVo.Get:canPopupWin()
	if self.config == nil then
		return true;
	end

	local contentTemplate = self.config.contentTemplate;
	if contentTemplate ~= nil and self.step > 1 then
		if contentTemplate == 1 
			or contentTemplate == 2 
			or contentTemplate == 3 
			or contentTemplate == 4 then
			return false;
		end
	end
	return true;
end

-- 步骤为空
function Lua_GuideStepVo.Get:isEmpty()
	return self.config == nil;
end

function Lua_GuideStepVo.Get:skipStep()
	if self.config ~= nil and self.config.skipStep ~= nil then
		return self.config.skipStep;
	end
	return 0;
end

function Lua_GuideStepVo.Get:checkConditionTime()
	if self.config ~= nil and self.config.checkConditionTime ~= nil then
		return self.config.checkConditionTime / 1000;
	end
	return 0;
end

-- 完成条件成功后是否自动下一步
function Lua_GuideStepVo.Get:isCompleteConditionsNext()
	return self.config ~= nil 
		and self.config.completeConditionsNext ~= nil 
		and (self.config.completeConditionsNext == 1 or self.config.completeConditionsNext == 2)
		and (self.config.completeConditions ~= nil and #self.config.completeConditions > 0);
end

-- 检测条件 changeState 0 无变化，1 开始执行的检测条件，2 完成的判断条件， 3 是12都成立
function Lua_GuideStepVo:checkConditon(conditionType)
	if self.config == nil then
		return 0;
	end

	if self.config ~= nil and self.config.repeatConditions ~= nil and not self.isCompleteEvent then 
		local repeatConditions = self.config.repeatConditions;
		if self._repeatConditionsArr == nil then
			local bRepeat = true;
			for i = 1, #repeatConditions do
				local list = repeatConditions[i];
				local vo = self:createConditionVo();
				vo:setInfo(list);
				if not vo.isComplete then
					bRepeat = false;
					if self._repeatConditionsArr == nil then
						self._repeatConditionsArr = Array();
					end
					self._repeatConditionsArr:add(vo);
					if self._repeatConditionsTypeDic == nil then
						self._repeatConditionsTypeDic = {};
					end
					if self._repeatConditionsTypeDic[vo.conditionType] == nil then
						self._repeatConditionsTypeDic[vo.conditionType] = 1;
					else
						local num = self._repeatConditionsTypeDic[vo.conditionType];
						self._repeatConditionsTypeDic[vo.conditionType] = num + 1;
					end
				else
					self:returnConditonVo(vo);
				end
			end
			if bRepeat then
				return -1; -- 需重新引导该步骤
			end
		else
			local bRepeat = true;
			if conditionType == nil or 
				(self._repeatConditionsTypeDic ~= nil 
					and self._repeatConditionsTypeDic[conditionType] ~= nil 
					and self._repeatConditionsTypeDic[conditionType] > 0) then
				local num = self._repeatConditionsArr:getSize();
				if num >= 0 then
					for i = num, 1, -1 do
						local vo = self._repeatConditionsArr[i];
						if not vo.isComplete then
							bRepeat = false;
						else
							local count = self._repeatConditionsTypeDic[vo.conditionType];
							self._repeatConditionsTypeDic[vo.conditionType] = count - 1;
							self:returnConditonVo(self._repeatConditionsArr:remove(count));
						end
					end

					if bRepeat then
						return -1;
					end
				end
			end
		end
	end

	local changeState = 0;
	if not self.canOperate then
		if self._operateConditionsArr == nil then
			-- print("self.config.operateConditions", self.config.operateConditions == nil);
			-- if self.config.operateConditions ~= nil then
			-- 	print("self.config.operateConditions", #self.config.operateConditions);
			-- end
			if self.config == nil or self.config.operateConditions == nil or #self.config.operateConditions <= 0 then
				changeState = 1;
				self.canOperate = true;
			else
				changeState = 1;
				self.canOperate = true;
				local operateConditions = self.config.operateConditions;
				for i = 1, #operateConditions do
					local list = operateConditions[i];
					local vo = self:createConditionVo();
					vo:setInfo(list);
					if not vo.isComplete then
						changeState = 0;
						self.canOperate = false;
						if self._operateConditionsArr == nil then
							self._operateConditionsArr = Array();
						end
						self._operateConditionsArr:add(vo);

						if self._operateConditionTypesDic == nil then
							self._operateConditionTypesDic = {};
						end
						if self._operateConditionTypesDic[vo.conditionType] == nil then
							self._operateConditionTypesDic[vo.conditionType] = 1;
						else
							local num = self._operateConditionTypesDic[vo.conditionType];
							self._operateConditionTypesDic[vo.conditionType] = num + 1;
						end

						-- 记录win条件的窗口名
						self:setWinConditionDir(vo, true);
					else
						self:returnConditonVo(vo);
					end
				end
			end
		else
			-- if self._operateConditionTypesDic ~= nil then
			-- 	print("conditionType", conditionType, self._operateConditionTypesDic[conditionType], self._operateConditionsArr:getSize())
			-- end
			if conditionType == nil or 
				(self._operateConditionTypesDic ~= nil 
					and self._operateConditionTypesDic[conditionType] ~= nil 
					and self._operateConditionTypesDic[conditionType] > 0) then
				changeState = 1;
				self.canOperate = true;
				if self._operateConditionsArr ~= nil then
					local len = self._operateConditionsArr:getSize();
					while (len > 0) do
						local vo = self._operateConditionsArr[len];
						-- print("_operateConditionTypesDic len", len, vo.isComplete)
						if vo.isComplete then
							-- 清除win条件的窗口名
							self:setWinConditionDir(vo, false);

							local num = self._operateConditionTypesDic[vo.conditionType];
							self._operateConditionTypesDic[vo.conditionType] = num - 1;
							self:returnConditonVo(self._operateConditionsArr:remove(len));
							-- print("_operateConditionTypesDic", num)
							-- if num - 1 > 0 then
							-- 	changeState = 0;
							-- 	self.canOperate = false;
							-- end
						else
							changeState = 0;
							self.canOperate = false;
							-- break;
						end
						len = len - 1;
					end
				end
			end
		end
	end

	if self.config.completeConditionsNext ~= nil and self.config.completeConditionsNext == 2 then
		-- print("self.config.completeConditionsNext = ",self.config.completeConditionsNext, self.canOperate);
		if not self.canOperate then
			-- print("1changeState = ", changeState, self.guideId, self.step, self.isCompleteEvent, self.canOperate);
			return changeState;
		end
	end

	if not self.isCompleteEvent then
		if self._completeConditionsArr == nil then
			-- if self.config.completeConditions ~= nil then
			-- 	print("completeConditions", #self.config.completeConditions);
			-- else
			-- 	print("completeConditions self.config.completeConditions null");
			-- end
			if self.config == nil or self.config.completeConditions == nil or #self.config.completeConditions <= 0 then
				changeState = (changeState == 1) and 3 or 2;
				self.isCompleteEvent = false;
			else
				self.isCompleteEvent = true;
				local completeConditions = self.config.completeConditions;
				for i = 1, #completeConditions do
					local list = completeConditions[i];
					local vo = self:createConditionVo();
					vo:setInfo(list);
					if not vo.isComplete then
						self.isCompleteEvent = false;
						if self._completeConditionsArr == nil then
							self._completeConditionsArr = Array();
						end
						self._completeConditionsArr:add(vo);

						if self._completeConditionTypesDic == nil then
							self._completeConditionTypesDic = {};
						end
						if self._completeConditionTypesDic[vo.conditionType] == nil then
							self._completeConditionTypesDic[vo.conditionType] = 1;
						else
							local num = self._completeConditionTypesDic[vo.conditionType];
							self._completeConditionTypesDic[vo.conditionType] = num + 1;
						end

						-- 记录win条件的窗口名
						self:setWinConditionDir(vo, true);
					else
						self:returnConditonVo(vo);
					end
				end

				if self.isCompleteEvent then
					changeState = (changeState == 1) and 3 or 2;
				end
			end
		else
			if conditionType == nil or 
				(self._completeConditionTypesDic ~= nil 
					and self._completeConditionTypesDic[conditionType] ~= nil 
					and self._completeConditionTypesDic[conditionType] > 0) then
				-- print("11111checkConditon isCompleteEvent", self._completeConditionsArr:getSize());
				if self._completeConditionsArr ~= nil and self._completeConditionsArr:getSize() > 0 then
					self.isCompleteEvent = true;
					local len = self._completeConditionsArr:getSize();
					while (len > 0) do
						local vo = self._completeConditionsArr[len];
						if vo.isComplete then
							-- 清除win条件的窗口名
							self:setWinConditionDir(vo, false);

							local num = self._completeConditionTypesDic[vo.conditionType];
							self._completeConditionTypesDic[vo.conditionType] = num - 1;
							self:returnConditonVo(self._completeConditionsArr:remove(len));
						else
							self.isCompleteEvent = false;
							-- print("22222checkConditon isCompleteEvent false", vo.conditionType, vo.param1);
							-- break;
						end
						len = len - 1;
					end

					if self.isCompleteEvent then
						changeState = (changeState == 1) and 3 or 2;
					end
				end
			-- else
			-- 	print("checkConditon isCompleteEvent", conditionType);
			end
		end
	end

	-- print("2changeState = ", changeState, self.guideId, self.step, self.isCompleteEvent, self.canOperate);
	return changeState;
end

--function Lua_GuideStepVo:printInfo(index, changeState, param1, param2, param3, param4, param5, param6)
--	if self.id == 11002014 then
--		print("self.id == 11002014", param1, param2, param3, param4, param5, param6)
--	end
--end

function Lua_GuideStepVo:createConditionVo()
	local vo;
	if self._idleConditionVoArr ~= nil then
		vo = self._idleConditionVoArr:remove();
	end
	if vo == nil then
		vo = Lua_GuideConditionVo.new();
	end
	return vo;
end

function Lua_GuideStepVo:returnConditonVos(arr)
	if arr == nil then
		return;
	end

	local len = arr:getSize();
	while (len > 0) do
		self:returnConditonVo(arr:remove(len));
		len = len - 1;
	end
end

function Lua_GuideStepVo:returnConditonVo(vo)
	if vo == nil then
		return;
	end

	if self._idleConditionVoArr == nil then
		self._idleConditionVoArr = Array();
	end
	self._idleConditionVoArr.add(vo);
end

-- 处理引导中所需的win条件名
function Lua_GuideStepVo:setWinConditionDir(conditionVo, bSave)
	local ctype = Lua_GuideDefine.Trigger_Condition_OpenWin;
	if conditionVo ~= nil and conditionVo.conditionType == ctype and conditionVo.param1 ~= nil then
		if self.winNameDir == nil then
			self.winNameDir = {};
		end
		if bSave then
			self.winNameDir[conditionVo.param1] = true;
		else
			self.winNameDir[conditionVo.param1] = nil;
		end
	end
end

function Lua_GuideStepVo:isConditionByWinName(winName)
	if self.isCompleteEvent and self.canOperate then
		return false;
	end

	if self._operateConditionTypesDic == nil and self._completeConditionTypesDic == nil then
		return false;
	end

	if self.winNameDir ~= nil and self.winNameDir[winName] then
		return true;
	end

	return false;
	-- if self._operateConditionTypesDic ~= nil 
	-- 	and self._operateConditionTypesDic[ctype] ~= nil 
	-- 	and self._operateConditionTypesDic[ctype] > 0 then
	-- 	local len = self._operateConditionsArr:getSize();
	-- 	for i = 1, len do
	-- 		local vo = self._operateConditionsArr[i];
	-- 		if vo.conditionType == ctype and vo.param1 ~= nil and vo.param1 == winName then
	-- 			return true;
	-- 		end
	-- 	end
	-- end

	-- if self._completeConditionTypesDic ~= nil 
	-- 	and self._completeConditionTypesDic[ctype] ~= nil 
	-- 	and self._completeConditionTypesDic[ctype] > 0 then
	-- 	local len = self._completeConditionsArr:getSize();
	-- 	for i = 1, len do
	-- 		local vo = self._completeConditionsArr[i];
	-- 		if vo.conditionType == ctype and vo.param1 ~= nil and vo.param1 == winName then
	-- 			return true;
	-- 		end
	-- 	end
	-- end

	-- return false;
end

-- 是否需要检测超时
function Lua_GuideStepVo.Get:needCheckTimeOut()
	if self.config ~= nil and self.config.unCheckTimeOut ~= nil or self.config.unCheckTimeOut == 1 then
		return false;
	end
	return true;
end