-- 单个引导vo
checkRequire("Lua_GuideConditionVo");
Lua_GuideVo = class("Lua_GuideVo");
function Lua_GuideVo:ctor(id)
	self:superCall("ctor");
	self:reset(id);
end

function Lua_GuideVo.Get:nextGuideId()
	if self.config ~= nil and self.config.nextGuideId ~= nil then
		return self.config.nextGuideId;
	end
	return 0;
end

function Lua_GuideVo.Get:passStep()
	if self.config ~= nil and self.config.passStep ~= nil then
		return self.config.passStep;
	end
	return 0;
end

function Lua_GuideVo.Get:skipStep()
	if self.config ~= nil and self.config.skipStep ~= nil and self.config.skipStep > 0 then
		return self.config.skipStep;
	end

	-- 查找最后一步
	return DataCenter.guideUtil:getMaxStepByGuideId(self.guideId, 0);
end

function Lua_GuideVo:getSkipStep(startStep)
	if self.config ~= nil and self.config.skipStep ~= nil and self.config.skipStep > 0 then
		return self.config.skipStep;
	end

	-- 查找最后一步
	return DataCenter.guideUtil:getMaxStepByGuideId(self.guideId, startStep);
end

function Lua_GuideVo.Get:sort()
	if self.config ~= nil and self.config.sort ~= nil then
		return self.config.sort;
	end
	return 999999;
end

function Lua_GuideVo.Get:bShowMaskObj()
	if self.clickObjMaskType == -2 then
		self.clickObjMaskType = -1;
		if config == nil then
			return false;
		end

		local clickObjMaskType = config.clickObjMaskType;
		if clickObjMaskType == nil then
			return false;
		end

		if clickObjMaskType ~= Lua_GuideDefine.GuideWin_MaskObj_Circle 
			and clickObjMaskType ~= Lua_GuideDefine.GuideWin_MaskObj_Rectangle
			and clickObjMaskType ~= Lua_GuideDefine.GuideWin_MaskObj_CircleRectangle then
			return false;
		end

		local clickObjWin = config.clickObjWin;
		local clickObjPath = config.clickObjPath;
		local clickObjPos = config.clickObjPos;
		local clickObjSize = config.clickObjSize;
		if strIsNullOrEmpty(clickObjWin) 
			and strIsNullOrEmpty(clickObjPath) 
			and (clickObjSize == nil or #clickObjSize <= 0) then
			return false;
		end
		self.clickObjMaskType = clickObjMaskType;
	end
	return self.clickObjMaskType >= 0;
end

-- 重置数据
function Lua_GuideVo:reset(id)
	self.guideId = id;
	self.config = DataCenter.guideUtil:getGuideCfg(id);
	self.isCreateRoleGuide = false;
	self.clickObjMaskType = -2;
	if self._triggerConditionsYu ~= nil then
		self._triggerConditionsYu:clear();
		self._triggerConditionsYu = nil;
	end
	if self._triggerConditionsHuo ~= nil then
		self._triggerConditionsHuo:clear();
		self._triggerConditionsHuo = nil;
	end
end

-- 步骤
function Lua_GuideVo.Get:step()
	return DataCenter.guideData:getGuidePlayedStep(self.guideId);
end

-- 是否完成
function Lua_GuideVo.Get:isComplete()
	return self.passStep ~= -1 and self.step >= self.passStep;
end

function Lua_GuideVo.Get:isConditionEmpty()
	return self.triggerConditionsYu == nil and self.triggerConditionsHuo == nil;
end

-- 触发条件与
function Lua_GuideVo.Get:triggerConditionsYu()
	if self._triggerConditionsYu == nil then
		if self.config == nil or self.config.triggerList1 == nil or #self.config.triggerList1 <= 0 then
			return nil;
		end

		self._triggerConditionsYu = Array();
		local triggerList = self.config.triggerList1;
		for i = 1, #triggerList do
			local list = triggerList[i];
			local conditionVo = Lua_GuideConditionVo.new();
			conditionVo:setInfo(list, self.guideId);
			self._triggerConditionsYu:add(conditionVo);
			if conditionVo.conditionType == Lua_GuideDefine.Trigger_Condition_CreateRole then
				self.isCreateRoleGuide = true;
			end
		end
	end
	return self._triggerConditionsYu;
end

-- 触发条件或
function Lua_GuideVo.Get:triggerConditionsHuo()
	if self._triggerConditionsHuo == nil then
		if self.config == nil or self.config.triggerList2 == nil or #self.config.triggerList2 <= 0 then
			return nil;
		end

		self._triggerConditionsHuo = Array();
		local triggerList = self.config.triggerList2;
		for i = 1, #triggerList do
			local list = triggerList[i];
			local conditionVo = Lua_GuideConditionVo.new();
			conditionVo:setInfo(list, self.guideId);
			self._triggerConditionsHuo:add(conditionVo);
			if conditionVo.conditionType == Lua_GuideDefine.Trigger_Condition_CreateRole then
				self.isCreateRoleGuide = true;
			end
		end
	end
	return self._triggerConditionsHuo;
end

-- 检测触发
function Lua_GuideVo:checkTrigger(excludeConditionVo)
	local isAllEmpty = true;
	local arr = self.triggerConditionsYu; 
	if arr ~= nil then -- 全部条件成立才可以
		isAllEmpty = false;
		for i = 1, arr:getSize() do
			local conditionVo = arr[i];
			if excludeConditionVo == nil or excludeConditionVo ~= conditionVo then
				if not conditionVo.isComplete then
					return false;
				end
			end
		end
	end

	arr = self.triggerConditionsHuo;
	if arr ~= nil then -- 其中一个成立就可以
		isAllEmpty = false;
		for i = 1, arr:getSize() do
			local conditionVo = arr[i];
			if excludeConditionVo ~= nil and excludeConditionVo == conditionVo then
				return true;
			elseif conditionVo.isComplete then
				return true;
			end
		end
		return false;
	end

	local lastGuideId = DataCenter.guideData:getLastGuideId(self.guideId);
	-- print("checkTrigger", self.guideId, lastGuideId);
	if lastGuideId ~= nil and lastGuideId > 0 then -- 有上一个引导，则由上一个引导触发
		return true;
	end
	return not isAllEmpty; 
end