-- 引导
Lua_GuideUtil = class("Lua_GuideUtil");
function Lua_GuideUtil:ctor()
	self:superCall("ctor");
	
end

-- 引导配置
function Lua_GuideUtil.Get:guideConfig()
	if self._guideConfig == nil then
		self._guideConfig = getConfig("guide");
	end
	return self._guideConfig;
end

-- 引导数据
function Lua_GuideUtil:getGuideCfg(gid)
	if self.guideConfig == nil then
		return nil;
	end
	return self._guideConfig[gid];
end

function Lua_GuideUtil:getGuidePassStep(gid)
	local cfg = self:getGuideCfg(gid);
	if cfg == nil or cfg.passStep == nil then
		return self:getMaxStepByGuideId(gid, 1);
	end
	return cfg.passStep;
end

function Lua_GuideUtil:getGuideNextId(gid)
	local cfg = self:getGuideCfg(gid);
	if cfg == nil or cfg.nextGuideId == nil then
		return 0;
	end
	return cfg.nextGuideId;
end

-- 引导步骤配置
function Lua_GuideUtil.Get:guideStepConfig()
	if self._guideStepConfig == nil then
		self._guideStepConfig = getConfig("guide_step");
	end
	return self._guideStepConfig;
end

-- 引导步骤数据
function Lua_GuideUtil:getGuideStepCfgById(id)
	if self.guideStepConfig == nil then
		return nil;
	end
	return self._guideStepConfig[id];
end

-- 引导步骤数据
function Lua_GuideUtil:getGuideStepCfg(gid, step)
	local id = gid * 1000 + step;
	return self:getGuideStepCfgById(id);
end

function Lua_GuideUtil:getMaxStepByGuideId(gid, startStep)
	if startStep == nil or startStep <= 0 then startStep = 1; end
	local cfg = self:getGuideStepCfg(gid, startStep);
	while (cfg ~= nil) do
		startStep = startStep + 1;
		cfg = self:getGuideStepCfg(gid, startStep);
	end
	return startStep - 1;
end

-- 引导背景渐变时长
function Lua_GuideUtil.Get:maskAlphaTime()
	return 0.5;
end

-- 引导自动播放按钮旋转一圈时长
function Lua_GuideUtil.Get:autoBtnRotCircleTime()
	return 2;
end

--文本逐字播放加速时间
function Lua_GuideUtil.Get:textPlayeAccelerateTime()
	return 0.5;
end

-- 查找对象时间
function Lua_GuideUtil.Get:findObjLimitTime()
	return 2;
end

-- 是否是毛发模型
function Lua_GuideUtil:isFurModel(modelName)
	if self.furModelNameDic == nil then
		self.furModelNameDic = {};
		self.furModelNameDic["109000100_h"] = true;
		self.furModelNameDic["109000400_h"] = true;
		self.furModelNameDic["109000500_h"] = true;
		self.furModelNameDic["109000600_h"] = true;
	end
	return self.furModelNameDic[modelName];
end

-- 获取界面动画时间
function Lua_GuideUtil:getWinTweenTime(winName)
	if strIsNullOrEmpty(winName) then
		return 0;
	end
	
	if winName == "Lua_MainIconWindow" then
		return 0.5;
	elseif winName == "Lua_MainLineWindow" then
		return 0.5;
	end

	return DataCenter.winTweenManager:getWinTweenCountTime(winName, 0.5);
end

-- 获取场景对象路径
function Lua_GuideUtil:getSceneObjPath(sceneId, fightId, roleId)
	if sceneId == nil or sceneId <= 0 then sceneId = DataCenter.sceneData.curSceneId; end
	if fightId == nil or fightId <= 0 then fightId = DataCenter.fightData.curPlayFightRecordId; end
	if roleId == nil then
		if fightId > 0 then
			return "Thing/"..sceneId.."_"..fightId;
		else
			return "Thing/"..sceneId;
		end
	else
		if fightId > 0 then
			return "Thing/"..sceneId.."_"..fightId.."/"..roleId;
		else
			return "Thing/"..sceneId.."/"..roleId;
		end
	end
end

function Lua_GuideUtil.Get:guideStepSkipOverTime()
	if self._guideStepSkipOverTime == nil then
		self._guideStepSkipOverTime = getConfig("guide_const","guide_step_skip_overtime","value");
		if self._guideStepSkipOverTime == nil then
			self._guideStepSkipOverTime = 90000;
		end
		self._guideStepSkipOverTime = self._guideStepSkipOverTime / 1000;
	end
	return self._guideStepSkipOverTime;
end

-- 检测该时间内是否 查找到对象
function Lua_GuideUtil.Get:findGuideTargetTime()
	if self._findGuideTargetTime == nil then
		self._findGuideTargetTime = getConfig("guide_const", "guide_step_tolerate_findui_time", "value");
		if self._findGuideTargetTime == nil then
			self._findGuideTargetTime = 5000;
		end
		self._findGuideTargetTime = self._findGuideTargetTime / 1000;
	end
	return self._findGuideTargetTime;
end

-- 重新查找对象的次数
function Lua_GuideUtil.Get:findGuideTargetCount()
	if self._findGuideTargetCount == nil then
		self._findGuideTargetCount = getConfig("guide_const", "guide_step_tolerate_findui_count", "value");
		if self._findGuideTargetCount == nil then
			self._findGuideTargetCount = 2;
		end
	end
	return self._findGuideTargetCount;
end

function Lua_GuideUtil:getAssociationType(guideId)
	local cfg = self:getGuideCfg(guideId);
	if cfg ~= nil then
		return cfg.associationType;
	end
end

-- 自动播放类型
function Lua_GuideUtil:getAutoPlayTypeByGuideId(guideId)
	local cfg = self:getGuideCfg(guideId);
	if cfg == nil then
		return;
	end

	local type = cfg.type;
	return type == 5 and type or 1; -- 5：剧情    1：主线
end