-- 登录公告
Lua_AnnouncementWindow = class("Lua_AnnouncementWindow", Lua_Window)
function Lua_AnnouncementWindow:ctor()
    self:superCall("ctor");
    self.windowName = "Lua_AnnouncementWindow";
    self.skinName = "AnnouncementWindow_skin";
    self.showBlackMask = true;
    self.showClose = true;
    self.isClickClose = true;
end

function Lua_AnnouncementWindow:buildShowItem()
    self:superCall("buildShowItem");
end

function Lua_AnnouncementWindow:dispose()
    UploadManager.getInstance():clearRequest(UrlManager.prefixRequestReportUrl, self, nil);
    if self.dragContainer ~= nil then
        self.dragContainer:dispose();
        self.dragContainer = nil;
    end
    self:superCall("dispose");
end

function Lua_AnnouncementWindow:showWin(param)
    self:superCall("showWin", param);
    if param ~= nil then
        self:updateData(param);
        return;
    end

    local formData = {
        func = "notice",
        package = ServerConfig.market
    };
    local tokenstr = decodeTabToJson(formData);
    UploadManager.getInstance():uploadPhpPost(UrlManager.prefixRequestReportUrl, tokenstr, self, self.loadAnnouncementComplete, self.loadAnnouncementErr, true);
end

function Lua_AnnouncementWindow:loadAnnouncementComplete(text)
    if LanUtil.IsNullOrEmpty(text) then
        self:hide();
        return;
    end

    local obj = decodeJson(text);
    DataCenter.loginData.announcementObj = obj;
    if obj == nil then
        self:hide();
        return;
    end

    self:updateData(obj);
end

function Lua_AnnouncementWindow:loadAnnouncementErr(text)
    print("公告报错");
    self:hide();
end

function Lua_AnnouncementWindow:updateData(val)
    if val ~= nil then
        if strIsNullOrEmpty(val.title) then
            self.titleTxt:setEnable(false);
            self.leftSpr:setEnable(false);
            self.rightSpr:setEnable(false);
        else
            self.titleTxt:setEnable(true);
            self.titleTxt.text = val.title;
            self.titleTxt:resetSize();
            local posX = self.titleTxt.width / 2 + 20
            self.leftSpr:setEnable(true);
            self.leftSpr:setLocation(-posX, 409, 0);
            self.rightSpr:setEnable(true);
            self.rightSpr:setLocation(posX, 409, 0);
        end

        if not strIsNullOrEmpty(val.content) then
            if self.dragContainer == nil then
                self.dragContainer = Lua_DragContainer.new(self.content, 598, 800, Lua_AxisDefine.Vertical);
                self.dragContainer:setLocation(0, -45, 0);
                self.dragContainer:addSubContainer(Lua_AnnouncementItem, val, true);
            else
                self.dragContainer:setEnable(true);
                self.dragContainer:setChildData(val);
            end
            self:showNone(false);
        else
            self:showNone(true);
            if self.dragContainer then
                self.dragContainer:setEnable(false);
            end
        end
    else
        self:hide();
    end
end

function Lua_AnnouncementWindow:showNone(bShow)
    if bShow then
        if self.noneIcon == nil then
            self.noneIcon = UICreate.createUIAtlasImage(self.content, "login_atlas", "xui_common_widget_012", 0, 50, 0, 0);

            self.noneText = UICreate.createLabel(self.content, nil, 0, 19, 100, 31, 40);
            self.noneText.pivotValue = 4;
            self.noneText.fontName = "huakang_font";
            self.noneText.alignmentValue = 4;
            self.noneText.horizontalOverflowValue = 1;
            self.noneText.verticalOverflowValue = 1;
            self.noneText.colorStr = Lua_ColorDefine.transColorStr("938680");
            self.noneText:setLangValue(123733);
        end
        self.noneIcon:setEnable(true);
        self.noneText:setEnable(true);
    else
        if self.noneIcon then
            self.noneIcon:setEnable(false);
            self.noneText:setEnable(false);
        end
    end
end




















Lua_AnnouncementItem = class("Lua_AnnouncementItem", Lua_DragContainerItem)
function Lua_AnnouncementItem:ctor(sub)
    self:superCall("ctor", sub);
end

function Lua_AnnouncementItem:buildShowItem()
    self:superCall("buildShowItem");

    self.desTxt = UICreate.createLabel(self.content, nil, -299, 0, 598, 0, 24);
    self.desTxt.name = "desTxt";
    self.desTxt.anchorValue = 1;
    self.desTxt.pivotValue = 0;
    self.desTxt.verticalOverflowValue = 1;
    self.desTxt.colorStr = Lua_ColorDefine.transColorStr("726763");
    
    self.timeTxt = UICreate.createLabel(self.content, nil, 0, 0, 598, 0, 24);
    self.timeTxt.name = "timeTxt";
    self.timeTxt.anchorValue = 2;
    self.timeTxt.pivotValue = 2;
    self.timeTxt.alignmentValue = 8;
    self.timeTxt.verticalOverflowValue = 1;
    self.timeTxt.colorStr = Lua_ColorDefine.transColorStr("726763");
end

function Lua_AnnouncementItem:dispose()
    if self.desTxt ~= nil then
        self.desTxt:dispose();
        self.desTxt = nil;
    end
    if self.timeTxt ~= nil then
        self.timeTxt:dispose();
        self.timeTxt = nil;
    end
    if self.imgShowArr ~= nil then
        self.imgShowArr:dispose();
        self.imgShowArr:clear();
        self.imgShowArr = nil;
    end
    if self.imgHideArr ~= nil then
        self.imgHideArr:dispose();
        self.imgHideArr:clear();
        self.imgHideArr = nil;
    end
    self:superCall("dispose");
end

function Lua_AnnouncementItem:setData(data)
    local arr = self:splitStr(data.content, "<img>");
    local showStr = "";
    for i = 1, arr:getSize() do
        local tab = self:stringToTable(arr[i]);
        if tab ~= nil and type(tab) == "table" and tab.url ~= nil and tab.w ~= nil and tab.h ~= nil then
            showStr = spliceImgStr(tab.url, tab.w, tab.h);
        else
            showStr = showStr .. arr[i];
        end
    end
    showStr = ltrim(showStr);
    self.desTxt.text = string.gsub(showStr, "{space}", "\u{3000}");
    self.desTxt:resetSize()

    if data.update_time ~= nil then
        self.timeTxt:setLocation(0, -self.desTxt.height, 0);
        self.timeTxt.text = "\n"..Lua_TimeTransform.transSeconds25(data.update_time);
    end
    self.timeTxt:resetSize()
end

function Lua_AnnouncementItem:splitStr(str, reps)
    local arr = Array();
    local showStr = str;
    local len = string.len(showStr);
    local startIndex, endIndex = string.find(showStr, reps);
    while(startIndex ~= nil) do
        if startIndex ~= 1 then
            arr:add(string.sub(showStr, 1, startIndex - 1));
        end
        if endIndex == len then break end;

        showStr = string.sub(showStr, endIndex + 1);
        startIndex, endIndex = string.find(showStr, reps);
    end
    arr:add(showStr);
    return arr;
end

function Lua_AnnouncementItem:stringToTable(str)
    local func = loadstring("return " .. str);
    local tab = nil;
    if func ~= nil then
        tab = func();
    end
    return tab;
end

function Lua_AnnouncementItem:getWidth()
    return 598;
end

function Lua_AnnouncementItem:getHeight()
    return self.desTxt.height + self.timeTxt.height;
end