-- 登录选服界面
Lua_ServerSelectWindow = class("Lua_ServerSelectWindow", Lua_Window);
function Lua_ServerSelectWindow:ctor()
    self:superCall("ctor");
    
    self.windowName = "Lua_ServerSelectWindow";
    self.skinName = "ServerSelectWindow_skin";
    self.isFullSizeWin = true;
    self.openSound = nil;
    self.closeSound = nil;

    self.lastTime = -10;
    self._bLoadServerList = false;
    self.protocolNum = 0;
end

function Lua_ServerSelectWindow:dispose()
    UploadManager.getInstance():clearRequest(UrlManager.prefixRequestLoginStr, self, nil);
    if self.serverData ~= nil then
        self.serverData:clean();
        self.serverData = nil;
    end
     self:superCall("dispose");
end

function Lua_ServerSelectWindow:buildShowItem()
    self:superCall("buildShowItem");
    GlobalVar.isInGame = 0;

    self.protocolNum = PlayerPrefsUtil.instance:GetInt(SetUpDefine.PROTOCOL, 0);
    DataCenter.gameReported:loginReportedData(Lua_GameReported.REPORT_ENTER_SERVERSELECT_WIN);
    
    -- 用户协议
    if ServerConfig.isSecret then
        self.cbProtocol:setEnable(true);
        self.cbProtocol:addBoxChange(self, self.onChangeBox);--游戏协议
        self.cbProtocol.selected = self.protocolNum == 1;

        self.protocolTxt:setEnable(true);
        self.protocolTxt:setLangValue(123704);
        self.protocolTxt:resetSize();
        self.protocolTxt:addUrlClickFuntion(self, self.onProtocolClick);
    else
        self.cbProtocol:setEnable(false);
        self.protocolTxt:setEnable(false);
    end

    -- 显示著作权信息
    if ServerConfig.bShowZhuzuoInfo then
        self.tipsTtile:setLangValue(123702);
        self.tipsDes:setLangValue(123703);
    else--健康公告
        self.tipsTtile:setLangValue(123700);
        self.tipsDes:setLangValue(123701);
    end

    MsgUtil.callFunc("LOGIN_BG_CHANGE", "/images/loading/login_bg.jpg");

    self.btnBack:setEnable(false);
    if Lua_PlatformUtil.IsNoSdk() then
        self.btnBack:setEnable(true);
        self.btnBack:addClickFuntion(self, self.onBackClick);
    end
    self.annIcon:addClickFuntion(self, self.onAnnClick);--公告
    self.btnEnter:addClickFuntion(self, self.onEnterClick);--进入游戏
    self.repairIcon:addClickFuntion(self, self.onRepairClick);--修复工具处理
    self.serverNameBg:addClickFuntion(self, self.onServerNameClick);--服务器选择

    self:addCSharpEvent("SERVER_SELECT_BACK", self, self.onServerSelectBack);
    self:addLuaEvent("SELECTE_SERVER", self, self.updateSelectedServer);--更新选择的服务器

    Lua_WindowUtil.openWin("Lua_LoadingScreenWindow", Lua_WinParam.ACTION_CLOSE);
    Lua_WindowUtil.openWin("Lua_StartWindow", Lua_WinParam.ACTION_CLOSE);
    Lua_WindowUtil.openWin("Lua_LoginWindow", Lua_WinParam.ACTION_CLOSE);
    -- Lua_WindowUtil.openWin("Lua_LoginPhoneWindow", Lua_WinParam.ACTION_CLOSE);

    -- 获取服务器列表
    if Lua_WindowUtil.getWinListData(self.windowName, false) == nil then
    	Lua_MsgUtil.callFunc("LOGIN_BACK_SEVER_SELECTED");
    end
end

function Lua_ServerSelectWindow:showWin(param)
    self:superCall("showWin", param);

    if param ~= nil then
        self._bLoadServerList = true;
        if self.serverData ~= nil then
            self.serverData:clean();
            self.serverData = nil;
        end

        self.serverData = param;
        self:updateSelectedServer(self.serverData:getDefaultServer());
        self:loadAnnouncement(); -- 有公告则显示公告
    elseif not self._bLoadServerList then
        Lua_MsgUtil.callFunc("LOGIN_BACK_SEVER_SELECTED");
    end
end

function Lua_ServerSelectWindow:onChangeBox(val)
    self.cbProtocol.selected = val;
    self.protocolNum = val and 1 or 0;
    PlayerPrefsUtil.instance:SetInt(SetUpDefine.PROTOCOL, self.protocolNum);
end

function Lua_ServerSelectWindow:onProtocolClick(urlStr)
    if LanUtil.IsNullOrEmpty(urlStr) then
        return;
    end

    local t = getTime();
    if t - self.lastTime < 1 then
        return;
    end
    self.lastTime = t;

    print(urlStr)
    if urlStr == "user_protocol" then
        if LanUtil.IsNullOrEmpty(ServerConfig.protocolUrl) then
            Lua_WindowUtil.openWin("Lua_UserProtocolWindow", Lua_WinParam.ACTION_OPEN, urlStr);
        else
            if Lua_PlatformUtil.IsAndroid() or Lua_PlatformUtil.IsIPhonePlayer() then
                local param = { title = LanUtil.getValue(123708), url = ServerConfig.protocolUrl };
                param = decodeTabToJson(param);
                print("webActivity jsonStr = ", param);
                DataCenter.sdkInterface:callSDKInterface(Lua_SDKMessageType.WebActivity, param);
            else
                print("OpenURL: ", ServerConfig.protocolUrl);
                UnityEngine.Application.OpenURL(ServerConfig.protocolUrl);
            end
        end
    elseif urlStr == "user_secret" then
        if LanUtil.IsNullOrEmpty(ServerConfig.privacyUrl) then
            Lua_WindowUtil.openWin("Lua_UserProtocolWindow", Lua_WinParam.ACTION_OPEN, urlStr);
        else
            if Lua_PlatformUtil.IsAndroid() or Lua_PlatformUtil.IsIPhonePlayer() then
                local param = { title = LanUtil.getValue(123709), url = ServerConfig.privacyUrl };
                param = decodeTabToJson(param);
                print("webActivity jsonStr = ", param);
                DataCenter.sdkInterface:callSDKInterface(Lua_SDKMessageType.WebActivity, param);
            else
                print("OpenURL: ", ServerConfig.protocolUrl);
                UnityEngine.Application.OpenURL(ServerConfig.privacyUrl);
            end
        end
    end
end

function Lua_ServerSelectWindow:loadAnnouncement()
    if PlayerData.instance.phpData.bTishenServer or DataCenter.loginData.bLocalServer then
        return;
    end
    Lua_MsgUtil.callFunc("GetAnnouncementServerInfo", false);
end

-- 更新选择的服务器
function Lua_ServerSelectWindow:updateSelectedServer(data)
	-- print("Lua_ServerSelectWindow updateSelectedServer", data == nil);
    if data == nil then
        self.signIcon:setEnable(false);
        self.stateIcon:setEnable(false);
        self.stateTxt.text = nil;
        self.serverNameTxt.text = nil;
    else
        if data.ad_status == 1 or data.ad_status == 2 then--新区/推荐
            self.signIcon:setEnable(true);
        else
            self.signIcon:setEnable(false);
        end
        self.stateIcon:setEnable(true);
        self.stateIcon:setAltas("login_atlas", data.stateIcon);

        self.stateTxt:setLangValue(data.stateLangId);
        if DataCenter.loginData.bLocalServer then
            self.serverNameTxt.text = data.server_name;
        else
            self.serverNameTxt.text = data.name;
        end
        self.serverNameTxt:resetSize()
        local posX = -147 + self.serverNameTxt.width + 53;
        posX = posX < 46 and 46 or posX;
        self.stateTxt:setLocation(posX, 290, 0)
    end
    self.serverData.selectServerData = data;
end

function Lua_ServerSelectWindow:onServerSelectBack()
    self:onBackClick();
end

function Lua_ServerSelectWindow:onEnterClick()
    if self.serverData == nil then
        return;
    end

    DataCenter.gameReported:loginReportedData(Lua_GameReported.REPORT_CLICK_SERVERSELECTWIN_ENTERBTN);
    if self.protocolNum == 0 and ServerConfig.isSecret then
        OperateAlert.showLangValue(123710);
        return;
    end

    local t = getTime();
    if t - self.lastTime < 1 then
        OperateAlert.showLangValue(123711);
        return;
    end
    self.lastTime = t;

    if self.serverData.isBan then
        OperateAlert.showLangValue(123712);
        return;
    end

    local selectedServer = self.serverData.selectServerData;
    if selectedServer == nil then
        OperateAlert.showLangValue(123713);
        return;
    end

    if selectedServer.is_create_role == 0 and selectedServer.is_account_create == 0 then --账号创建角色达到上限
        OperateAlert.showLangValue(123714);
        return;
    end

    if selectedServer.is_create == 0 then
        OperateAlert.showLangValue(123715);
        return;
    end

    if selectedServer.status == -1 then
        local str = LanUtil.getValue(123716);
        if not strIsNullOrEmpty(selectedServer.text) then
            str = selectedServer.text;
        end

        local tipsData = {};
		tipsData.des = str; --文本描述
		tipsData.btnType = 4;
		tipsData.sureLabel = LanUtil.getValue(123717);
		Lua_WindowUtil.openWin("Lua_LoginTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
        OperateAlert.showLangValue(123718);
        return;
    elseif selectedServer.status == 0 then
    	local tipsData = {};
		tipsData.des = selectedServer.text; --文本描述
		tipsData.btnType = 4;
		tipsData.sureLabel = LanUtil.getValue(123717);
		Lua_WindowUtil.openWin("Lua_LoginTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
        return;
    end

    PlayerPrefsUtil.instance:SetString(SetUpDefine.SELECT_SERVER, selectedServer.flag);
    local max = selectedServer.max_online;
    if max ~= nil and selectedServer.online >= max and max ~= 0 then
    	local tipsData = {};
		tipsData.des = LanUtil.getValue(123719); --文本描述
		tipsData.btnType = 4;
		tipsData.sureLabel = LanUtil.getValue(123717);
		Lua_WindowUtil.openWin("Lua_LoginTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
        return;
    end

    Lua_MsgUtil.callFunc("SERVER_VALISOCKET_CONNECT", selectedServer);
end

function Lua_ServerSelectWindow:onServerNameClick(go)
    if self.serverData == nil then
        return;
    end
    Lua_WindowUtil.openWin("Lua_ServerListWindow", Lua_WinParam.ACTION_OPEN, self.serverData);
end

function Lua_ServerSelectWindow:onAnnClick(go)
    Lua_MsgUtil.callFunc("GetAnnouncementServerInfo", true);
end

function Lua_ServerSelectWindow:onBackClick(go)
    DataCenter.connectionManager:relogin();
end

function Lua_ServerSelectWindow:onRepairClick(go)
    --修复工具处理
    local tipsData = {};
    tipsData.des = LanUtil.getValue(380029); --文本描述
    tipsData.btnType = 1;
    tipsData.owner = self;
    tipsData.sureFun = self.onRepairSure;
    Lua_WindowUtil.openWin("Lua_LoginTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
end

function Lua_ServerSelectWindow:onRepairSure()
    LuaUtil.repairUpdateData();
end