-- 登录数据
Lua_LoginData = class("Lua_LoginData");
function Lua_LoginData:ctor()
	self:superCall("ctor");
	self:init();
end

function Lua_LoginData:init()
    self.isCreateRole = false; -- 是否已创角
    self.bAutoCreateRole = false; -- 是否自动创建
    self.bLoadingVideoType = false; -- 是否视频加载进度
    self.bShowVideoFrame = true; -- 如果是创角，是否显示视频第一帧
    self.bPreLoadComplete = false; -- 预加载完成
    self.openServerDays = 0; -- 开服天数
    self.createRoleDays = 0; -- 创号天数
    self.isEnterGameWorld = false;
    self.tweeningState = false; 
    self.scrollRectMovingCount = 0;
    self.hasSendHeartMsg = false;

    self.waitShowScene = nil;
    self.waitInitedWinName = nil;
    self.bStartLoadEnterGame = false;
    self.isUpdatePlayerInfo = false;
    self.videoSign = nil;
    self.operatingClientDataTab = {};
    -- self.videoSign = "LOGINLOADING";

    if self.protoMsgArr == nil then
        self.protoMsgDic = {};
        self.protoMsgArr = Array();
    else
        local len = self.protoMsgArr:getSize();
        while (len > 0) do
            local msgId = self.protoMsgArr[len];
            self.protoMsgDic[msgId] = nil;
            self.protoMsgArr:remove(len);
            len = len - 1;
        end
    end
end

function Lua_LoginData:parseProtoJson(jsonStr)
    if not strIsNullOrEmpty(jsonStr) then
        local jsonObj = decodeJson(jsonStr);
        if jsonObj ~= nil then
            for k, v in pairs(jsonObj) do
                local msgId = v.mid;
                self.protoMsgDic[msgId] = {index = k, msgId = v.mid, desc = v.mname, isProto = v.syn == 1};
                self.protoMsgArr:add(msgId);
            end
        end
    end
end

function Lua_LoginData:getProtoInfo(msgId)
    return self.protoMsgDic[msgId];
end

function Lua_LoginData.Get:globalClientData()
    if self._globalClientData == nil then
        self._globalClientData = PlayerPrefsUtil.instance:GetString("client_develop_debug", "");
    end
    return self._globalClientData;
end

function Lua_LoginData.Set:globalClientData(val)
    if self._globalClientData ~= val then
        self._globalClientData = val;
        PlayerPrefsUtil.instance:SetString("client_develop_debug", val);
    end
end

function Lua_LoginData:dispose()
    self._appUpdateUrl = nil;
    self._checkAppVerison = nil;
    self:clearHeartCollectTimer();
    self:clearChcekVideo();
    self:clearHeart()
    self:init()
end

-- 检测角色名是否无效
function Lua_LoginData:checkRoleNameIsInvalid(str)
    local nameLen = stringWidth(str)
	if nameLen <= 0 then
		OperateAlert.showLangValue(123724);
		return true;
	end

    local nameLimit = Lua_CommonUtil.getCommonConfig("name_character_limit", 3)
    if nameLen < nameLimit[1] / 2 then
        OperateAlert.showLangValue(123731);
        return true
    end

	if nameLen > nameLimit[2] / 2 then
		OperateAlert.showLangValue(123725);
		return true;
	end

	-- 包含敏感字或空格
	if FilterText.GetInstance():IsInvalid(str, true, 0) or FilterText.GetInstance():IsInvalid(str, true, 1) then
		OperateAlert.showLangValue(123726);
		return true;
	end

	local si, ei = string.find(str, "\\");
	if si ~= nil and si > 0 then
		OperateAlert.showLangValue(123726);
		return true;
	end

	-- 数字或者字母太多
	if LanUtil.checkStrNumberCount(str) >= 7 or LanUtil.checkStrEngAlphabetCount(str) >= 5 then
		OperateAlert.showLangValue(123727);
		return true;
	end

	return false;
end

function Lua_LoginData:onInit()
	CameraControl.Instance:init();
    if Lua_PlatformUtil.IsWindowsEditor() then
    	LuaUtil.showFps(true);
    end
end

-- 心跳
function Lua_LoginData:addHeart()
    if self.bAddHeartTimer ~= true then
        self.bAddHeartTimer = true;
        Lua_CommonUtil.addTimer("onHeart", 5, self, self.onHeart);
    end
end

function Lua_LoginData:onHeart(dt)
    -- local con = DataCenter.connectionManager:getConnection(Lua_Connection.MAIN_SOCKET);
    -- if con == nil then
    --     print("心跳 主socket null", self.hasSendHeartMsg);
    -- else
    --     print("心跳 主socket状态", con.isConnecting, con.available, con.addressFamily, con.blocking, con.sendTimeout, con.noDelay, self.hasSendHeartMsg);
    -- end

    if self.hasSendHeartMsg and GlobalVar.bHandleServerMsg then  -- 上次心跳还未返回
        self:clearHeartCollectTimer(); -- 之所以增加个1s后处理，是为了后台切换给1s的更新间隔，防止切后台频繁断线重连
        self._bAddHeartCollectTimer = true;
        Lua_CommonUtil.delayCall(1, self, self.onCheckHeartState); -- 后台切换下一次判断
        return;
    end

    self.hasSendHeartMsg = true;
    sendMsg(10100006);
end

function Lua_LoginData:onCheckHeartState()
    if self.hasSendHeartMsg and GlobalVar.bHandleServerMsg then  -- 上次心跳还未返回
        -- print("上次心跳还未返回，onCheckHeartState");
        GlobalVar.canReconnect = true;
        DataCenter.connectionManager:errorAlert(LanUtil.getValue(10003), true);
    end
end

function Lua_LoginData:clearHeartCollectTimer()
    if self._bAddHeartCollectTimer then
        self._bAddHeartCollectTimer = false;
        Lua_CommonUtil.removeTimer(self, self.onCheckHeartState, false, true);
    end
end

function Lua_LoginData:clearHeart()
    if self.bAddHeartTimer == true then
        self.bAddHeartTimer = false;
        Lua_CommonUtil.removeTimer(self, self.onHeart);
    end
end

-- 游戏后台状态切换
function Lua_LoginData:updateRunStateChange(stateVal)
    if stateVal == 0 then 
        self.hasSendHeartMsg = false; -- 移除这次心跳判断断线
        self:clearHeartCollectTimer();          
        if not DataCenter.connectionManager:isConnection(Lua_Connection.MAIN_SOCKET) then
            GlobalVar.canReconnect = true;
            DataCenter.connectionManager:errorAlert(LanUtil.getValue(10003), true);
            print("断线重连 updateRunStateChange");
        end
    end
end

function Lua_LoginData:onServerUpdateEnd()
    if self.bLoadingVideoType then -- 未创角，播放动画进入场景
        self.bLoadingVideoType = false;
        self.videoSign = "LOGINLOADING";
        DataCenter.connectionManager:setConnectionHandleMaxNum(Lua_Connection.MAIN_SOCKET, 1);
        local showSkipBtnTime = Lua_CommonUtil.getCommonConfig("create_role_skip_time", 1);
        LuaUtil.showVideoLoadingScreenWin(self.videoSign, nil, "/video/loginvideo3.mp4", -1, -1, -1, -1, 5, 1, -1, true, true, showSkipBtnTime);
        showSkipBtnTime = showSkipBtnTime / 2;
        showSkipBtnTime = showSkipBtnTime < 5 and 5 or showSkipBtnTime;
        Lua_CommonUtil.delayCall(showSkipBtnTime, self, self.onCheckVideo);
        if DataCenter.mainData ~= nil then -- 提前初始化了
            self:parseServerPlayerInfoMsg();
        end
    else
        self:loadMainSec();
    end
end

function Lua_LoginData:clearChcekVideo()
    Lua_CommonUtil.removeTimer(self, self.onCheckVideo);
end

function Lua_LoginData:onCheckVideo()
    if not self.bShowVideoFrame then -- 第一帧还没显示
        Lua_WindowUtil.openWin("VideoLoadingScreenWindow", Lua_WinParam.ACTION_CLOSE);
        self.bShowVideoFrame = true;
        self.videoSign = nil;
        self:loadMainSec();
    end
end

function Lua_LoginData:loadMainSec()
    LuaManager.getInstance():loadLuaModule("main_sec", self, self.onMainSecLoaded);
end

function Lua_LoginData:onMainSecLoaded()
    if DataCenter ~= nil and DataCenter.mainData == nil then
        local data = checkRequire("Lua_MainData");
        DataCenter.mainData = data.new();
    end

    self:parseServerPlayerInfoMsg();
    self:onInit();
    self:loadSecConfig();
end

function Lua_LoginData:updatePlayerInfo(data, isReconnect)
    if data == nil then
        return;
    end

    self.serverPlayerInfoMsg = data;
    if isReconnect then
        self:parseServerPlayerInfoMsg();
        self:onInit();
        self:onPreLoad();
        -- self:onEnteredInit();
        return;
    end

    if self.isUpdatePlayerInfo ~= true then
        self.isUpdatePlayerInfo = true;
        self.bStartLoadEnterGame = true;
        
        GlobalVar.bHandleServerMsg = false;
        GlobalVar.bShowLoadingWinBg = true;
        if LuaUtil.checkServerUpdate() then
            self:onServerUpdateEnd();
        end
    else
        self:parseServerPlayerInfoMsg();
    end
end

function Lua_LoginData:parseServerPlayerInfoMsg()
    if self.serverPlayerInfoMsg == nil then
        return;
    end

    local msg = ProtoPublic_pb.Proto80101001();
    msg:ParseFromString(self.serverPlayerInfoMsg);
    self.serverPlayerInfoMsg = nil;

    local t = tonumber(tostring(msg.time));
    DataCenter.roleData.serverTime = t;
    GlobalVar.cur_tick = msg.time;
    local roleId = tonumber(tostring(msg.show.id));
    GlobalVar.roleId = roleId;

    self.openServerDays = msg.open_days; -- 开服天数
    self.createRoleDays = msg.create_days; -- 创号天数
    local data = PlayerData.instance.data;
    data.exp = tonumber(tostring(msg.exp));
    local show = msg.show;
    data.roleId = roleId;
    data.nickname = show.name;
    data.gender = show.sex;
    data.career = show.career;
    data.lvl = show.level;
    data.vipLv = show.vip;
    data.fighting = show.force;
    data.area = show.area;
    data.agentCode = show.agent_code;
    data.serverFlag = show.server_flag;
    data.serverName = show.server_name;
    data.serverPrev = show.server_prev;
    data.league_id = show.league_id;
    data.league_pos = show.league_position;
    data.league_name = show.league_name;
    LightingMapManager.instance:init();
    local buglyUserId = show.server_name .. "_" .. roleId;
    DataCenter.sdkInterface:callBuglyInterface(Lua_SDKMessageType.bugly_setUserId, buglyUserId);

    -- 用于sdk关闭引导
    if DataCenter.guideData.bOpenGuide then
        local isStopGuide = LuaUtil.getSceneDataExtendVal("isStopGuide");
        if isStopGuide then DataCenter.guideData.bOpenGuide = false; end
    end

    -- lua部分更新
    DataCenter.mainLineData.checkpointId = msg.stage_id;
    Lua_ProxyManager.getProxy("Lua_RoleProxy"):updateRoleInfo(msg);-- 更新角色信息
end

-- 本地进入游戏场景测试，构造服务端数据
function Lua_LoginData:enterLocalGame()

end

function Lua_LoginData:loadSecConfig()
    ConfigManager.getInstance():loadOtherConfig();
    -- Lua_WindowUtil.openWin("Lua_CreateRoleWindow", Lua_WinParam.ACTION_CLOSE);
    -- Lua_WindowUtil.openWin("Lua_CreateRoleAniShowWindow", Lua_WinParam.ACTION_CLOSE);
    -- Lua_WindowUtil.openWin("Lua_StartWindow", Lua_WinParam.ACTION_CLOSE);
    -- Lua_WindowUtil.openWin("Lua_LoginPhoneWindow", Lua_WinParam.ACTION_CLOSE);
    -- Lua_WindowUtil.openWin("Lua_LoginWindow", Lua_WinParam.ACTION_CLOSE);
    -- Lua_WindowUtil.openWin("Lua_ServerListWindow", Lua_WinParam.ACTION_CLOSE);
    -- Lua_WindowUtil.openWin("Lua_ServerSelectWindow", Lua_WinParam.ACTION_CLOSE);
    -- Lua_WindowUtil.openWin("LoginBgWindow", Lua_WinParam.ACTION_CLOSE);
end

-- 登录预加载资源
function Lua_LoginData:preLoadGameRes()
    self.bPreLoadComplete = false;
    DataCenter.preLoadData:enterGamePreLoad(self, self.onPreLoad);
end

function Lua_LoginData:onPreLoad()
    if self.videoSign == nil then
        checkRequire("tb_monster_player");
        checkRequire("tb_monster_player_t1");
        checkRequire("tb_monster");
        checkRequire("tb_monster_t1");
    end

    self.bPreLoadComplete = true;
    self:checkEnterGame();
end

function Lua_LoginData:checkEnterGame()
    print("preload complete", self.isCreateRole, self.bPreLoadComplete, self.bShowVideoFrame, self.isEnterGameWorld);
    if not self.bShowVideoFrame or not self.bPreLoadComplete then
        return;
    end

    DataCenter.guideData:checkPlayGuide();
    if not self.isCreateRole then -- 未触发引导及未创角则自动打开创角界面
        local guideVo = DataCenter.guideData.curGuideVo;
        if guideVo == nil or not guideVo.isCreateRoleGuide then -- 非创角引导
            Lua_WindowUtil.openWin("Lua_CreateRoleWindow", WinParam.ACTION_OPEN);
        end
    end

    local iconId = Lua_MenuViewDefine.ICON_MAINCITY;
    local iconVo = DataCenter.mainMenuData:getMenuIconVo(iconId);
    if iconVo ~= nil then
        if iconVo.operateType == 1 then -- 打开窗口
            if (strIsNullOrEmpty(iconVo.operateParam) or Lua_WindowUtil.isWinInitComplete(iconVo.operateParam)) then -- 初始完成
                self:completeEnterGame();
            else
                self.waitInitedWinName = iconVo.operateParam;
            end
        elseif iconVo.operateType == 3 then -- 打开场景
            local sceneId = iconVo.sceneId;
            if iconVo.iconId == Lua_MenuViewDefine.ICON_MAINLINE then
                sceneId = DataCenter.mainLineData.curSceneId;
            end

            if sceneId <= 0 or Scene3DManager.instance:isShowingScene(sceneId, 0) then -- 已打开
                self:completeEnterGame();
            else
                self.waitShowScene = sceneId;
            end
        else -- 暂时
            self:completeEnterGame();
        end

        DataCenter.mainMenuData:clickIconEvent(iconVo, false);
    else
        self:completeEnterGame();
    end
end

-- 进入游戏
function Lua_LoginData:completeEnterGame()
    Lua_WindowUtil.openWin("LoadingScreenWindow", Lua_WinParam.ACTION_CLOSE);
    DataCenter.mainData:onFirstEnterGame(self.videoSign ~= nil);
    self.waitShowScene = nil;
    self.waitInitedWinName = nil;
    print("completeEnterGame");
    if self.videoSign ~= nil then -- 视频播放
        self.isEnterGameWorld = self.isCreateRole;
        MsgUtil.callFunc(EventsDefine.VIDEOLODING_SHOW_SKIP, self.videoSign);
    else -- 非视频播放进入
        self.isEnterGameWorld = true;
        self:onEnteredInit();
    end
end

-- 创角成功或加载窗口关闭后初始化
function Lua_LoginData:onEnteredInit()
    Lua_MsgUtil.callFunc("MAIN_ICON_SELECT", Lua_MenuViewDefine.ICON_MAINCITY);
    DataCenter.gameReported:loginReportedData(Lua_GameReported.REPORT_ENTER_GAME_SCENE);
    DataCenter.connectionManager:setConnectionHandleMaxNum(Lua_Connection.MAIN_SOCKET, 30);
    DataCenter.chatData:playEnterBroadcast();
    if not DataCenter.guideData.isGuiding then
        DataCenter.popupWinManager:checkPopupWin();
    end

    -- 执行一次完整GC
    DataCenter.connectionManager:setCollectInfo(100);
    LoadManager.getInstance():clearResource();
end

function Lua_LoginData.Get:isInGameWorld()
    return self.isEnterGameWorld and self.isCreateRole;
end

function Lua_LoginData:updateTweenState(state)
   self.tweeningState = state; 
   self:checkFpsState();
end

function Lua_LoginData:updateScrollRectMoveState(state)
    if state then
        self.scrollRectMovingCount = self.scrollRectMovingCount + 1;
    else
        self.scrollRectMovingCount = self.scrollRectMovingCount - 1;
        if self.scrollRectMovingCount < 0 then
            self.scrollRectMovingCount = 0;
        end
    end
    self:checkFpsState();
end

function Lua_LoginData:checkFpsState()
    if not DataCenter.bConfigLoaded then
        return;
    end

    local fps = self.normalFps;
    if self.tweeningState then
        fps = self.hightFps;
    elseif self.scrollRectMovingCount ~= nil and self.scrollRectMovingCount > 0 then
        fps = self.hightFps;
    elseif DataCenter.fightData ~= nil and DataCenter.fightData.isFighting then
        fps = self.hightFps;
    elseif DataCenter.mainLineCamActController ~= nil and not DataCenter.mainLineCamActController.bActComplete then
        fps = self.hightFps;
    elseif DataCenter.crackShotData.isCrackShotTween then
        fps = self.hightFps;
    end
    MsgUtil.callFunc("FPS_UPDATE", fps);
end

function Lua_LoginData.Get:normalFps()
    if self._normalFps == nil then
        self._normalFps = Lua_CommonUtil.getCommonConfig("normalFps", 1, 30);
    end
    return self._normalFps;
    
end

function Lua_LoginData.Get:hightFps()
    if self._hightFps == nil then
        self._hightFps = Lua_CommonUtil.getCommonConfig("hightFps", 1, 60);
    end
    return self._hightFps;
end

function Lua_LoginData.Get:checkAppVerison()
    if self._checkAppVerison == nil then
        self._checkAppVerison = PlayerData.instance:getPhpConfigNumberValue("phpClientInfo", "AppVersion");
    end
    return self._checkAppVerison;
end

function Lua_LoginData.Get:appUpdateUrl()
    if self._appUpdateUrl == nil then
        self._appUpdateUrl = PlayerData.instance:getPhpConfigStringValue("phpClientInfo", "AppUrl");
    end
    return self._appUpdateUrl;
end

-- 检测强更app
function Lua_LoginData:checkUpdateApp(unShowTips)
    if not Lua_PlatformUtil.IsAndroid() then
        return false;
    end

    local checkAppVerison = self.checkAppVerison;
    print("AppVersion", checkAppVerison);
    if checkAppVerison == nil then
        return false;
    end

    checkAppVerison = tonumber(tostring(checkAppVerison))
    if checkAppVerison <= 0 then
        return false;
    end

    local version = tonumber(tostring(PragramVersion.PROGRAM_VERSION));
    print("checkUpdateApp", version, checkAppVerison);
    if version < checkAppVerison then -- 强更
        local url = self.appUpdateUrl; --"/小浣熊-10001_v1.0.1.1_(0331).apk";
        print("url = ", url);
        if strIsNullOrEmpty(url) then
            return false;
        end

        if unShowTips then
            return true;
        end

        local cancelLabel = LanUtil.getValue(10065);
        if Lua_PlatformUtil.IsNoSdk() or strIsNullOrEmpty(cancelLabel) or tostring(cancelLabel) == "10065" then
            local tipsData = {};
            tipsData.des = LanUtil.getValue(10063); --文本描述
            tipsData.btnType = 4;
            tipsData.sureLabel = LanUtil.getValue(10064);
            tipsData.owner = self;
            tipsData.sureFun = self.onUpdateApp;
            tipsData.url = url;
            Lua_WindowUtil.openWin("Lua_LoginTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
        else
            local tipsData = {};
            tipsData.des = LanUtil.getValue(10063); --文本描述
            tipsData.btnType = 6;
            tipsData.sureLabel = LanUtil.getValue(10064);
            tipsData.cancelLabel = cancelLabel;
            tipsData.owner = self;
            tipsData.sureFun = self.onUpdateApp;
            tipsData.cancelFun = self.onOpenSdkLogin;
            tipsData.url = url;
            Lua_WindowUtil.openWin("Lua_LoginTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
        end
        return true;
    end
    return false;
end

function Lua_LoginData:onUpdateApp(showData)
    if showData == nil then return; end
    local url = showData.url;
    print("onUpdateApp", url);
    if strIsNullOrEmpty(url) then return; end
    UnityEngine.Application.Quit();
    UnityEngine.Application.OpenURL(url);
end

function Lua_LoginData:onOpenSdkLogin()
    if Lua_PlatformUtil.IsAndroid() or Lua_PlatformUtil.IsIPhonePlayer() then
        DataCenter.sdkInterface:callSDKInterface(Lua_SDKMessageType.Login);
    end
end