-- 登录服务器列表数据
checkRequire("Lua_ServerButtinListData");
Lua_ServerListData = class("Lua_ServerListData");
function Lua_ServerListData:ctor()
	self:superCall("ctor");
    self.isBan = false;
    self.allZoneList = Array();
    self.allServerDic = {};
    self.myServerList = Array();
    self.recommendServerList = Array();
    self.selectServerData = nil;
    self.localServerList = nil;
end

function Lua_ServerListData:fromServer(list)
    if list == nil or #list <= 0 then
        return;
    end
    self.localServerList = list;
end

function Lua_ServerListData:popErrorTipsWin()
    -- print("服务器列表解析失败");
    local tipsData = {};
	tipsData.des = LangDefine.getValue(16); --文本描述
	tipsData.btnType = 4;
	tipsData.sureLabel = LangDefine.getValue(1);
	Lua_WindowUtil.openWin("Lua_LoginTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
end

function Lua_ServerListData:readFromJson(json, bShowTips)
	if bShowTips == nil then bShowTips = true; end
	if json == nil or json.zone == nil or json.server == nil then
        if bShowTips then self:popErrorTipsWin(); end
        return false;
    end

    local zones = json.zone;
    local server = json.server;
    -- print("server len = ", #server);
    for k = 1, #server do
        local zld = Lua_OneServerData.new(server[k]);
        local str = tostring(zld.zone);
        zld.zoneName = tostring(zones[str]);
        if self.allServerDic[str] == nil then
            self.allServerDic[str] = Array();
        end
        self.allServerDic[str]:add(zld);

        -- 已有角色服务器
        if zld.roleList ~= nil and zld.roleList:getSize() > 0 then
            self.myServerList:add(zld);
        elseif zld.status == 2 then
            -- 流畅且没有角色，加入推荐列表
            self.recommendServerList:add(zld);
        end
    end

    -- 推荐服务器和我的服务器修改为常驻显示
    if self.recommendServerList ~= nil and self.recommendServerList:getSize() > 0 then
        local sbld= Lua_ServerButtinListData.new();
        sbld.name = LanUtil.getValue(123629);
        sbld.zone = -1;
        self.allZoneList:add(sbld);
    end

    -- 我的服务器
    sbld = Lua_ServerButtinListData.new();
    sbld.name = LanUtil.getValue(123608);
    sbld.zone = 0;
    self.allZoneList:add(sbld);
    
    for k, v in pairs(zones) do
    	sbld = Lua_ServerButtinListData.new();
        sbld.name = tostring(v);
        sbld.zone = tonumber(tostring(k));
        self.allZoneList:add(sbld);
    end
    self.allZoneList:sort("zone", false);
    
    local users = json.user;
    ServerConfig.userID = users.openId;
    PlayerPrefsUtil.instance:SetString(SetUpDefine.USERID, ServerConfig.userID);

    if users.package ~= nil then
    	ServerConfig.market = tonumber(tostring(users.package));
        ServerConfig.ChannelID = tostring(users.package);
    end

    if users.channel ~= nil then
        ServerConfig.channel = tostring(users.channel);
    end

    self.isBan = false;
    if users.is_ban ~= nil then -- is_ban=1就是被封
        if tonumber(tostring(users.is_ban)) == 1 then
            self.isBan = true;
        end
    end

    -- pc相关
    local recordSign = PlayerPrefsUtil.instance:GetInt(SetUpDefine.RECORD_PHONE_SIGN, 1);
    if recordSign > 0 and json.pc_param ~= nil then
        self:parsePcParam(json.pc_param);
    end

    return true;
end

-- 解析pc相关
function Lua_ServerListData:parsePcParam(dic)
    local expiresTime = nil;
    local ticket = nil;
    if dic ~= nil then
    	if dic.expires_time ~= nil then
            expiresTime = tostring(dic.expires_time);
        end

        if dic.ticket ~= nil then
            ticket = tostring(dic.ticket);
        end
    end

    if LanUtil.IsNullOrEmpty(expiresTime) == false then
        print("记录时间戳：", expiresTime);
        PlayerPrefsUtil.instance:SetString(SetUpDefine.WEB_EXPIRES_TIME_STR, expiresTime);
    else
        PlayerPrefsUtil.instance:DeleteKey(SetUpDefine.WEB_EXPIRES_TIME_STR);
    end

    if LanUtil.IsNullOrEmpty(ticket) == false then
        PlayerPrefsUtil.instance:SetString(SetUpDefine.WEB_TICKET_STR, ticket);
    else
        PlayerPrefsUtil.instance:DeleteKey(SetUpDefine.WEB_TICKET_STR);
    end
end

-- 无角色，推荐自动创角得服务器
function Lua_ServerListData:getAutoLoginServer()
    if PlayerData.instance:getPhpConfigNumberValue(PlayerData.instance.phpConfigKey, "autoCreateRole") <= 0 then
        return nil;
    end

    if self.isBan then
        return nil;
    end

    if self.myServerList ~= nil and self.myServerList:getSize() > 0 then
        return nil;
    end

    local dt = 0;
    local st = 0;
    local rs = nil;
    local defaultRs = nil;
    for k, list in pairs(self.allServerDic) do
        for i = 1, list:getSize() do
            local data = list[i];
            if data.is_create ~= 0 and data.status ~= -1 and data.status ~= 0 then
            	local open_time = tonumber(tostring(data.open_time));
                if data.defaultV == 1 then
	                if dt < open_time then
	                    dt = open_time;
	                    defaultRs = data;
	                end
	            else
	                if st < open_time then
	                    st = open_time;
	                    rs = data;
	                end
	            end
            end
        end
    end

    if defaultRs ~= nil then
        return defaultRs;
    elseif rs ~= nil then
        return rs;
    end

    return nil;
end

function Lua_ServerListData:getDefaultServer()
    if self.localServerList ~= nil then
        return self.localServerList[1];
    end

    if self.myServerList:getSize() > 0 then
        self.myServerList:sort("login_time", true);
        return self.myServerList[1];
    end
    
    local dt = 0;
    local st = 0;
    local rs = nil;
    local defaultRs = nil;
    local firstRs = nil;
    local selectRs = nil;
    local sltServer = PlayerPrefsUtil.instance:GetString(SetUpDefine.SELECT_SERVER, "");
    for k, list in pairs(self.allServerDic) do
        for i = 1, list:getSize() do
            local data = list[i];
            if data.flag == sltServer then
                selectRs = data;
                break;
            end

            if firstRs == nil then -- 防止一个都没有
                firstRs = data;
            end

            if data.is_create ~= 0 and data.status ~= -1 and data.status ~= 0 then
            	local open_time = tonumber(tostring(data.open_time));
	            if data.defaultV == 1 then
	                if dt <= open_time then
	                    dt = open_time;
	                    defaultRs = data;
	                end
	            else 
	                if st <= open_time then
	                    st = open_time;
	                    rs = data;
	                end
	            end
	        end
	    end
	end

    if selectRs ~= nil then
        return selectRs;
    elseif defaultRs ~= nil then
        return defaultRs;
    elseif rs ~= nil then
        return rs;
    elseif firstRs ~= nil then
        return firstRs;
    end

    return nil;
end

function Lua_ServerListData:clean()
    self.localServerList = nil;
	self.selectServerData = nil;
	if self.allZoneList ~= nil then
		self.allZoneList:clear();
		self.allZoneList = nil;
	end

	if self.allServerDic ~= nil then
		for k, arr in pairs(self.allServerDic) do
			arr:clear();
		end
		self.allServerDic = nil;
	end

	if self.myServerList ~= nil then
		self.myServerList:clear();
		self.myServerList = nil;
	end

	if self.recommendServerList ~= nil then
		self.recommendServerList:clear();
		self.recommendServerList = nil;
	end
end