-- 预加载管理
checkRequire("Lua_PreLoadVo");
checkRequire("Lua_PreLoadResInfo");
Lua_PreLoadData = class("Lua_PreLoadData");
function Lua_PreLoadData:ctor()
	self:superCall("ctor");
	
	self.maxIdleNum = 6;
	self.idleVoArr = Array();
	self.preLoadVoArr = Array();
end

function Lua_PreLoadData:dispose()
	self:clearTimer();
	self:removeAll();
end

function Lua_PreLoadData:removeAll()
	local vo;
	local len = self.preLoadVoArr:getSize();
	while (len > 0) do
		vo = self.preLoadVoArr:remove(len);
		vo:clean();
		if self.idleVoArr:getSize() < self.maxIdleNum then
			self.idleVoArr:add(vo);
		end
		len = len - 1;
	end
end

function Lua_PreLoadData:remove(vo)
	if vo == nil then
		return;
	end

	local len = self.preLoadVoArr:getSize();
	for i = 1, len do
		if self.preLoadVoArr[i] == vo then
			self.preLoadVoArr:remove(i);
			vo:clean();
			if self.idleVoArr:getSize() < self.maxIdleNum then
				self.idleVoArr:add(vo);
			end
			return;
		end
	end
end

function Lua_PreLoadData:removeBySign(signStr, tab, fun)
	if signStr == nil or signStr == "" then
		return;
	end

	local vo;
	local len = self.preLoadVoArr:getSize();
	while (len > 0) do
		vo = self.preLoadVoArr[len];
		if vo.signStr == signStr then
			if tab == nil and fun == nil then
				self.preLoadVoArr:remove(len);
				vo:clean();
				if self.idleVoArr:getSize() < self.maxIdleNum then
					self.idleVoArr:add(vo);
				end
			elseif vo.tab == tab then
				if fun == nil or vo.fun == fun then
					self.preLoadVoArr:remove(len);
					vo:clean();
					if self.idleVoArr:getSize() < self.maxIdleNum then
						self.idleVoArr:add(vo);
					end
				end
			end
		end

		len = len - 1;
	end
end

function Lua_PreLoadData:getEmptyPreLoadVo()
	local vo;
	if self.idleVoArr ~= nil then
		local len = self.idleVoArr:getSize();
		if len > 0 then
			vo = self.idleVoArr:remove(len);
			return vo;
		end
	end

	vo = Lua_PreLoadVo.new();
	return vo;
end

function Lua_PreLoadData:getLoadVo(signStr, tab, fun)
	if signStr == nil or signStr == "" then
		return nil;
	end

	local vo;
	local len = self.preLoadVoArr:getSize();
	for i = 1, len do
		vo = self.preLoadVoArr[i];
		if vo.signStr == signStr then
			if tab == nil and fun == nil then
				return vo;
			elseif vo.tab == tab then
				if fun == nil or vo.fun == fun then
					return vo;
				end
			end
		end
	end

	return nil;
end

-- 预显示模型场景
-- 界面以窗口名为sign
function Lua_PreLoadData:preShowModelScene(signStr, loadType, tab, fun, param, heroVo, bModelLoaded, bPlanarShadow)
	if signStr == nil or signStr == "" or heroVo == nil then
		return;
	end

	local vo = self:getLoadVo(signStr, tab, fun);
	if vo ~= nil then
		return;
	end

	vo = self:getEmptyPreLoadVo();
	self.preLoadVoArr:add(vo);
	vo.signStr = signStr;
	vo.loadType = loadType;
	vo.tab = tab;
	vo.fun = fun;
	vo.param = param;
	vo.showHeroVo = heroVo;
	vo.bModelLoaded = bModelLoaded;
	vo.bPlanarShadow = bPlanarShadow;
	vo:startLoad();
	self:addTimer();
end

-- 预显示场景
-- 界面以窗口名为sign
function Lua_PreLoadData:preShowScene(signStr, loadType, tab, fun, param, sceneId, fightId)
	if signStr == nil or signStr == "" then
		return;
	end

	local vo = self:getLoadVo(signStr, tab, fun);
	if vo ~= nil then
		return;
	end

	vo = self:getEmptyPreLoadVo();
	self.preLoadVoArr:add(vo);
	vo.signStr = signStr;
	vo.loadType = loadType;
	vo.tab = tab;
	vo.fun = fun;
	vo.param = param;
	local info = Lua_PreLoadResInfo.new();
	info.openSceneId = sceneId;
	info.fightRecordId = fightId;
	local loadList = Array();
	loadList:add(info);
	vo.loadList = loadList;
	vo:startLoad();
	self:addTimer();
end

-- 预加载资源、场景、模型展示
-- 界面以窗口名为sign
function Lua_PreLoadData:preLoad(signStr, loadType, tab, fun, param, loadList, heroVo, maxVal, addStep)
	if signStr == nil or signStr == "" then
		return;
	end

	local vo = self:getLoadVo(signStr, tab, fun);
	if vo ~= nil then
		print("signStr", signStr);
		return;
	end

	vo = self:getEmptyPreLoadVo();
	self.preLoadVoArr:add(vo);
	vo.signStr = signStr;
	vo.loadType = loadType;
	vo.tab = tab;
	vo.fun = fun;
	vo.param = param;
	vo.loadList = loadList;
	vo.showHeroVo = heroVo;
	vo.maxValue = maxVal;
	vo.addStep = addStep;
	vo:startLoad();
	self:addTimer();
end

function Lua_PreLoadData:preLoadByVo(vo)
	if vo == nil or vo.signStr == nil or vo.signStr == ""  then
		return false;
	end

	if self:getLoadVo(vo.signStr, vo.ab, vo.fun) == nil then
		self.preLoadVoArr:add(vo);
	end
	
	vo:startLoad();
	self:addTimer();
end

function Lua_PreLoadData:addTimer()
	if self.bAddTimer ~= true then
		self.bAddTimer = true;
		Lua_CommonUtil.addTimer("onPreLoadTimer", 0.02, self, self.onPreLoadTimer, nil, -1);
	end
end

function Lua_PreLoadData:clearTimer()
	if self.bAddTimer == true then
		self.bAddTimer = false;
		Lua_CommonUtil.removeTimer(self, self.onPreLoadTimer);
	end
end

function Lua_PreLoadData:onPreLoadTimer()
	local sceneId = nil;
	local fightId = nil;
	if self.needCheckScene then
		sceneId = DataCenter.sceneData.openSceneId;
		fightId = DataCenter.fightData.curPlayFightRecordId;
	end

	local vo;
	local len = self.preLoadVoArr:getSize();
	while (len > 0) do
		local bRemove = false;
		vo = self.preLoadVoArr[len];
		if vo == nil then
			self.preLoadVoArr:remove(len);
		else
			if vo:checkComplete() == true then
				bRemove = true;
			elseif vo.leftTime <= 0 then
				bRemove = true;
				vo.loadState = -1;
				vo:disposeModelScene();
			elseif self.needCheckScene then
				if fightId > 0 then
					bRemove = true;
					vo.loadState = -1;
					vo:disposeModelScene();
				elseif not vo:checkModelScene(sceneId) then
					bRemove = true;
					vo.loadState = -1;
					vo:disposeModelScene();
				end
			end

			if bRemove then
				self.preLoadVoArr:remove(len);
				vo:executeCallBack();
				vo:clean();
				if self.idleVoArr:getSize() < self.maxIdleNum then
					self.idleVoArr:add(vo);
				end
			end
		end
		
		len = len - 1;
	end

	self.needCheckScene = false;
	if self.preLoadVoArr:getSize() <= 0 then
		self:clearTimer();
	end
end

function Lua_PreLoadData:checkSceneShow(sceneId)
	self.needCheckScene = true;
end



-- 进入游戏预加载
function Lua_PreLoadData:enterGamePreLoad(tab, fun)
	DataCenter.preLoadUtil:initPreLoadInfo();
    local arr1 = DataCenter.preLoadUtil:getPreLoadListByLv(0);
    local arr2 = DataCenter.preLoadUtil:getPreLoadListByLv(DataCenter.roleData.level);
    local guideResArr = DataCenter.guideData:getPreLoadListByGuideId();
    if (arr1 == nil or arr1:getSize() <= 0) and (arr2 == nil or arr2:getSize() <= 0) and (guideResArr == nil or guideResArr:getSize() <= 0) then
        if tab ~= nil and fun ~= nil then
        	fun(tab);
        end
    else
    	local arr = Array();
    	if arr1 ~= nil then
    		for i = 1, arr1:getSize() do
    			local vo = Lua_PreLoadResInfo.new();
    			local obj = DataCenter.preLoadUtil:getPreLoadInfo(arr1[i]);
		        vo.resType = obj.resType; -- 资源类型
		        vo.resName = obj.path; -- 资源名
		        vo.keepTime = obj.keepTime;
		        -- print("preLoad2", obj.resType, obj.path, obj.keepTime);
		        arr:add(vo);
    		end
    	end

    	if arr2 ~= nil then
    		for i = 1, arr2:getSize() do
    			local vo = Lua_PreLoadResInfo.new();
    			local obj = DataCenter.preLoadUtil:getPreLoadInfo(arr2[i]);
		        vo.resType = obj.resType; -- 资源类型
		        vo.resName = obj.path; -- 资源名
		        vo.keepTime = obj.keepTime;
		        -- print("preLoad2", obj.resType, obj.path, obj.keepTime);
		        arr:add(vo);
    		end
    	end

    	if guideResArr ~= nil then
    		for i = 1, guideResArr:getSize() do
    			local vo = Lua_PreLoadResInfo.new();
    			local obj = guideResArr[i];
			    vo.resType = obj.resType;-- 资源类型
			    vo.resName = obj.path; -- 资源名
			    vo.keepTime = obj.keepTime;
				-- print("preLoad2 guideResArr", obj.resType, obj.path, obj.keepTime);
			    arr:add(vo);
    		end
    	end

    	if arr:getSize() > 0 then
		    local addStep = 98 / arr:getSize();
		    self:preLoad("enterGamePreLoad", Lua_PreLoadDefine.LOADSHOW_TYPE_CSHARP_PROGRESS, tab, fun, nil, arr, nil, 100, addStep);
		end
    end
end