-- 音效
checkRequire("Lua_AudioVo");
Lua_AudioManager = class("Lua_AudioManager");
function Lua_AudioManager:ctor()
	self:superCall("ctor");
	
	self._chatPlayTime = 0;
    self._isPlayingChat = false;
    self._curFrameTime = 0;
    self._sceneAudioRoot = nil;
    self._audioRoot = nil;
    self._idleAudioItemList = Array();
    self._idleAudioVoList = Array();

    self._playAudioIdsList = Array(); -- 当前播放音频audioid列表
    self._playAudioIdsDic = {}; -- 当前播放音频id集合
    self._playAudioVosDic = {}; -- 当前播放音频集合，key 音频id val 音频vo
    self._playAudioKindsDic = {}; -- 当前音频类型对应id集合
    self._playAudioPlayerIdsDic = {}; -- 当前音频播放器对应id集合
    self._audioPlayTimeDic = {}; -- 背景音乐播放点计时器

    if self.audioRoot == nil then return; end
    if self.sceneAudioRoot == nil then return; end
end

function Lua_AudioManager:dispose()
    self._chatPlayTime = 0;
    self._isPlayingChat = false;
    local count = self._idleAudioVoList:getSize();
    while (count > 0) do
        self._idleAudioVoList[count]:dispose();
        self._idleAudioVoList:remove(count);
        count = count - 1;
    end

    coun = self._idleAudioItemList:getSize();
    while (count > 0) do
    	self._idleAudioItemList[count]:dispose();
    	self._idleAudioItemList:remove(count);
    	count = count - 1;
    end

    table.clear(self._audioPlayTimeDic);
    table.clear(self._playAudioIdsDic);
    table.clear(self._playAudioKindsDic);
    table.clear(self._playAudioPlayerIdsDic);
    count = self._playAudioIdsList:getSize();
    while (count > 0) do
    	local audioId = self._playAudioIdsList[count];
    	self._playAudioIdsList:remove(count);
    	local vo = self._playAudioVosDic[audioId];
    	self._playAudioVosDic[audioId] = nil;
    	vo:dispose();
    	count = count - 1;
    end
end

function Lua_AudioManager:returnAudioVo(vo)
    if vo == nil then 
    	return; 
    end

    vo:clean();
    self._idleAudioVoList:add(vo);
end

function Lua_AudioManager:createAudioVo()
    local vo = nil;
    local count = self._idleAudioVoList:getSize();
    while (count > 0) do
        vo = self._idleAudioVoList[count];
        self._idleAudioVoList:remove(count);
        count = count - 1;
        return vo;
    end

    vo = Lua_AudioVo.new();
    return vo;
end

function Lua_AudioManager:returnAudioItem(item)
	if item == nil then 
		return; 
	end

	item:clean();
    self._idleAudioItemList:add(item);
end

function Lua_AudioManager:createAudioItem(parent)
	local item = nil;
    local count = self._idleAudioItemList:getSize();
    while (count > 0) do
        item = self._idleAudioItemList[count];
        self._idleAudioItemList:remove(count);
        item:setParent(parent);
        item:setActive(true);
        count = count - 1;
        return item;
    end

    item = AudioItem(parent);
    return item;
end

function Lua_AudioManager.Get:audioRoot()
	if self._audioRoot == nil then
        local tf = GlobalVar.mainCamera.transform;
        self._audioRoot = ContainerUtil.getContainerTran("AudioSound", tf);
        if self._audioRoot == nil then
            self._audioRoot = UICreate.createGameObject(tf.gameObject);
            self._audioRoot.name = "AudioSound";
        end
    end
    return self._audioRoot;
end

function Lua_AudioManager.Get:sceneAudioRoot()
	if self._sceneAudioRoot == nil then
        self._sceneAudioRoot = UICreate.createGameObject(nil);
        self._sceneAudioRoot.name = "SceneSound";
    end
    return self._sceneAudioRoot;
end

function Lua_AudioManager:removeAllAudio()
    self._chatPlayTime = 0;
    self._isPlayingChat = false;
    table.clear(self._audioPlayTimeDic);
    table.clear(self._playAudioIdsDic);
    table.clear(self._playAudioKindsDic);
    table.clear(self._playAudioPlayerIdsDic);
    count = self._playAudioIdsList:getSize();
    while (count > 0) do
    	local audioId = self._playAudioIdsList[count];
    	self._playAudioIdsList:remove(count);
    	local vo = self._playAudioVosDic[audioId];
    	self._playAudioVosDic[audioId] = nil;
    	self:returnAudioVo(vo);
    	count = count - 1;
    end
end

function Lua_AudioManager:getVoByAudioId(audioId)
	return self._playAudioVosDic[audioId];
end

function Lua_AudioManager:getCountByPlayerId(playerId)
	local arr = self._playAudioPlayerIdsDic[playerId];
	return arr ~= nil and arr:getSize() or 0;
end

function Lua_AudioManager.Get:isPlayingChat()
    return self._isPlayingChat;
end

function Lua_AudioManager:isPlayingByPlayerId(playerId)
	return self:getCountByPlayerId(playerId) > 0;
end

function Lua_AudioManager:isPlayingById(id)
	local arr = self._playAudioIdsDic[id];
    return arr ~= nil and arr:getSize() > 0;
end

function Lua_AudioManager:isPlayingByKind(kind)
	local arr = self._playAudioKindsDic[kind];
	return arr ~= nil and arr:getSize() > 0;
end

function Lua_AudioManager:isPlaying(playerId, id)
    if strIsNullOrEmpty(id) then
        return self:isPlayingByPlayerId(playerId);
    end
    return self:isPlayingById(id);
end

function Lua_AudioManager:resetMusicVolume()
    self:resetVolume(Lua_AudioDefine.PLAYER_BG);
    self:resetVolume(Lua_AudioDefine.PLAYER_SCENE);
end

function Lua_AudioManager:resetSoundVolume()
    self:resetVolume(Lua_AudioDefine.PLAYER_SOUND);
    self:resetVolume(Lua_AudioDefine.PLAYER_UI);
    self:resetVolume(Lua_AudioDefine.PLAYER_NPC);
end

function Lua_AudioManager:resetVolume(playerId)
    local arr = self._playAudioPlayerIdsDic[playerId];
    if arr ~= nil then
    	for i = 1, arr:getSize() do
    		local vo = self:getVoByAudioId(arr[i]);
    		if vo ~= nil then
    			vo.volume = vo.volume;
    		end
    	end
    end
end

function Lua_AudioManager:resetVolumeByKind(kind)
	local arr = self._playAudioKindsDic[kind];
    if arr ~= nil then
    	for i = 1, arr:getSize() do
    		local vo = self:getVoByAudioId(arr[i]);
    		if vo ~= nil then
    			vo.volume = vo.volume;
    		end
    	end
    end
end

function Lua_AudioManager:addVo(vo)
	if vo == nil then
		return;
	end

	local audioId = vo.audioId;
	if self._playAudioVosDic[audioId] ~= nil then
		return;
	end
	self._playAudioVosDic[audioId] = vo;
	self._playAudioIdsList:add(audioId);

	if self._playAudioIdsDic[vo.id] == nil then
		self._playAudioIdsDic[vo.id] = Array();
	end
	self._playAudioIdsDic[vo.id]:add(audioId);

	if self._playAudioKindsDic[vo.kind] == nil then
		self._playAudioKindsDic[vo.kind] = Array();
	end
	self._playAudioKindsDic[vo.kind]:add(audioId);

	if self._playAudioPlayerIdsDic[vo.playerId] == nil then
		self._playAudioPlayerIdsDic[vo.playerId] = Array();
	end
	self._playAudioPlayerIdsDic[vo.playerId]:add(audioId);
	self._playAudioPlayerIdsDic[vo.playerId]:sortf(self, self.audioSoftFun);
end

function Lua_AudioManager:audioSoftFun(audioId1, audioId2)
	local vo1 = self:getVoByAudioId(audioId1);
	local vo2 = self:getVoByAudioId(audioId2);
	if vo1 ~= nil and vo2 ~= nil then
		return vo1.priority < vo2.priority;
	elseif vo1 == nil then
		return true;
	elseif vo2 == nil then
		return false;
	end
	return false;
end

function Lua_AudioManager:removeVo(vo, ignorIdsList, ignorIdsDic, ignorKindsDic, ignorPlayerIdsDic)
	if vo == nil then
		return;
	end

	local audioId = vo.audioId;
	if self._playAudioVosDic[audioId] == nil then
		return;
	end
	self._playAudioVosDic[audioId] = nil;

	if not ignorIdsList then
		self._playAudioIdsList:removeItem(audioId);
	end

	if not ignorIdsDic and self._playAudioIdsDic[vo.id] ~= nil then
		self._playAudioIdsDic[vo.id]:removeItem(audioId);
	end

	if not ignorKindsDic and self._playAudioKindsDic[vo.kind] ~= nil then
		self._playAudioKindsDic[vo.kind]:removeItem(audioId);
	end

	if not ignorPlayerIdsDic and self._playAudioPlayerIdsDic[vo.playerId] ~= nil then
		self._playAudioPlayerIdsDic[vo.playerId]:removeItem(audioId);
	end

	self:returnAudioVo(vo);
end

function Lua_AudioManager:playChat(voiceId)
	local delaytimes = ChatUtil.getLengthByVoiceId(voiceId);
	delaytimes = tonumber(tostring(delaytimes));
    if delaytimes <= 0 then
    	return;
    end

    local chatVo;
    local arr = self._playAudioIdsDic[Lua_AudioDefine.chatAudioIdConst];
    if arr ~= nil and arr:getSize() > 0 then
        chatVo = self:getVoByAudioId(arr[1]);
    end

    delaytimes = delaytimes + 2; -- 播放完延迟3秒再播放别的声音
    if chatVo ~= nil then
        chatVo:stop();
    else
        chatVo = self:createAudioVo();
        chatVo.id = Lua_AudioDefine.chatAudioIdConst;
        chatVo.kind = Lua_AudioDefine.CHAT;
        chatVo.playerId = Lua_AudioDefine.PLAYER_CHAT;
       	chatVo.audioId = getTime(); 
       	chatVo.audioItem = self:createAudioItem(self.audioRoot);
       	self:addVo(chatVo);
    end
    
    local audioItem = chatVo.audioItem;
    self._chatPlayTime = delaytimes;
    audioItem:setChatClip(voiceId);
    audioItem.name = "chatSound";
    audioItem.mute = false;
    chatVo.volume = 1;
    chatVo:play(0);
    if not self._isPlayingChat then
        self._isPlayingChat = true;
        self:muteAllMusic();
    end
end

-- 静音所有音乐
function Lua_AudioManager:muteAllMusic()
    local count = self._playAudioIdsList:getSize();
    for i = 1, count do
    	local vo = self:getVoByAudioId(self._playAudioIdsList[i]);
    	if vo ~= nil and not vo.isChat then
    		vo.mute = true;
    	end
    end
end

-- 播放所有音乐
function Lua_AudioManager:playAllMusic()
	local count = self._playAudioIdsList:getSize();
    for i = 1, count do
    	local vo = self:getVoByAudioId(self._playAudioIdsList[i]);
    	if vo ~= nil and not vo.isChat then
    		vo.mute = false;
    	end
    end
end

function Lua_AudioManager:allMusicDownTo(volume, playerId, stopWhenZero, isFade)
    if self._isPlayingChat then
        return;
    end

    isFade = isFade ~= nil and isFade or true;
    stopWhenZero = stopWhenZero ~= nil and stopWhenZero or false;
    local arr = playerId ~= nil and self._playAudioPlayerIdsDic[playerId] or self._playAudioIdsList;
    local count = arr:getSize();
    for i = 1, count do
        local vo = self:getVoByAudioId(arr[i]);
    	if vo ~= nil and not vo.isChat and vo.audioItem ~= nil then
    		if isFade then
                vo:fade(volume, stopWhenZero);
            else
                vo.volume = volume;
            end
    	end
    end
end

function Lua_AudioManager:muteAllMusicByFadeOut()
	local count = self._playAudioIdsList:getSize();
    for i = 1, count do
        local vo = self:getVoByAudioId(self._playAudioIdsList[i]);
    	if vo ~= nil and not vo.isChat and vo.audioItem ~= nil then
    		vo:fadeOut();
    	end
    end
end

function Lua_AudioManager:muteAllMusicByFadeIn()
	local count = self._playAudioIdsList:getSize();
    for i = 1, count do
        local vo = self:getVoByAudioId(self._playAudioIdsList[i]);
    	if vo ~= nil and not vo.isChat and vo.audioItem ~= nil then
    		vo:fadeIn();
    	end
    end
end

function Lua_AudioManager:stopByPlayerId(playerId, isDispose)
	if playerId == nil then
		return;
	end

	local arr = self._playAudioPlayerIdsDic[playerId];
	if arr == nil then
		return;
	end

    local count = arr:getSize();
    while (count > 0) do
    	local vo = self:getVoByAudioId(arr[count]);
    	if playerId == Lua_AudioDefine.PLAYER_BG and vo.unique and vo.isPlaying then
            self._audioPlayTimeDic[vo.id] = vo.audioItem.time;
        end

        vo:stop();
        if isDispose == nil or isDispose then 
        	arr:remove(count);
        	self:removeVo(vo, false, false, false, true);
        end
    	count = count - 1;
    end
end

function Lua_AudioManager:stop(id, isDispose)
	if id == nil then
		return;
	end

	local arr = self._playAudioIdsDic[id];
	if arr == nil then
		return;
	end

	local count = arr:getSize();
    while (count > 0) do
    	local vo = self:getVoByAudioId(arr[count]);
    	if vo.playerId == Lua_AudioDefine.PLAYER_BG and vo.unique and vo.isPlaying then
            self._audioPlayTimeDic[vo.id] = vo.audioItem.time;
        end

        vo:stop();
        if isDispose == nil or isDispose then 
        	arr:remove(count);
        	self:removeVo(vo, false, true, false, false);
        end
    	count = count - 1;
    end
end

function Lua_AudioManager:play(id, fightId, playSpeed, playSpdRate)
    if strIsNullOrEmpty(id) then
        return;
    end

    if DataCenter.loginData:checkUpdateApp(true) then
        return;
    end

    local cfg = nil;
    if id == Lua_AudioDefine.LOGIN_WIN_BG then
        cfg = Lua_AudioDefine.loginWinConfig;
    elseif id == Lua_AudioDefine.LOGIN_CLICK then
        cfg = Lua_AudioDefine.loginClickConfig;
    else
        cfg = getConfig("sound_new", id);
        if cfg == nil then
        	return;
        end
    end

    -- 播放概率
    local play_rate = cfg.play_rate;
    if play_rate ~= nil and play_rate < Mathf.Random(0, 100) then
        return;
    end

    local vo;
    fightId = fightId ~= nil and fightId or 0;
    playSpeed = playSpeed ~= nil and playSpeed or 1;
    playSpdRate = playSpdRate ~= nil and playSpdRate or 1;
    local playerId = cfg.player_id;
    local maxCount = Lua_AudioDefine.getPlayerCount(playerId);
    local playerIdsArr = self._playAudioPlayerIdsDic[playerId];
    local count = playerIdsArr ~= nil and playerIdsArr:getSize() or 0;
    local unique = cfg.unique;
    if unique ~= nil and unique == 1 then -- 唯一
    	local arr = self._playAudioIdsDic[id];
    	if arr ~= nil and arr:getSize() > 0 then
    		vo = self:getVoByAudioId(arr[1]);
    		if vo ~= nil then
                vo:setData(id, cfg);
    		end
    	end
    end

    if vo == nil then
    	if maxCount <= 0 then return; end
		if maxCount <= count and count > 0 then -- 超出上限
			local preVo = self:getVoByAudioId(playerIdsArr[1]);
			local priority = cfg.priority ~= nil and cfg.priority or 0;
			if preVo ~= nil and preVo.priority > priority then
				return;
			end
		end

		count = count + 1;
        while (maxCount < count) do
            local rvo = self:getVoByAudioId(playerIdsArr[1]);
            if rvo.playerId == Lua_AudioDefine.PLAYER_BG and rvo.unique and rvo.isPlaying then
                self._audioPlayTimeDic[rvo.id] = rvo.audioItem.time;
            end
            self:removeVo(rvo);
            count = count - 1;
        end

	    vo = self:createAudioVo();
		vo:setData(id, cfg);
		vo.audioId = getTime();
		local audioItem = self:createAudioItem(playerId == Lua_AudioDefine.PLAYER_SCENE and self.sceneAudioRoot or self.audioRoot);
		vo:setAudio(audioItem);
		self:addVo(vo);
	end

    if fightId > 0 and vo.kind ~= Lua_AudioDefine.SOUND then
        playSpeed = 1;
        playSpdRate = 1;
    end

	vo.fightId = fightId;
    vo.audioItem:setPlaySpeed(playSpeed, playSpdRate);
	vo.audioItem.mute = self._isPlayingChat;
	vo.volume = 1;
	local t = self._audioPlayTimeDic[id] ~= nil and self._audioPlayTimeDic[id] or 0;
    if id == Lua_AudioDefine.LOGIN_WIN_BG and GlobalVar.loginAudio ~= nil then
        t = GlobalVar.loginAudio.time;
        MsgUtil.callFunc("Audio_Login_Play"); -- 关闭gameenter的登录背景音乐
    elseif playerId == Lua_AudioDefine.PLAYER_BG then
        vo.audioItem.volume = 0;
        vo:fadeIn();
    end
	vo:play(t);
end

function Lua_AudioManager:updateAudioPlaySpeed(fightId, speed, rate, kind)
	local arr = kind ~= nil and self._playAudioKindsDic[kind] or self._playAudioIdsList;
    local count = arr:getSize();
    for i = 1, count do
    	local vo = self:getVoByAudioId(arr[i]);
    	if vo ~= nil and vo.fightId == fightId then
    		vo.audioItem:setPlaySpeed(speed, rate);
    	end
    end
end

function Lua_AudioManager:frameScript()
    local nowTime = getTime();
    local dt = nowTime - self._curFrameTime;
    self._curFrameTime = nowTime;

    local bChangeMute = false;
    if self._chatPlayTime > 0 then
        self._chatPlayTime = self._chatPlayTime - dt;
        if self._chatPlayTime <= 0 then
            bChangeMute = true;
            self._chatPlayTime = 0;
            self._isPlayingChat = false;
        end
    end

    local count = self._playAudioIdsList:getSize();
    while (count > 0) do
    	local audioId = self._playAudioIdsList[count];
    	local vo = self:getVoByAudioId(audioId);
    	if vo.isChat then
    		if not self._isPlayingChat then -- 移除
    			self._playAudioIdsList:remove(count);
    			self:removeVo(vo, true);
    		end
    	else
    		if vo:playEnd() then
    			self._playAudioIdsList:remove(count);
    			self:removeVo(vo, true);
    		elseif bChangeMute then
    			vo.audioItem.mute = false;
    		end
    	end
        count = count - 1;
    end
end