-- 音效
Lua_AudioVo = class("Lua_AudioVo");
function Lua_AudioVo:ctor(id, cfg)
	self:superCall("ctor");
	self.id = id;
	if cfg == nil then
		cfg = getConfig("sound_new", id);
	end
	self.config = cfg;
	self._fightId = 0;
	self._volume = 1;
end

-- 分配的唯一id，根据时间戳赋值
function Lua_AudioVo.Get:audioId()
	return self._audioId;
end

function Lua_AudioVo.Set:audioId(val)
	self._audioId = val;
end

function Lua_AudioVo.Get:fightId()
	return self._fightId;
end

function Lua_AudioVo.Set:fightId(val)
	self._fightId = val;
end

function Lua_AudioVo:setData(id, cfg)
	self.id = id;
	if cfg == nil then
		cfg = getConfig("sound_new", id);
	end
	self.config = cfg;
	self:refreshAudioItem();
end

function Lua_AudioVo:setAudio(item)
	self.audioItem = item;
	self:refreshAudioItem();
end

function Lua_AudioVo:refreshAudioItem()
	if self.config == nil then return; end
	if self.audioItem == nil then return; end
	if strIsNullOrEmpty(self.fileName) then return; end
	self.audioItem.name = self.id;
	self.audioItem.loop = self.loop;
	self.audioItem:setFilePath(self.fileName, self.clipName);
	if self.playerId == Lua_AudioDefine.PLAYER_SCENE then
		self.audioItem:setLocation(self.px, self.py, self.pz);
		self.audioItem.maxDistance = self.maxDistance;
		self.audioItem.minDistance = vo.minDistance;
		self.audioItem.volume = self._volume;
		self.audioItem.rollOffVal = 1;
	end
end

function Lua_AudioVo:play(t, delayt)
	if self.audioItem ~= nil then
		delayt = delayt == nil and self.delayTime or delayt;
		self.audioItem:play(t, delayt);
	end
end

function Lua_AudioVo:stop()
	if self.audioItem ~= nil then
		self.audioItem:stop();
	end
end

function Lua_AudioVo:pause()
	if self.audioItem ~= nil then
		self.audioItem:pause();
	end
end

function Lua_AudioVo.Get:mute()
	return self.audioItem ~= nil and self.audioItem.mute or false;
end

function Lua_AudioVo.Set:mute(val)
	if self.audioItem ~= nil then
		self.audioItem.mute = val;
	end
end

function Lua_AudioVo.Get:volume()
	return self._volume;
end

function Lua_AudioVo.Set:volume(val)
	self._volume = val;
	if self.audioItem ~= nil then
		self.audioItem.volume = DataCenter.audioUtil:checkVolume(self.playerId, val, self.cfgVolume);
	end
end

function Lua_AudioVo:fade(vol, stopWhenZero)
	if self.audioItem ~= nil then
		vol = DataCenter.audioUtil:checkVolume(self.playerId, vol, self.cfgVolume);
		self.audioItem:fade(vol, stopWhenZero);
	end
end

function Lua_AudioVo:fadeIn()
	if self.audioItem ~= nil then
		local vol = DataCenter.audioUtil:checkVolume(self.playerId, 1, self.cfgVolume);
		self.audioItem:fadeIn(vol);
	end
end

function Lua_AudioVo:fadeOut()
	if self.audioItem ~= nil then
		self.audioItem:fadeOut();
	end
end

function Lua_AudioVo:playEnd()
	if self.audioItem == nil then
		return true;
	end

	if self.audioItem.isPlayEnd then
		return true;
	end

	return false;
end

function Lua_AudioVo.Get:isEmpty()
	return self.config == nil;
end

function Lua_AudioVo:clean()
	self.id = nil;
	self.config = nil;
	self._kind = nil;
	self._playerId = nil;
	self._audioId = nil;
	self._fightId = 0;
	self._volume = 1;
	if self.audioItem ~= nil then
		DataCenter.audioManager:returnAudioItem(self.audioItem);
		self.audioItem = nil;
	end
end

function Lua_AudioVo:dispose()
	self.config = nil;
	if self.audioItem ~= nil then
		self.audioItem:dispose();
		self.audioItem = nil;
	end
end

-- 是否是聊天语音
function Lua_AudioVo.Get:isChat()
	return self.id == Lua_AudioDefine.chatAudioIdConst;
end

function Lua_AudioVo.Get:isPlaying()
	return self.audioItem ~= nil and self.audioItem.isPlaying or false;
end

-- 分类 1 音乐 2 音效 3 聊天语音
function Lua_AudioVo.Get:kind()
	if self._kind ~= nil then
		return self._kind;
	end
	return (self.config ~= nil and self.config.kind ~= nil) and self.config.kind or 2;
end

function Lua_AudioVo.Set:kind(val)
	self._kind = val;
end

-- 播放器
function Lua_AudioVo.Get:playerId()
	if self._playerId ~= nil then
		return self._playerId;
	end
	return (self.config ~= nil and self.config.player_id ~= nil) and self.config.player_id or 7;
end

function Lua_AudioVo.Set:playerId(val)
	self._playerId = val;
end

function Lua_AudioVo.Get:url()
	if self._url == nil then
		if self.config ~= nil and self.config.fileName ~= nil then
			self._url = "/audio/"..self.config.fileName;
		end
	end
	return self._url;
end

function Lua_AudioVo.Get:clipName()
	return self.config ~= nil and self.config.clipName or nil;
end

-- 文件名
function Lua_AudioVo.Get:fileName()
	return self.config ~= nil and self.config.fileName or nil;
end

-- 概率
function Lua_AudioVo.Get:playRate()
	return (self.config ~= nil and self.config.play_rate ~= nil) and self.config.play_rate or 100;
end

-- 唯一
function Lua_AudioVo.Get:unique()
	return (self.config ~= nil and self.config.unique ~= nil) and self.config.unique == 1 or false;
end

-- 循环
function Lua_AudioVo.Get:loop()
	return (self.config ~= nil and self.config.loop ~= nil) and self.config.loop == 1 or false;
end

-- x
function Lua_AudioVo.Get:px()
	return (self.config ~= nil and self.config.px ~= nil) and self.config.px or 0;
end

-- y
function Lua_AudioVo.Get:py()
	return (self.config ~= nil and self.config.py ~= nil) and self.config.py or 0;
end

-- z
function Lua_AudioVo.Get:pz()
	return (self.config ~= nil and self.config.pz ~= nil) and self.config.pz or 0;
end

-- 渐变最大距离
function Lua_AudioVo.Get:maxDistance()
	return (self.config ~= nil and self.config.maxDistance ~= nil) and self.config.maxDistance or -1;
end

-- 渐变最小距离
function Lua_AudioVo.Get:minDistance()
	return (self.config ~= nil and self.config.minDistance ~= nil) and self.config.minDistance or -1;
end

-- 延迟播放
function Lua_AudioVo.Get:delayTime()
	return (self.config ~= nil and self.config.delay_time ~= nil) and self.config.delay_time/30 or 0;
end

-- 音量
function Lua_AudioVo.Get:cfgVolume()
	return (self.config ~= nil and self.config.volume ~= nil) and self.config.volume or 1;
end

function Lua_AudioVo.Get:priority()
	return (self.config ~= nil and self.config.priority ~= nil) and self.config.priority or 0;
end