Lua_UITweenConfigData = class("Lua_UITweenConfigData");
function Lua_UITweenConfigData.getConfigData(id)
    if id == nil or id <= 0 then
        return nil;
    end

    if Lua_UITweenConfigData.configs == nil then
        Lua_UITweenConfigData.configs = { };
    end

    if Lua_UITweenConfigData.configs[id] == nil then
        local config = getConfig("ui_tween", id);
        if config == nil then
            return nil;
        end
        Lua_UITweenConfigData.configs[id] = Lua_UITweenConfigData.new(id, config);
    end
    return Lua_UITweenConfigData.configs[id];
end

function Lua_UITweenConfigData:ctor(id, config)
    self:superCall("ctor");
    self.id = id;
    self.config = config;
    if config == nil then
        self.config = getConfig("ui_tween", id);
    end
end

-------------------- 配置数据 --------------------
function Lua_UITweenConfigData.Get:ui_object()
    return self.config.ui_object;
end

function Lua_UITweenConfigData.Get:uiObjectArr()
    if self.ui_object == nil then
        return nil;
    end

    if self._uiObjectArr == nil then
        self._uiObjectArr = string.split(self.ui_object, "/");
    end
    return self._uiObjectArr;
end

function Lua_UITweenConfigData.Get:keepTime()
    if self.config == nil then
        return 0;
    end
    return self.config.keep_time or 0;
end

function Lua_UITweenConfigData.Get:delayTime()
    if self.config == nil then
        return 0;
    end
    return self.config.delay_time or 0;
end

function Lua_UITweenConfigData.Get:bFade()
    if self.config == nil then
        return false;
    end
    return self.config.fadeType == -1 or self.config.fadeType == 1;
end

function Lua_UITweenConfigData.Get:isFadeIn()
    if self.config == nil then
        return false;
    end
    return self.config.fadeType == 1;
end

function Lua_UITweenConfigData.Get:fadeValue()
    local val = self.config.gradual_change;
    if val == nil then
        val = -1;
    end
    return val;
end

function Lua_UITweenConfigData.Get:startPos()
    if self._startPos == nil then
        if self.config ~= nil and self.config.start_pos ~= nil then
            local x = #self.config.start_pos > 0 and self.config.start_pos[1] or 0;
            local y = #self.config.start_pos > 1 and self.config.start_pos[2] or 0;
            local z = #self.config.start_pos > 2 and self.config.start_pos[3] or 0;
            self._startPos = Vector3(x, y, z);
        end
    end
    return self._startPos;
end

function Lua_UITweenConfigData.Get:startPosTab()
    if self._startPosTab == nil then
        if self.config ~= nil and self.config.start_pos ~= nil then
            local x = #self.config.start_pos > 0 and self.config.start_pos[1] or 0;
            local y = #self.config.start_pos > 1 and self.config.start_pos[2] or 0;
            local z = #self.config.start_pos > 2 and self.config.start_pos[3] or 0;
            self._startPosTab = {x, y, z};
        end
    end
    return self._startPosTab;
end

function Lua_UITweenConfigData.Get:targetPos()
    if self._targetPos == nil then
        if self.config ~= nil and self.config.target_pos ~= nil then
            local x = #self.config.target_pos > 0 and self.config.target_pos[1] or 0;
            local y = #self.config.target_pos > 1 and self.config.target_pos[2] or 0;
            local z = #self.config.target_pos > 2 and self.config.target_pos[3] or 0;
            self._targetPos = Vector3(x, y, z);
        else
            self._targetPos = Vector3(0, 0, 0);
        end
    end
    return self._targetPos;
end

function Lua_UITweenConfigData.Get:targetPosTab()
    if self.config == nil then
        return nil;
    end
    return self.config.target_pos;
end

function Lua_UITweenConfigData.Get:offsetPos()
    if self._offsetPos == nil then
        if self.config ~= nil and self.config.offset_pos ~= nil then
            local x = #self.config.offset_pos > 0 and self.config.offset_pos[1] or 0;
            local y = #self.config.offset_pos > 1 and self.config.offset_pos[2] or 0;
            local z = #self.config.offset_pos > 2 and self.config.offset_pos[3] or 0;
            self._offsetPos = Vector3(x, y, z);
        end
    end
    return self._offsetPos;
end

function Lua_UITweenConfigData.Get:offsetPosTab()
    if self._offsetPosTab == nil then
        if self.config ~= nil and self.config.offset_pos ~= nil then
            local x = #self.config.offset_pos > 0 and self.config.offset_pos[1] or 0;
            local y = #self.config.offset_pos > 1 and self.config.offset_pos[2] or 0;
            local z = #self.config.offset_pos > 2 and self.config.offset_pos[3] or 0;
            self._offsetPosTab = {x, y, z};
        end
    end
    return self._offsetPosTab;
end

function Lua_UITweenConfigData.Get:startScale()
    if self._startScale == nil then
        if self.config ~= nil and self.config.start_scale ~= nil then
            local x = #self.config.start_scale > 0 and self.config.start_scale[1] or 1;
            local y = #self.config.start_scale > 1 and self.config.start_scale[2] or 1;
            local z = #self.config.start_scale > 2 and self.config.start_scale[3] or 1;
            self._startScale = Vector3(x, y, z);
        end
    end
    return self._startScale;
end

function Lua_UITweenConfigData.Get:startScaleTab()
    if self._startScaleTab == nil then
        if self.config ~= nil and self.config.start_scale ~= nil then
            local x = #self.config.start_scale > 0 and self.config.start_scale[1] or 1;
            local y = #self.config.start_scale > 1 and self.config.start_scale[2] or 1;
            local z = #self.config.start_scale > 2 and self.config.start_scale[3] or 1;
            self._startScaleTab = {x, y, z};
        end
    end
    return self._startScaleTab;
end

function Lua_UITweenConfigData.Get:targetScale()
    if self._targetScale == nil then
        if self.config ~= nil and self.config.target_scale ~= nil then
            local x = #self.config.target_scale > 0 and self.config.target_scale[1] or 1;
            local y = #self.config.target_scale > 1 and self.config.target_scale[2] or 1;
            local z = #self.config.target_scale > 2 and self.config.target_scale[3] or 1;
            self._targetScale = Vector3(x, y, z);
        else
            self._targetScale = Vector3(1, 1, 1);
        end
    end
    return self._targetScale;
end

function Lua_UITweenConfigData.Get:target_scale()
    if self.config == nil then
        return nil;
    end
    return self.config.target_scale;
end

function Lua_UITweenConfigData.Get:targetScaleTab()
    if self._targetScaleTab == nil then
        if self.config ~= nil and self.config.target_scale ~= nil then
            local x = #self.config.target_scale > 0 and self.config.target_scale[1] or 1;
            local y = #self.config.target_scale > 1 and self.config.target_scale[2] or 1;
            local z = #self.config.target_scale > 2 and self.config.target_scale[3] or 1;
            self._targetScaleTab = {x, y, z};
        else
            self._targetScaleTab = {1, 1, 1};
        end
    end
    return self._targetScaleTab;
end

function Lua_UITweenConfigData.Get:startRot()
    if self._startRot == nil then
        if self.config ~= nil and self.config.start_rot ~= nil then
            local x = #self.config.start_rot > 0 and self.config.start_rot[1] or 0;
            local y = #self.config.start_rot > 1 and self.config.start_rot[2] or 0;
            local z = #self.config.start_rot > 2 and self.config.start_rot[3] or 0;
            self._startRot = Quaternion.Euler(x, y, z);
        end
    end
    return self._startRot;
end

function Lua_UITweenConfigData.Get:startRotTab()
    if self._startRotTab == nil then
        if self.config ~= nil and self.config.start_rot ~= nil then
            local x = #self.config.start_rot > 0 and self.config.start_rot[1] or 0;
            local y = #self.config.start_rot > 1 and self.config.start_rot[2] or 0;
            local z = #self.config.start_rot > 2 and self.config.start_rot[3] or 0;
            self._startRotTab = {x, y, z};
        end
    end
    return self._startRotTab;
end

function Lua_UITweenConfigData.Get:targetRot()
    if self._targetRot == nil then
        if self.config ~= nil and self.config.target_rot ~= nil then
            local x = #self.config.target_rot > 0 and self.config.target_rot[1] or 0;
            local y = #self.config.target_rot > 1 and self.config.target_rot[2] or 0;
            local z = #self.config.target_rot > 2 and self.config.target_rot[3] or 0;
            self._targetRot = Quaternion.Euler(x, y, z);
        else
            self._targetRot = Quaternion.Euler(0, 0, 0);
        end
    end
    return self._targetRot;
end

function Lua_UITweenConfigData.Get:target_rot()
    if self.config == nil then
        return nil;
    end
    return self.config.target_rot;
end

function Lua_UITweenConfigData.Get:targetRotTab()
    if self._targetRotTab == nil then
        if self.config ~= nil and self.config.target_rot ~= nil then
            local x = #self.config.target_rot > 0 and self.config.target_rot[1] or 0;
            local y = #self.config.target_rot > 1 and self.config.target_rot[2] or 0;
            local z = #self.config.target_rot > 2 and self.config.target_rot[3] or 0;
            self._targetRotTab = {x, y, z};
        else
            self._targetRotTab = {0, 0, 0};
        end   
    end
    return self._targetRotTab;
end

function Lua_UITweenConfigData.Get:offect()
    if self.config == nil then
        return 0;
    end
    return self.config.offect or 0;
end

function Lua_UITweenConfigData.Get:delayShow()
    if self.config == nil then
        return false;
    end
    return self.config.delay_show == 1;
end

function Lua_UITweenConfigData.Get:firstDelayTime()
    if self.config == nil then
        return 0;
    end
    return self.config.first_delay_time or 0;
end