-- 界面缓动动画控制器
checkRequire("Lua_UITweenConfigData");
Lua_WinTweenManager = class("Lua_WinTweenManager");
function Lua_WinTweenManager:ctor()
    self:removAllWinTween();
end

function Lua_WinTweenManager:dispose()
    self:removAllWinTween();
end

function Lua_WinTweenManager:removAllWinTween()
    self.tweenDic = {};
    self.tweenStartTimeDic = {};
    self.tweenCountTimeDic = {};
end

-- 判断界面缓动是否结束，dt是容错时间
function Lua_WinTweenManager:isOverTween(winName, dt)
    if winName ~= nil and self.tweenDic[winName] ~= nil then
        local t = getTime() - self.tweenStartTimeDic[winName];
        if self.tweenCountTimeDic[winName] + dt > t then
            return false;
        end
    end
    return true;
end

-- 获取界面缓动时间，dt是容错时间
function Lua_WinTweenManager:getWinTweenCountTime(winName, dt)
    if winName ~= nil and self.tweenDic[winName] ~= nil then
        local t = getTime() - self.tweenStartTimeDic[winName];
        if self.tweenCountTimeDic[winName] + dt > t then
            return self.tweenCountTimeDic[winName] + dt - t
        end
    end
    return 0;
end

-- 根据配置表获取界面动画最大时间
function Lua_WinTweenManager:getWinTweenCfgCountTime(winName)
    if strIsNullOrEmpty(winName) then
        return 0;
    end

    local winData = Lua_WinConfigData.getConfigData(winName);
    if winName == nil or winData.tweenId == nil or #winData.tweenId <= 0 then
        return 0;
    end

    local countTime = 0;
    for i = 1, #winData.tweenId do
        local tweenId = winData.tweenId[i];
        local tweenData = Lua_UITweenConfigData.getConfigData(tweenId);
        if tweenData ~= nil then
            local t = tweenData.delayTime + tweenData.keepTime;
            if countTime < t then
                countTime = t;
            end
        end
    end
    return countTime;
end

-- 移除界面动画
function Lua_WinTweenManager:removeWinTween(winName)
    if not strIsNullOrEmpty(winName) and self.tweenDic[winName] ~= nil then
        local arr = self.tweenDic[winName];
        local len = arr:getSize();
        while len > 0 do
            TweenUtil.getInstance():stop(arr:remove(len));
            len = len - 1;
        end
        self.tweenDic[winName] = nil;
        self.tweenStartTimeDic[winName] = nil;
        self.tweenCountTimeDic[winName] = nil;
    end
end

-- 创建界面动画，返回窗口名（配置表字段）
function Lua_WinTweenManager:createWinTween(winName, tf)
    if tf == nil or strIsNullOrEmpty(winName) then
        return nil;
    end

    if self.tweenDic[winName] ~= nil then
        return winName;
    end

    -- print("createWinTween", winName);
    local winData = Lua_WinConfigData.getConfigData(winName);
    if winData ~= nil and winData.tweenId ~= nil and #winData.tweenId > 0 then
        local tweenList = Array();
        for i = 1, #winData.tweenId do
            local tweenId = winData.tweenId[i];
            local objTf = self:playWinTween(tweenId, tf);
            if objTf ~= nil then
                local tweenData = Lua_UITweenConfigData.getConfigData(tweenId);
                local t = tweenData.delayTime + tweenData.keepTime;
                if self.tweenCountTimeDic[winName] == nil or self.tweenCountTimeDic[winName] < t then
                    self.tweenCountTimeDic[winName] = t;
                end

                if tweenList == nil then 
                    tweenList = Array(); 
                    tweenList:add(objTf);
                elseif tweenList:indexOf(objTf) == -1 then
                    tweenList:add(objTf);
                end
            end
        end

        if tweenList ~= nil then
            self.tweenDic[winName] = tweenList;
            self.tweenStartTimeDic[winName] = getTime();
            return winName;
        end
    end
    return nil;
end

-- 播放界面动画
function Lua_WinTweenManager:playWinTween(tweenId, tf)
    if tweenId == nil or tf == nil then
        return nil;
    end

    local tweenData = Lua_UITweenConfigData.getConfigData(tweenId);
    if tweenData == nil or strIsNullOrEmpty(tweenData.ui_object) then
        return nil;
    end

    local targetTf;
    if tf.name == tweenData.ui_object then
        targetTf = tf;
    else
        targetTf = ContainerUtil.findBoneTransform(tf, tweenData.ui_object);
    end
    if targetTf == nil then
        return nil;
    end

    if self:playWinTweenByObj(targetTf, tweenId, tweenData) then
        return targetTf;
    end
    return nil;
end

-- 播放界面动画
function Lua_WinTweenManager:playWinTweenByObj(tf, tweenId, tweenData)
    if tf == nil then
        return false;
    end

    if tweenData == nil then
        if tweenId == nil then
            return false;
        end

        tweenData = Lua_UITweenConfigData.getConfigData(tweenId);
        if tweenData == nil then
            return false;
        end
    end

    if tweenData.startPosTab ~= nil then
        LuaUtil.setTfPos(tf, tweenData.startPosTab[1], tweenData.startPosTab[2], tweenData.startPosTab[3]);
    end

    if tweenData.startRotTab ~= nil then
        LuaUtil.setTFRot(tf, tweenData.startRotTab[1], tweenData.startRotTab[2], tweenData.startRotTab[3]);
    end

    if tweenData.startScaleTab ~= nil then
        LuaUtil.setTfScale(tf, tweenData.startScaleTab[1], tweenData.startScaleTab[2], 1);
    end

    if tweenData.bFade then
        TweenUtil.getInstance():addTweenTargetAlpha(tf, tweenData.keepTime, tweenData.delayTime, tweenData.fadeValue, tweenData.isFadeIn, nil, nil, tweenData.offect, false, 0);
    end

    local px;
    local py;
    local pz;
    local rx;
    local ry;
    local rz;
    local sx;
    local sy;
    local sz;
    local targetPos = tweenData.targetPosTab;
    if targetPos == nil or #targetPos == 0 then
        local anchoredPosition3D = tf.anchoredPosition3D;
        px = anchoredPosition3D.x;
        py = anchoredPosition3D.y;
        pz = anchoredPosition3D.z;
    elseif #targetPos > 2 then
        px = targetPos[1];
        py = targetPos[2];
        pz = targetPos[3];
    elseif #targetPos == 2 then
        px = targetPos[1];
        py = targetPos[2];
        pz = tf.anchoredPosition3D.z;
    elseif #targetPos == 1 then
        px = targetPos[1];
        local anchoredPosition3D = tf.anchoredPosition3D;
        py = anchoredPosition3D.y;
        pz = anchoredPosition3D.z;
    end

    local targetRot = tweenData.target_rot;
    if targetRot == nil or #targetRot == 0 then
        local localEulerAngles = tf.localEulerAngles;
        rx = localEulerAngles.x;
        ry = localEulerAngles.y;
        rz = localEulerAngles.z;
    elseif #targetRot > 2 then
        rx = targetRot[1];
        ry = targetRot[2];
        rz = targetRot[3];
    elseif #targetRot == 2 then
        rx = targetRot[1];
        ry = targetRot[2];
        rz = tf.localEulerAngles.z;
    elseif #targetRot == 1 then
        rx = targetRot[1];
        local localEulerAngles = tf.localEulerAngles;
        ry = localEulerAngles.y;
        rz = localEulerAngles.z;
    end

    local targetScale = tweenData.target_scale;
    if targetScale == nil or #targetScale == 0 then
        local localScale = tf.localScale;
        sx = localScale.x;
        sy = localScale.y;
        sz = localScale.z;
    elseif #targetScale > 2 then
        sx = targetScale[1];
        sy = targetScale[2];
        sz = targetScale[3];
    elseif #targetScale == 2 then
        sx = targetScale[1];
        sy = targetScale[2];
        sz = tf.localScale.z;
    elseif #targetScale == 1 then
        sx = targetScale[1];
        local localScale = tf.localScale;
        sy = localScale.y;
        sz = localScale.z;
    end

    tf.gameObject:SetActive(not tweenData.delayShow);
    TweenUtil.getInstance():addTfTween(tf, tweenData.keepTime, tweenData.delayTime, 
        px, py, pz, 
        rx, ry, rz, 
        sx, sy, sz,
        nil, nil, tweenData.offect, tweenData.delayShow, 0);
    return true;
end