function isNumber(v)
    return type(v) == 'number'
end

function Array(...)
    local new_array = {}

    local __array__ = {...}

    local __methods__ = {}
    function __methods__:typeof()
        return 'Array'
    end
    function __methods__:add(v, at)
        local len = #__array__ + 1
        at = isNumber(at) and at or len
        at = math.min(at, len)
        table.insert(__array__, at, v)
    end
    function __methods__:insert(v, index)
        index = isNumber(index) and index or #__array__ + 1
        index = index <= 0 and 1 or index;
        table.insert(__array__, index, v)
    end
    function __methods__:getSize()
        return #__array__;
    end
    function __methods__:remove(at)
        at = isNumber(at) and at or #__array__
        return table.remove(__array__, at);
    end
    function __methods__:removeItem(item)
        for i,v in ipairs(__array__) do
            if v == item then
                self:remove(i);
                return true;
            end
        end
        return false;
    end
    function __methods__:getItem(index)
        return __array__[index];
    end
    function __methods__:shift()
        return self:remove(1)
    end
    function __methods__:unshift(v)
        self:insert(v, 1)
    end
    function __methods__:append(...)
        local elements = {...}
        for i= 1, #elements do
            self:insert(elements[i])
        end
    end
    function __methods__:clear()
        -- while( #__array__ > 0 )
        -- do
        --     self:remove(1);
        -- end
        local len = self:getSize();
        while (len > 0) do
            self:remove(len);
            len = len -1;
        end
    end
    function __methods__:dispose()
        local len = self:getSize();
        for i = 1, len do
            local v = __array__[i];
            if type(v) == "table" and v.dispose ~= nil and v.isDispose_2 ~= true then
                v:dispose();
            end
        end
    end
    function __methods__:indexOf(v)
        local len = self:getSize();
        for i = 1, len do
            if __array__[i] == v then
                return i;
            end
        end
        return -1;
    end
    --up:true 降序， false：升序
    function __methods__:sort(name, up)
        up = up == nil and true or up;
        local len = self:getSize();
        local a;
        local b;
        for i = 1, len do
            for j = i + 1, len do
                if strIsNullOrEmpty(name) then
                    a = __array__[i];
                    b = __array__[j];
                else
                    a = __array__[i][name];
                    b = __array__[j][name];
                end
                local r = a < b;
                if up == false then
                    r = not r;
                end
                if (r) then
                    local temp = __array__[i];
                    __array__[i] = __array__[j];
                    __array__[j] = temp;
                end
            end
        end
    end
    --多权重排序
    function __methods__:sortf(owner, sortfunction, param)
        local len = self:getSize();
        for i = 1,len do
            for j = i + 1,len do
                local a = __array__[i]
                local b = __array__[j]
                if not sortfunction(owner, a, b, param) then
                    local temp = __array__[i]
                    __array__[i] = __array__[j]
                    __array__[j] = temp
                end
            end
        end
    end
    local mt = {
        __index = function(t, k)
            if isNumber(k) then
                return __array__[k]
            end
            if __methods__[k] then
                return __methods__[k]
            end
        end,
        __newindex = function(t, k, v)
            if nil == __array__[k] then
                print(string.format('warning : [%s] index out of range.', tostring(k)))
                return
            end
            if nil == v then
                print(string.format('warning : can not remove element by using  `nil`.'))
                return
            end
            __array__[k] = v
        end
    }
    setmetatable(new_array, mt)

    return new_array
end