
function clone(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for key, value in pairs(object) do
            new_table[_copy(key)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

function class(classname, super, resetIndex)
    local cls = {}
    cls.classname = classname
    cls.class = cls
    cls.Get = {}
    cls.Set = {}

    local Get = cls.Get
    local Set = cls.Set

    if super then
        -- copy super method
        for key, value in pairs(super) do
            if type(value) == "function" and key ~= "ctor" then
                cls[key] = value
            end
        end

        -- copy super getter
        for key, value in pairs(super.Get) do
            Get[key] = value
        end

        -- copy super setter
        for key, value in pairs(super.Set) do
            Set[key] = value
        end

        cls.super = super
        cls.resetIndex = super.resetIndex;
    end
    if resetIndex ~= nil then
        cls.resetIndex = resetIndex;
    end
    function cls.__index(self, key)
        local func = cls[key];
        if func ~= nil and key ~= "checkSaveParam" and cls.resetIndex == 1 and self:checkSaveParam(key) == true and self.saveParam ~= nil then
            self:saveFunKey(key);
            return self.saveParam;
        end
        
        if func then
           return func
        end

        local getter = Get[key]
        if getter then
            return getter(self)
        end

        return nil
    end

    function cls.__newindex(self, key, value)
        local setter = Set[key]
        if setter then
            if cls.resetIndex == 1 and self:checkSaveParam(key) == true then
                self:saveParamKey(key, value);
                return;
            end
            setter(self, value)
            return
        end

        if Get[key] then
            assert(false, "readonly property")
        end

        rawset(self, key, value)
    end
    function cls.new(...)
        local self = setmetatable({}, cls)
        local function create(cls, ...)
            if cls.ctor ~= nil then
                cls.ctor(self,...);
            else
                self:superCall("ctor", ...);
            end
        end
        create(cls, ...)

        return self
    end
    function cls:superCall(funStr, ...)
        if funStr ~= "checkSaveParam" and cls.resetIndex == 1 and self:checkSaveParam(funStr) == true then
            self:saveFunKey(funStr);
            return;
        end
        if self.superCalled == nil then
            self.superCalled = {};
        end
        local s = self;
        local s_fun = nil;
        while true do
            if s ~= nil then
                s = s.super;
                if s ~= nil then
                    local hasRun = false;
                    for i,v in pairs(self.superCalled) do
                        if v == s[funStr]  then
                            hasRun = true;
                            break;
                        end
                    end
                    if s[funStr] == self[funStr] then
                        hasRun = true;
                    end
                    if hasRun == false then
                        s_fun = s[funStr];
                        if s_fun ~= nil then
                            self.superCalled[s.classname] = s_fun;
                            break;
                        end
                    end
                end
            else
                break;
            end
        end
        if s_fun ~= nil then
            s_fun(self, ...);
        end
        self.superCalled = nil;
    end
    -- compat
    cls.dtor = nil
    function cls.delete(self)
        if tracebacks[self] < 0 then return end
        local destroy
        destroy = function(cls)
            if cls.dtor then
                cls.dtor(self)
            end
            if cls.super then
                destroy(cls.super)
            end
        end
        destroy(cls)
        tracebacks[self] = -tracebacks[self]
    end
    return cls
end