local require = require
local string = string
local table = table
local json = require 'cjson';
local spliceTab = {};
type_str_table = "table"
type_str_number = "number"
type_str_string = "string"
type_str_array = "Array"

function LuaGC()
    -- local c = collectgarbage("count")
    --print("Begin gc count = "..c.." kb")
    collectgarbage("collect")
    -- c = collectgarbage("count")
    --print("End gc count = "..c.." kb")
end

function getDistance(sx,sy, tx, ty)
    local dx = sx - sy;
    local dy = tx - ty;
    return Mathf.Sqrt(dx * dx + dy * dy);
end

function getTime()
    return UnityEngine.Time.realtimeSinceStartup; -- （只读）表示自游戏开始后的总时间，即使暂停也会不断的增加
end

function decodeJson(str)
    local data = json.decode(str)
    return data;
end

function decodeTabToJson(tab)
    if tab ~= nil then
        return json.encode(tab);
    end
    return nil;
end

function IsNil(uobj)
    return uobj == nil or uobj:Equals(nil)
end

function getConfigInLuaContain(aa,bb,cc,dd,ee,ff,gg,hh)
    local l = getConfigInLua(aa,bb,cc,dd,ee,ff,gg,hh);
    if l == nil then
        return false;
    end
    return true;
end

function getConfigInLuaLen(aa,bb,cc,dd,ee,ff,gg,hh)
    local l = getConfigInLua(aa,bb,cc,dd,ee,ff,gg,hh);
    if l == nil then
        return 0;
    end
    if type(l) ~= "table" then
        return 0;
    end
    return #l;
end

function getConfigInLuaInt(aa,bb,cc,dd,ee,ff,gg,hh)
    local l = getConfigInLua(aa,bb,cc,dd,ee,ff,gg,hh);
    if l == nil then
        return 0;
    end
    if type(l) ~= "number" then
        if type(l) == "table" and type(l[1]) == "number" then
            return l[1];
        end
    end
    return l;
end

function getConfigInLua(aa, bb, cc, dd, ee, ff, gg, hh)
    aa = convertCSharpLong(aa);
    bb = convertCSharpLong(bb);
    cc = convertCSharpLong(cc);
    dd = convertCSharpLong(dd);
    ee = convertCSharpLong(ee);
    ff = convertCSharpLong(ff);
    gg = convertCSharpLong(gg);
    hh = convertCSharpLong(hh);
    if hh ~= nil then
        return getConfig(aa, bb, cc, dd, ee, ff, gg, hh);
    elseif gg ~= nil then
        return getConfig(aa, bb, cc, dd, ee, ff, gg);
    elseif ff ~= nil then
        return getConfig(aa, bb, cc, dd, ee, ff);
    elseif ee ~= nil then
        return getConfig(aa, bb, cc, dd, ee);
    elseif dd ~= nil then
        return getConfig(aa, bb, cc, dd);
    elseif cc ~= nil then
        return getConfig(aa, bb, cc);
    elseif bb ~= nil then
        return getConfig(aa, bb);
    elseif aa ~= nil then
        return getConfig(aa);
    end
end

function convertCSharpLong(longNum)
    if longNum ~= nil and type(longNum) == "userdata" then
        return tonumber(tostring(longNum));
    end
    return longNum;
end

function getRandomRange(val)
    math.randomseed(getTime());
    return Mathf.Floor(val * Mathf.Random()) + 1;
end

function getTraceback()
    return debug.traceback();
end

function string.split(str,reps)
    local resultStrList = Array();
    string.gsub(str,'[^'..reps..']+',function ( w )
        resultStrList.add(resultStrList,w)
    end)
    return resultStrList
end

function string.formateDic(s, dic)
    local str = s;
    for k,v in pairs(dic) do
        str = string.gsub(str, "{"..k.."}", v);
    end
    return str;
end

-- 需要传递tab到C# tab中有对应字段的数值
-- str = "使用后立即回复[18c828]|{5000+level*50}|点生命[-]"
-- str = "使用后获得[18c828]|{0#(80000+8*(1.04^(level-1)))}|点经验值[-]"
-- print(string.caculateString(str, DataCenter.roleData))
function string.caculateString(str, tab, toChinese)
    local arr = string.split(str,"|");
    local rs = "";
    for i = 1, arr:getSize() do
        local isC = string.find(arr[i], "{(.+)}");
        if isC == nil then
            rs = rs..arr[i];
        else
            local funStr = string.sub(arr[i], 2, string.len(arr[i])-1);
            local f = FormulaUtil.createFormula(funStr);
            local r = f:getRsValue(tab);
            if toChinese == true then
                rs = rs..numberTrans(r);
            else
                rs = rs..r;
            end
        end
    end
    return rs;
end

function stringWidth(inputstr)
    local lenInByte = #inputstr
    local width = 0
    local i = 1
    while (i<=lenInByte)
    do
        local curByte = string.byte(inputstr, i)
        local byteCount = 1;
        if curByte>0 and curByte<=127 then
            byteCount = 1                                           --1字节字符
        elseif curByte>=192 and curByte<223 then
            byteCount = 2                                           --双字节字符
        elseif curByte>=224 and curByte<239 then
            byteCount = 3                                           --汉字
        elseif curByte>=240 and curByte<=247 then
            byteCount = 4                                           --4字节字符
        end

        i = i + byteCount                                 -- 重置下一字节的索引
        width = width + 1                                 -- 字符的个数（长度）
    end
    return width
end

--单个字符的长度
function charWidth(char)
    if not char or char == "" then
        return 0
    end
    local code = string.byte(char)
    if code < 127 then
        return 1
    elseif code <= 223 then
        return 2
    elseif code <= 239 then
        return 3
    elseif code <= 247 then
        return 4
    else
        -- 讲道理不会走到这里^_^
        return 0
    end
end

function StringHelperSub(str, startIndex, endIndex)
    local tempStr = str
    local byteStart = 1 -- string.sub截取的开始位置
    local byteEnd = -1 -- string.sub截取的结束位置
    local index = 0  -- 字符记数
    local bytes = 0  -- 字符的字节记数

    startIndex = Mathf.Max(startIndex, 1)
    endIndex = endIndex or -1
    while string.len(tempStr) > 0 do
        if index == startIndex - 1 then
            byteStart = bytes+1;
        elseif index == endIndex then
            byteEnd = bytes;
            break;
        end
        bytes = bytes + charWidth(tempStr)
        tempStr = string.sub(str, bytes+1)

        index = index + 1
    end
    return string.sub(str, byteStart, byteEnd)
end

function table.indexOf(tb, val)
    if tb then
        local index = 1;
        for _, v in pairs(tb) do
            if v == val then
                return index;
            end
            index = index + 1;
        end
    end
    return -1;
end

function table.length(tb)
    local num = 0;
    if tb then
        for _, v in pairs(tb) do
            num = num + 1;
        end
    end
    return num;
end

function table.clear(tb)
    if tb and tb._fields == nil and tb.classname == nil then
        for k, v in pairs(tb) do
            if type(v) == type_str_table then
                if v.typeof and v:typeof() == type_str_array then
                    v:clear()
                else
                    table.clear(v)
                end
            end
            tb[k] = nil
        end
    end
end

function checkRequire(name)
    if rmodules == nil then
        rmodules = {};
    end
    if rmodules[name] == nil then
        local e = loadstring("return "..name..";");
        local b = e();
        if b == nil then
            local f = loadstring("require \""..name.."\"; return "..name..";");
            if f == nil then return nil; end
            local c = f();
            rmodules[name] = c;
        else
            rmodules[name] = b;
        end
    end
    -- if (string.EndsWith(name,"Window") or string.EndsWith(name,"_skin") or string.EndsWith(name,"Panel") or string.EndsWith(name,"Item")) and
    --        DataCenter and DataCenter.bConfigLoaded and Lua_PlatformUtil.IsWindowsEditor() and
    --        ServerConfig.isLuaPack  then
    --    LuaUtil.RefreshLuaFile(name);
    --    updateRequire(name)
    -- end

    return rmodules[name];
end

function gameFindedObjCallBack(t)
    if DataCenter ~= nil and DataCenter.gameFindObjUtil ~= nil then
        DataCenter.gameFindObjUtil:callFinded(t);
    end
end

function getConfig(value, key1, key2, key3, key4, key5)
    local c
    -- if type(value) == type_str_table then
    --     c = checkRequire(value[1])
    --     for i = 2, #value do
    --         if c == nil then
    --             break
    --         end
    --         c = c[value[i]]
    --     end
    -- else
        c = checkRequire(value)
        if key1 and c then
            c = c[key1]
        else
            return c
        end
        if key2 and c then
            c = c[key2]
        else
            return c
        end
        if key3 and c then
            c = c[key3]
        else
            return c
        end
        if key4 and c then
            c = c[key4]
        else
            return c
        end
        if key5 and c then
            c = c[key5]
        else
            return c
        end
    -- end
    return c
end

function getProtoMsg(msgName)
    if checkRequire(msgName) == nil then return nil; end
    local f = loadstring("return ProtoPublic_pb."..msgName.."();");
    local c = f();
    return c;
end

function import(moduleName, currentModuleName)
    local currentModuleNameParts
    local moduleFullName = moduleName
    local offset = 1

    while true do
        if string.byte(moduleName, offset) ~= 46 then
            moduleFullName = string.sub(moduleName, offset)
            if currentModuleNameParts and #currentModuleNameParts > 0 then
                moduleFullName = table.concat(currentModuleNameParts, ".") .. "." .. moduleFullName
            end
            break
        end
        offset = offset + 1

        if not currentModuleNameParts then
            if not currentModuleName then
                local n,v = debug.getlocal(3, 1)
                currentModuleName = v
            end

            currentModuleNameParts = string.split(currentModuleName, ".")
        end
        table.remove(currentModuleNameParts, #currentModuleNameParts)
    end

    return require(moduleFullName)
end

--重新require一个lua文件，替代系统文件。
function reimport(name)
    local package = package
    package.loaded[name] = nil
    package.preload[name] = nil
    return require(name)    
end

--判断字符长度，1个中文算2个长度，其他1个长度
function strlen(str)
    local bytes = { string.byte(str, 1, #str) }
    local length, begin = 0, false
    for _, byte in ipairs(bytes) do
        if byte < 128 or byte >= 192 then
            begin = false
            length = length + 1
        elseif not begin then
            begin = true
            length = length + 1
        end
    end
    return length
end

--debug 遍历lua数据
function showInfo(obj, pre, rs, index)
    rs = rs == nil and {} or rs;
    pre = pre == nil and "" or pre;
    index = index == nil and 0 or index;
    index = index +1;
    if index > 10 then
        return;
    end
    for k,v in pairs(obj) do
        local strType = type(v)
        local kType = type(k)
        if kType == "string" or kType == "number" or kType == "function" then
            rs[pre.."."..k] = strType
            if strType == "table" and v ~= obj and k ~= "loaded" and k ~= "_listener_for_children" then
                showInfo(v, pre.."."..k, rs, index);
            end
        end
    end
    return rs;
end

-- debug 保存文件
function saveToFile(obj, path)
    local cOutputHandle = nil
    local cFile = assert(io.open(path, "w"))
    cOutputHandle = cFile
    cOutputEntry = cFile.write
    local cOutputer = function (strContent)
        if cOutputHandle then
            cOutputEntry(cOutputHandle, strContent)
        else
            cOutputEntry(strContent)
        end
    end
    if type(obj) == "table" then
        for k,v in pairs(obj) do
            cOutputer(k.."|"..v.."\n")
        end
    else
        cOutputer(obj)
    end
end

-- debug 保存当前lua数据
function saveDebugLuaDatas()
    local rs = showInfo(_G, "_G");
    local path = UnityEngine.Application.dataPath.."/StreamingAssets/Lua.txt";
    saveToFile(rs, path);
    OperateAlert.show(path.." 当前lua数据保存成功");
end

-- 时间戳转换成日期格式
function unixTimeConvertToDate(unixTime)
    if unixTime ~= nil and unixTime >= 0 then
        local tb = {
            year = tonumber(os.date("%Y", unixTime)),
            month = tonumber(os.date("%m", unixTime)),
            day = tonumber(os.date("%d", unixTime)),
            hour = tonumber(os.date("%H", unixTime)),
            minute = tonumber(os.date("%M", unixTime)),
            second = tonumber(os.date("%S", unixTime))
        }
        return tb;
    end
    return nil;
end

-- 获取到第二日0点的剩余时间 返回秒数
function getReaminSecondsTo24()
    local serverTime = DataCenter.roleData.serverTime
    local nextTime = os.time(
        { year = os.date("%Y", serverTime), month = os.date("%m", serverTime), day = os.date("%d", serverTime), hour = 24, minute = 0, second = 0 })
    return nextTime - serverTime
end

--根据时间戳计算两个日期相差多少天
function DatediffByUnixTime(date1,date2)
    local day1 = {};
    local day2 = {};
    local numDay1;
    local numDay2;

    date1 = tonumber(os.date("%Y%m%d", date1))
    date2 = tonumber(os.date("%Y%m%d", date2))
    if date1 < 19700101 or date2 < 19700101 then
        return 0;
    end
    day1.year,day1.month,day1.day = string.match(date1,"(%d%d%d%d)(%d%d)(%d%d)");
    day2.year,day2.month,day2.day = string.match(date2,"(%d%d%d%d)(%d%d)(%d%d)");
    numDay1 = os.time(day1);
    numDay2 = os.time(day2);

    return Mathf.Modf((numDay1-numDay2)/(3600*24))
end

function updateRequire(name)
    if rmodules == nil then
        return;
    end

    if rmodules[name] ~= nil then
        rmodules[name] = nil;
        local package = package;
        package.loaded[name] = nil;
        package.preload[name] = nil;
        local f = loadstring("require \""..name.."\"; return "..name..";");
        local c= f();
        rmodules[name] = c;
    end
end

--判断是否指定字符串开头
function string.StartsWith(String,Start)
    return string.sub(String,1,string.len(Start))==Start
end

--判断是否指定字符串结尾
function string.EndsWith(String,End)
    return End=='' or string.sub(String,-string.len(End))==End
end

function print_r( t )
    local print_r_cache={}
    local function sub_print_r(t,indent)
        if (print_r_cache[tostring(t)]) then
            print(indent.."*"..tostring(t))
        else
            print_r_cache[tostring(t)]=true
            if (type(t)=="table") then
                for pos,val in pairs(t) do
                    if (type(val)=="table") then
                        print(indent.."["..pos.."] => "..tostring(t).." {")
                        sub_print_r(val,indent..string.rep(" ",string.len(pos)+8))
                        print(indent..string.rep(" ",string.len(pos)+6).."}")
                    elseif (type(val)=="string") then
                        print(indent.."["..pos..'] => "'..val..'"')
                    else
                        print(indent.."["..pos.."] => "..tostring(val))
                    end
                end
            else
                print(indent..tostring(t))
            end
        end
    end
    if (type(t)=="table") then
        print(tostring(t).." {")
        sub_print_r(t,"  ")
        print("}")
    else
        sub_print_r(t,"  ")
    end
    print()
end

--将Lua消息号的数据转成字符串 方便输出查看
function msgToString(msg)
    local function printTable(tab, stepBlack)
        local s = "";
        if stepBlack == nil then 
            stepBlack = "  "; 
        else
            stepBlack = stepBlack .. "  ";
        end

        local bStart = false;
        for k in pairs(tab._fields) do
            if bStart then
                s = s .. ",\n";
            end
            bStart = true;
            if type(tab[k.name]) == "table" then
                if #tab[k.name] > 0 then
                    s = s .. stepBlack .. k.name .. " = {\n";
                    for i = 1, #tab[k.name] do
                        if type(tab[k.name][i]) == "table" then
                            if i == #tab[k.name] then
                                s = s..stepBlack.."  [".. i .. "] = {\n" ..stepBlack.."".. printTable(tab[k.name][i], stepBlack.."  ")..stepBlack.."}";
                            else
                                s = s..stepBlack.."  [".. i .. "] = {\n" ..stepBlack.."".. printTable(tab[k.name][i], stepBlack.."  ")..stepBlack.."},\n";
                            end
                        else
                            if i == #tab[k.name] then
                                s = s .. stepBlack .. "  [".. i .. "] = " .. tostring(tab[k.name][i]);
                            else
                                s = s .. stepBlack .. "  [".. i .. "] = " .. tostring(tab[k.name][i]) .. ", ";
                            end
                        end
                    end

                    s = s .. stepBlack .. "\n" .. stepBlack .. "}";
                else
                    s = s .. stepBlack .. k.name .. " = {" .. printTable(tab[k.name], stepBlack) .. "\n}";
                end
            else
                s = s .. stepBlack .. k.name .. " = " .. tostring(tab[k.name]) .. "";
            end
        end
        return s;
    end

    return "{\n" .. printTable(msg) .. "\n}";
end

--阿拉伯数字转中文
function convert_arab_to_chinese(number, bClearOne) -- bClearOne：两位数中十位数若为1不输出一
    assert(tonumber(number), "传入参数非正确number类型！")

    local numerical_names = {[0] = LanUtil.getValue(100), LanUtil.getValue(101), LanUtil.getValue(102), LanUtil.getValue(103), LanUtil.getValue(104), 
                            LanUtil.getValue(105), LanUtil.getValue(106), LanUtil.getValue(107), LanUtil.getValue(108), LanUtil.getValue(109)}
    
    local numerical_length = string.len(number)
    if numerical_length < 2 then
        return numerical_names[number];
    end

    local numerical_units = {"", LanUtil.getValue(110), LanUtil.getValue(150), LanUtil.getValue(151), LanUtil.getValue(152),
                             LanUtil.getValue(110), LanUtil.getValue(150), LanUtil.getValue(151), LanUtil.getValue(153),
                             LanUtil.getValue(110), LanUtil.getValue(150), LanUtil.getValue(151), LanUtil.getValue(158),
                             LanUtil.getValue(110), LanUtil.getValue(150), LanUtil.getValue(151), LanUtil.getValue(159)}

    --数字转成表结构存储
    local numerical_tbl = {}
    for i = 1, numerical_length do
        numerical_tbl[i] = tonumber(string.sub(number, i, i))
    end

    --对应数字转中文处理
    local result_numberical = ""
    local to_append_zero, need_filling = false, true
    for index, number in ipairs(numerical_tbl) do
        --从高位到底位的顺序数字转成对应的从低位到高位的顺序数字单位.
        local real_unit_index = numerical_length - index + 1
        if number == 0 then
            if need_filling then
                if real_unit_index == 5 then--万位
                    result_numberical = result_numberical .. LanUtil.getValue(152)
                    need_filling = false
                end
                if real_unit_index == 9 then--亿位
                    result_numberical = result_numberical .. LanUtil.getValue(153)
                    need_filling = false
                end
                if real_unit_index == 13 then--兆位
                    result_numberical = result_numberical .. LanUtil.getValue(158)
                    need_filling = false
                end
                if real_unit_index == 17 then--京位
                    result_numberical = result_numberical .. LanUtil.getValue(159)
                    need_filling = false
                end
            end
            to_append_zero = true
        else
            if to_append_zero then
                result_numberical = result_numberical .. LanUtil.getValue(100)
                to_append_zero = false
            end
            
            if bClearOne and index == #numerical_tbl - 1 and #numerical_tbl == 2 and number == 1 then -- 两位数若十位数为1，不输出一
                result_numberical = result_numberical  .. numerical_units[real_unit_index];
            else
                result_numberical = result_numberical  .. numerical_names[number] .. numerical_units[real_unit_index]
            end

            if real_unit_index == 5 or real_unit_index == 9 or real_unit_index == 13 or real_unit_index == 17 then
                need_filling = false
            else
                need_filling = true
            end
        end
    end
    return result_numberical
end

--格式化倒计时 时间为 00:00:09
-- 例如：10 9 8 7 6 5 4 3 2 1
function formatTime(iTime)
    local function formatNum(iNum)
        return iNum < 10 and "0"..iNum or iNum
    end
    return string.format("%s:%s:%s", formatNum(Mathf.Floor(iTime / 3600)),formatNum(Mathf.Floor((iTime % 3600) / 60)),formatNum(Mathf.Floor((iTime % 3600) % 60)))
end

-- 数值单位转换
-- transType：默认 -> 万 亿 兆 京 1 -> 万 亿 万亿 万万亿
-- decimalNum 保留n位小数 默认1位
function numberTrans(val, transType, decimalNum)
    val = tonumber(tostring(val));

    if val < 100000 then -- 1-99999
        return tostring(val);
    end

    decimalNum = decimalNum or 1
    local count;
    if val < 100000000 then -- 10万-9999.9万
        count = Mathf.Floor(val / 1000);
        count = count / 10;
        count = getPreciseDecimal(count, decimalNum)
        return LanUtil.StringProfile(tostring(count), LanUtil.getValue(152));
    end

    if val < 1000000000000 then -- 1亿-9999.9亿
        count = Mathf.Floor(val / 10000000);
        count = count / 10;
        count = getPreciseDecimal(count, decimalNum)
        return LanUtil.StringProfile(tostring(count), LanUtil.getValue(153));
    end

    if val < 10000000000000000 then -- 1万亿-9999.9万亿 1兆-9999.9兆
        count = Mathf.Floor(val / 100000000000);
        count = count / 10;
        count = getPreciseDecimal(count, decimalNum)
        if transType == 1 then
            return LanUtil.StringProfile(tostring(count), LanUtil.getValue(155));
        end
        return LanUtil.StringProfile(tostring(count), LanUtil.getValue(158));
    end

    count = Mathf.Floor(val / 1000000000000000); -- 1京-9999.9京
    count = count / 10;
    count = getPreciseDecimal(count, decimalNum)
    if transType == 1 then
        return LanUtil.StringProfile(tostring(count), LanUtil.getValue(156));
    end
    return LanUtil.StringProfile(tostring(count), LanUtil.getValue(159));
end

-- 数值艺术字转换
-- transType：默认 -> 万 亿 兆 京
-- decimalNum 保留n位小数 默认1位
function numImageTrans(val, transType, decimalNum)
    val = tonumber(tostring(val));

    if val < 100000 then
        return tostring(val);
    end

    decimalNum = decimalNum or 1
    local count;
    if val < 100000000 then
        count = Mathf.Floor(val / 1000);
        count = count / 10;
        count = getPreciseDecimal(count, decimalNum)
        return LanUtil.StringProfile(tostring(count), "w");
    end

    if val < 1000000000000 then
        count = Mathf.Floor(val / 10000000);
        count = count / 10;
        count = getPreciseDecimal(count, decimalNum)
        return LanUtil.StringProfile(tostring(count), "y");
    end

    if val < 10000000000000000 then
        count = Mathf.Floor(val / 100000000000);
        count = count / 10;
        count = getPreciseDecimal(count, decimalNum)
        return LanUtil.StringProfile(tostring(count), "z");
    end

    count = Mathf.Floor(val / 1000000000000000);
    count = count / 10;
    count = getPreciseDecimal(count, decimalNum)
    return LanUtil.StringProfile(tostring(count), "j");
end

-------------------- lua的深拷贝和浅拷贝 --------------------
--- Deep copies a table into a new table.
-- Tables used as keys are also deep copied, as are metatables
-- @param orig The table to copy
-- @return Returns a copy of the input table
function deep_copy(orig)
    local copy
    if type(orig) == "table" then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deep_copy(orig_key)] = deep_copy(orig_value)
        end
        setmetatable(copy, deep_copy(getmetatable(orig)))
    else
        copy = orig
    end
    return copy
end

--- Copies a table into a new table.
-- neither sub tables nor metatables will be copied.
-- @param orig The table to copy
-- @return Returns a copy of the input table
function shallow_copy(orig)
    local copy
    if type(orig) == "table" then
        copy = {}
        for orig_key, orig_value in pairs(orig) do
            copy[orig_key] = orig_value
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

-- 保留指定小数位数
function getPreciseDecimal(num, n)
    if type(num) ~= "number" then
        return num;
    end
    
    n = n or 0;
    n = Mathf.Floor(n)
    if n < 0 then
        n = 0;
    end
    local nDecimal = 10 ^ n
    local nTemp = Mathf.Floor(num * nDecimal);
    local nRet = nTemp / nDecimal;
    return nRet;
end

-- 判断UTF-8字符byte长度
-- 字符串的首个byte表示了该utf8字符的长度
-- 0xxxxxxx - 1 byte
-- 110yxxxx - 192, 2 byte
-- 1110yyyy - 225, 3 byte
-- 11110zzz - 240, 4 byte
function chsize(char)
    if not char then
        return 0
    elseif char > 240 then
        return 4
    elseif char > 225 then
        return 3
    elseif char > 192 then
        return 2
    else
        return 1
    end
end

-- 计算UTF-8字符串字符数, 各种字符都按一个字符计算
function utf8len(str)
    local len = 0
    local currentIndex = 1
    while currentIndex <= #str do
        local char = string.byte(str, currentIndex)
        currentIndex = currentIndex + chsize(char)
        len = len +1
    end
    return len
end

-- 截取UTF-8字符串
-- str:          要截取的字符串
-- startChar:    开始字符下标,从1开始
-- numChars:     要截取的字符长度
function utf8sub(str, startChar, numChars)
    local startIndex = 1
    while startChar > 1 do
        local char = string.byte(str, startIndex)
        startIndex = startIndex + chsize(char)
        startChar = startChar - 1
    end

    local currentIndex = startIndex
    while numChars > 0 and currentIndex <= #str do
        local char = string.byte(str, currentIndex)
        currentIndex = currentIndex + chsize(char)
        numChars = numChars -1
    end
    return str:sub(startIndex, currentIndex - 1)
end

function strIsNullOrEmpty(str)
    if str == nil or str == "" then
        return true;
    end
    return false;
end

function randomString(modl, len)
    local BC = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    local SC = "abcdefghijklmnopqrstuvwxyz";
    local NO = "0123456789";
    local maxLen = 0;
    local temp = "";
    if modl == 1 then -- 纯大写字母
        maxLen = 26;
        temp = BC;
    elseif modl == 2 then -- 纯小写字母
        maxLen = 26;
        temp = SC;
    elseif modl == 3 then -- 大小写字母
        maxLen = 52;
        temp = BC..SC;
    elseif modl == 4 then -- 纯数字
        maxLen = 10;
        temp = NO;
    elseif modl == 5 then -- 大写字母+数字
        maxLen = 36;
        temp = BC..NO;
    elseif modl == 6 then -- 小写字母+数字
        maxLen = 36;
        temp = SC..NO;
    else                  -- 大小写字母+数字
        maxLen = 62;
        temp = BC..SC..NO;
    end

    -- 用随机结果放大处理后再置随机种子
    math.randomseed(os.clock() * Mathf.Random(1000000, 90000000) + Mathf.Random(1000000, 90000000));

    local str = {};
    local index;
    for i = 1, len do
        index = Mathf.Random(1, maxLen);
        str[i] = string.sub(temp, index, index);
    end
    return table.concat( str, "" );
end

-- 拼接字符串
function spliceStr(str1, str2, str3, str4, str5, str6, str7, str8, str9, ...)
    spliceTab[1] = str1;
    spliceTab[2] = str2;
    spliceTab[3] = str3;
    spliceTab[4] = str4;
    spliceTab[5] = str5;
    spliceTab[6] = str6;
    spliceTab[7] = str7;
    spliceTab[8] = str8;
    spliceTab[9] = str9;
    spliceTab[10], spliceTab[11], spliceTab[12] = ...;
    return table.concat( spliceTab, "" );
end

-- 拼接表情字符串
function spliceEmojiStr(sprite, width, height, urlStr)
    spliceStr();
    spliceTab[1] = "0x01";
    spliceTab[2] = sprite == nil and "" or sprite;
    spliceTab[3] = width == nil and 0 or width;
    spliceTab[4] = height == nil and 0 or height;
    spliceTab[5] = urlStr;
    return spliceStr("[", table.concat( spliceTab, "|" ), "]");
end

-- 拼接图标字符串
function spliceIconStrById(id, urlStr)
    local tvo = Lua_ToolVo.createToolVo(id);
    return spliceIconStr(tvo.iconAtlas, tvo.iconSprite, tvo.iconWidth, tvo.iconHeight, urlStr);
end

-- 拼接图标字符串
function spliceIconStr(atlas, sprite, width, height, urlStr)
    spliceStr();
    spliceTab[1] = "0x02";
    spliceTab[2] = atlas == nil and "" or atlas;
    spliceTab[3] = sprite == nil and "" or sprite;
    spliceTab[4] = width == nil and 0 or width;
    spliceTab[5] = height == nil and 0 or height;
    spliceTab[6] = urlStr;
    return spliceStr("[", table.concat( spliceTab, "|" ), "]");
end

-- 拼接纹理字符串
function spliceImgStr(imgUrl, width, height, urlStr)
    spliceStr();
    spliceTab[1] = "0x03";
    spliceTab[2] = imgUrl == nil and "" or imgUrl;
    spliceTab[3] = width == nil and 0 or width;
    spliceTab[4] = height == nil and 0 or height;
    spliceTab[5] = urlStr;
    return spliceStr("[", table.concat( spliceTab, "|" ), "]");
end

-- 战力计算
function calculateForce(tab, param)
    if tab then
        param = param or getConfig("tb_fightrule", 1, "rule")
        local arr = DataCenter.attributesUtil.attributeIdsArr;
        local name
        local value
        for i = 1, arr:getSize() do
            name = arr[i]
            value = tab[name]
            if value == nil then
                param = string.gsub(param, "%["..name.."%]", 0)
            else
                param = string.gsub(param, "%["..name.."%]", tostring(value))
            end
        end
        local script = "return "..param
        local func = loadstring(script)
        if func then
            return func()
        end
    end
    return 0
end

-- 多权重快速排序
function quickSortf(arr, owner, sortfunction, param, beginIdx, endIdx)
    if beginIdx == nil then beginIdx = 1; end
    if endIdx == nil then endIdx = arr:getSize(); end
    if beginIdx > endIdx then
        return;
    end

    if endIdx - beginIdx <= 8 then
        insertSortf(arr, owner, sortfunction, param, beginIdx, endIdx);
        return;
    end

    local tmp = arr[beginIdx];
    local i = beginIdx;
    local j = endIdx;
    while (i ~= j) do
        while (sortfunction(owner, arr[j], tmp, param) and j > i) do
            j = j - 1;
        end

        while (sortfunction(owner, tmp, arr[i], param) and j > i) do
            i = i + 1;
        end

        if (j > i) then
            local val = arr[i];
            arr[i] = arr[j];
            arr[j] = val;
        end
    end

    arr[beginIdx] = arr[i];
    arr[i] = tmp;
    quickSortf(arr, owner, sortfunction, param, beginIdx, i - 1);
    quickSortf(arr, owner, sortfunction, param, i + 1, endIdx);
end

-- 快速排序
function quickSort(arr, name, up, beginIdx, endIdx)
    if beginIdx == nil then beginIdx = 1; end
    if endIdx == nil then endIdx = arr:getSize(); end
    if beginIdx > endIdx then
        return;
    end

    if endIdx - beginIdx <= 8 then
        insertSort(arr, name, up, beginIdx, endIdx);
        return;
    end

    local tmp = arr[beginIdx];
    local i = beginIdx;
    local j = endIdx;
    while (i ~= j) do
        if name == nil then
            while (((up and arr[j] >= tmp) or (not up and arr[j] <= tmp)) and j > i) do
                j = j - 1;
            end

            while (((up and arr[i] <= tmp) or (not up and arr[i] >= tmp)) and j > i) do
                i = i + 1;
            end
        else
            while (((up and arr[j][name] >= tmp[name]) or (not up and arr[j][name] <= tmp[name])) and j > i) do
                j = j - 1;
            end

            while (((up and arr[i][name] <= tmp[name]) or (not up and arr[i][name] >= tmp[name])) and j > i) do
                i = i + 1;
            end
        end

        if (j > i) then
            local val = arr[i];
            arr[i] = arr[j];
            arr[j] = val;
        end
    end

    arr[beginIdx] = arr[i];
    arr[i] = tmp;
    quickSort(arr, name, up, beginIdx, i - 1);
    quickSort(arr, name, up, i + 1, endIdx);
end

-- 插入排序
function insertSort(arr, name, up, beginIdx, endIdx)
    if beginIdx == nil then beginIdx = 1; end
    if endIdx == nil then endIdx = arr:getSize(); end
    for i = beginIdx, endIdx - 1 do
        local ev = i;
        local tmp = arr[i + 1];
        while ev >= beginIdx do
            if name == nil then
                if (up and tmp < arr[ev]) or (not up and tmp > arr[ev])  then
                    arr[ev + 1] = arr[ev];
                    ev = ev - 1;
                else 
                    break;
                end
            else
                if (up and tmp[name] < arr[ev][name]) or (not up and tmp[name] > arr[ev][name]) then
                    arr[ev + 1] = arr[ev];
                    ev = ev - 1;
                else 
                    break;
                end
            end
        end
        arr[ev + 1] = tmp;
    end
end

-- 多权重插入排序
function insertSortf(arr, owner, sortfunction, param, beginIdx, endIdx)
    if beginIdx == nil then beginIdx = 1; end
    if endIdx == nil then endIdx = arr:getSize(); end
    for i = beginIdx, endIdx - 1 do
        local ev = i;
        local tmp = arr[i + 1];
        while ev >= beginIdx do
            if not sortfunction(owner, tmp, arr[ev], param) then
                arr[ev + 1] = arr[ev];
                ev = ev - 1;
            else
                break;
            end
        end
        arr[ev + 1] = tmp;
    end
end

-- 计算字符串 字符个数
function calculateStrNum(str)
    if str == nil then
        return 0;
    end

    local charNum = 0;
    local strLen = string.len(tostring(str));
    local startByte = 1;
    local byteLen;
    local tempStr;
    while startByte <= strLen do
        tempStr = string.sub(str, startByte, strLen);
        if tempStr == nil then
            break;
        end
        charNum = charNum + 1;
        byteLen = charWidth(tempStr);
        startByte = startByte + byteLen;
        -- print(tempStr, startByte, strLen,byteLen, charNum)
    end
    return charNum;
end

-- 空白字符一般包括空格、换行符\n、制表符\t以及回到行首符\r
-- 忽略字符串头部的空白字符
function ltrim(input)
    return string.gsub(input, "^[ \t\n\r]+", "")
end

-- 忽略字符串尾部的空白字符
function rtrim(input)
    return string.gsub(input, "[ \t\n\r]+$", "")
end

-- 忽略字符串首尾的空白字符
function trim(input)
    return string.gsub(input, "^%s*(.-)%s*$", "%1")
end