-- 贝塞尔曲线
Lua_BezierUtils = {}

-- 根据T值，计算贝塞尔曲线上面相对应的点, 二维
-- t T值
-- p0 起始点
-- p1 控制点
-- p2 目标点
function Lua_BezierUtils.Calculate2DimensionalBezierPoint(t, p0, p1, p2, result)
    local u = 1 - t;
    local tt = t * t;
    local uu = u * u;
    local cc = 2 * u * t;

    if result == nil then result = Vector3(); end
    result.x = uu * p0.x + cc * p1.x + tt * p2.x;
    result.y = uu * p0.y + cc * p1.y + tt * p2.y;
    return result;
end

-- 根据T值，计算贝塞尔曲线上面相对应的点，三维Vector3
-- t T值
-- p0 起始点 Vector3
-- p1 控制点 Vector3
-- p2 目标点 Vector3
function Lua_BezierUtils.CalculateCubicBezierPoint(t, p0, p1, p2, result)
    local u = 1 - t;
    local tt = t * t;
    local uu = u * u;
    local cc = 2 * u * t;

    if result == nil then result = Vector3(); end
    result.x = uu * p0.x + cc * p1.x + tt * p2.x;
    result.y = uu * p0.y + cc * p1.y + tt * p2.y;
    result.z = uu * p0.z + cc * p1.z + tt * p2.z;
    return result;
end

-- 根据T值，计算贝塞尔曲线上面相对应的点，三维Vector3
-- t T值
-- p0 起始点 
-- p1 控制点 
-- p2 目标点 
function Lua_BezierUtils.CalculateCubicBezierPointVal(t, px0, py0, pz0, px1, py1, pz1, px2, py2, pz2)
    local u = 1 - t;
    local tt = t * t;
    local uu = u * u;
    local cc = 2 * u * t;

    if result == nil then result = Vector3(); end
    local px = uu * px0 + cc * px1 + tt * px2;
    local py = uu * py0 + cc * py1 + tt * py2;
    local pz = uu * pz0 + cc * pz1 + tt * pz2;
    return px, py, pz;
end

-- 获取存储贝塞尔曲线点的数组
-- startPoint 起始点
-- controlPoint 控制点
-- endPoint 目标点
-- segmentNum 采样点的数量
function Lua_BezierUtils.GetBeizerList(startPoint, controlPoint, endPoint, segmentNum)
    local path = Array();
    for i = 1, segmentNum do
        local t = i / segmentNum;
        local pixel = Lua_BezierUtils.CalculateCubicBezierPoint(t, startPoint, controlPoint, endPoint);
        path:add(pixel);
    end
    return path;
end