Lua_CommonUtil = {};
Lua_CommonUtil.frameList = Array();
Lua_CommonUtil.lateUpdateList = Array();
Lua_CommonUtil.delayCollectList = Array();
function Lua_CommonUtil.addTimer(funName, interval, tb, fun, param, runTime, isLateUpdate)
	if fun == nil then
		return true;
	end

	if runTime == nil or runTime <= 0 then
		runTime = -1;
	end

	local arr = nil;
	if isLateUpdate then
		arr = Lua_CommonUtil.lateUpdateList;
	else
		arr = Lua_CommonUtil.frameList;
	end

	local len = arr:getSize();
	local t = nil;
	local hasIn = false;
	for i=1,len do
		if (arr[i].table == tb or tb == nil) and arr[i].fun == fun then
			t = arr[i];
			hasIn = true;
			break;
		end
	end
	if t == nil then
		t = {};
	end
	t.name = funName;
	if interval == nil then
		interval = 0.5;
	end
	t.interval = interval;
	t.table = tb;
	t.fun = fun;
	t.param = param;
	t.lastTime = getTime();
	t.runTime = runTime;
	t.isDispose = false;
	if hasIn == false then
		arr:add(t);
	end
	return hasIn;
end

function Lua_CommonUtil.removeTimer(tb, fun, isLateUpdate, isCollect)
	local obj;
	local arr;
	local len;
	if isLateUpdate then
		arr = Lua_CommonUtil.lateUpdateList;
		len = arr:getSize();
		for i = 1, len do
			obj = arr[i];
			if (obj.table == tb or tb == nil) and obj.fun == fun then
				obj.isDispose = true;
				obj.table = nil;
				obj.fun = nil;
				obj.param = nil;
				return;
			end
		end
	elseif isCollect then
		arr = Lua_CommonUtil.delayCollectList;
		len = arr:getSize();
		for i = 1, len do
			obj = arr[i];
			if (obj.table == tb or tb == nil) and obj.fun == fun then
				obj.isDispose = true;
				obj.table = nil;
				obj.fun = nil;
				obj.param = nil;
				return;
			end
		end
	end

	arr = Lua_CommonUtil.frameList;
	len = arr:getSize();
	for i = 1, len do
		obj = arr[i];
		if (obj.table == tb or tb == nil) and obj.fun == fun then
			obj.isDispose = true;
			obj.table = nil;
			obj.fun = nil;
			obj.param = nil;
			return;
		end
	end

	arr = Lua_CommonUtil.lateUpdateList;
	len = arr:getSize();
	for i = 1, len do
		obj = arr[i];
		if (obj.table == tb or tb == nil) and obj.fun == fun then
			obj.isDispose = true;
			obj.table = nil;
			obj.fun = nil;
			obj.param = nil;
			return;
		end
	end

	arr = Lua_CommonUtil.delayCollectList;
	len = arr:getSize();
	for i = 1, len do
		obj = arr[i];
		if (obj.table == tb or tb == nil) and obj.fun == fun then
			obj.isDispose = true;
			obj.table = nil;
			obj.fun = nil;
			obj.param = nil;
			return;
		end
	end
end

function Lua_CommonUtil.clearAllTimer()
	local obj;
	local arr = Lua_CommonUtil.frameList;
	local len = arr:getSize();
	while (len > 0) do
		obj = arr:remove(len);
		obj.table = nil;
		obj.fun = nil;
		obj.param = nil;
		obj = nil;
		len = len - 1;
	end

	arr = Lua_CommonUtil.lateUpdateList;
	len = arr:getSize();
	while (len > 0) do
		obj = arr:remove(len);
		obj.table = nil;
		obj.fun = nil;
		obj.param = nil;
		obj = nil;
		len = len - 1;
	end

	arr = Lua_CommonUtil.delayCollectList;
	len = arr:getSize();
	while (len > 0) do
		obj = arr:remove(len);
		obj.table = nil;
		obj.fun = nil;
		obj.param = nil;
		obj = nil;
		len = len - 1;
	end

	arr = nil;
	obj = nil;
	Lua_CommonUtil.frameList:clear();
	Lua_CommonUtil.lateUpdateList:clear();
	Lua_CommonUtil.delayCollectList:clear();
end

function Lua_CommonUtil.delayCall(interval, tb, fun, param)
	Lua_CommonUtil.addTimer("delayCall", interval, tb, fun, param, 1)
end

function Lua_CommonUtil.delayCollectCall(name, tb, fun, param, collectTime)
	local list = Lua_CommonUtil.delayCollectList;
	local len = list:getSize();
	for i = 1, len do
		if list[i].tb == tb and t.fun == fun then
			list[i].param = param;
			return;
		end
	end

	local t = {};
	t.name = name;
	t.endTime = collectTime + getTime();
	t.table = tb;
	t.fun = fun;
	t.param = param;
	t.isDispose = false;
	list:add(t);
end

function Lua_CommonUtil.lateUpdateFrame()
	local arr = Lua_CommonUtil.lateUpdateList;
	local len = arr:getSize();
	local nt = getTime();
	local obj;
	while (len > 0) do
		obj = arr[len];
		if obj == nil then
			break;
		end
		
		if obj.isDispose == true or obj.table == nil or obj.isDispose_3 == true then
			obj.table = nil;
			obj.fun = nil;
			obj.param = nil;
			arr:remove(len);
		elseif nt - obj.lastTime > obj.interval then
			obj.lastTime = nt;
			obj.fun(obj.table, obj.param);
			if obj.isDispose ~= true and obj.runTime ~= nil and obj.runTime ~= -1 then
				obj.runTime = obj.runTime - 1;
				if obj.runTime <= 0 then
					obj.isDispose = true;
				end
			end

			-- 防止回调移除导致报错
			if obj.isDispose == true then
				obj.table = tb;
				obj.fun = fun;
				obj.param = param;
				arr:remove(len);
			end
		end

		len = len - 1;
	end	
end

function Lua_CommonUtil.frameScript()
	local arr = Lua_CommonUtil.frameList;
	local len = arr:getSize();
	local nt = getTime();
	local obj;
	while (len > 0) do
		obj = arr[len];
		if obj == nil then
			break;
		end
		
		if obj.isDispose == true or obj.table == nil or obj.isDispose_3 == true then
			obj.table = nil;
			obj.fun = nil;
			obj.param = nil;
			arr:remove(len);
		elseif nt - obj.lastTime > obj.interval then
			obj.lastTime = nt;
			obj.fun(obj.table, obj.param);
			if obj.isDispose ~= true and obj.runTime ~= nil and obj.runTime ~= -1 then
				obj.runTime = obj.runTime - 1;
				if obj.runTime <= 0 then
					obj.isDispose = true;
				end
			end

			-- 防止回调移除导致报错
			if obj.isDispose == true then
				obj.table = tb;
				obj.fun = fun;
				obj.param = param;
				arr:remove(len);
			end
		end

		len = len - 1;
	end

	arr = Lua_CommonUtil.delayCollectList;
	len = arr:getSize();
	while (len > 0) do
		obj = arr[len];
		if obj.isDispose == true or obj.table == nil or obj.isDispose_3 == true then
			obj.table = nil;
			obj.fun = nil;
			obj.param = nil;
			arr:remove(len);
		elseif nt > obj.endTime then
			obj.fun(obj.table, obj.param);
			obj.table = nil;
			obj.fun = nil;
			obj.param = nil;
			arr:remove(len);
		end 

		len = len - 1;
	end
end

function Lua_CommonUtil.outputTimerInfo()
	local arr = Lua_CommonUtil.frameList;
	if arr ~= nil then
		local len = arr:getSize();
		for i = 1, len do 
			local d = arr[i].isDispose;
			if d == nil then
				d = false;
			end

			if arr[i].table ~= nil then
				if arr[i].table.windowName ~= nil then
					print("arr[i].table.windowName =", arr[i].table.windowName, d, arr[i].interval, arr[i].runTime)
				elseif arr[i].table.classname ~= nil then
					print("arr[i].table.classname =", arr[i].table.classname, d, arr[i].interval, arr[i].runTime)
				elseif arr[i].table.name ~= nil then
					print("arr[i].table.name =", arr[i].table.name, d, arr[i].interval, arr[i].runTime)
				else
					print("11111 outputTimerInfo", d, arr[i].interval, arr[i].runTime)
				end
			else
				print("22222 outputTimerInfo", d, arr[i].interval, arr[i].runTime)
			end
		end
	end
end

function Lua_CommonUtil.getCommonConfig(key, valType, defaultVal)
	local config = getConfig("const", key);
	if config == nil then
		return defaultVal;
	end
	
	if valType == 1 then
		return config.value;
	elseif valType == 2 then
		return config.strvalue;
	elseif valType == 3 then
		return config.jsonvalue;
	elseif valType == 4 then
		return config.floatvalue;
	end
	return defaultVal;
end

function Lua_CommonUtil.transChineseNumber(num)
	num = tonumber(num);
	if num == 0 then
		return LanUtil.getValue(100);
	end

	if num <= 10 then
		return LanUtil.getValue(num + 100);
	end

	if num < 20 then
		return LanUtil.getValue(110)..LanUtil.getValue(num - 10 + 100);
	end

	if num < 100 then
		local f = Mathf.Floor(num / 10);
		local g = num % 10;
		if g == 0 then
			return LanUtil.getValue(f + 100)..LanUtil.getValue(110);
		else
			return LanUtil.getValue(f + 100)..LanUtil.getValue(110)..LanUtil.getValue(g + 100);
		end
	end

	if num < 1000 then
		local b = Mathf.Floor(num / 100);
		local s = num-b*100;
		local g = num % 100;
		local z = s % 10;
		if g == 0 then
			return LanUtil.getValue(b + 100)..LanUtil.getValue(150);
		else
			local f = Mathf.Floor(s / 10);
			local m = Mathf.Floor(s-f*10);
			if z == 0 then
				return LanUtil.getValue(b + 100)..LanUtil.getValue(150)..LanUtil.getValue(f + 100)..LanUtil.getValue(110);
			else
				if s >9 then
					return LanUtil.getValue(b + 100)..LanUtil.getValue(150)..LanUtil.getValue(f + 100)..LanUtil.getValue(110)..LanUtil.getValue(m + 100); 
				elseif s <= 9 then
					m = Mathf.Floor(s);
					return LanUtil.getValue(b + 100)..LanUtil.getValue(150)..LanUtil.getValue(100)..LanUtil.getValue(m + 100);
				end
			end
		end
	end
	return tostring(num);
end

function Lua_CommonUtil.fixNumerLen(val, len)
	local str = tostring(val);
	while true do
		local l = string.len(str);
		if l >= len then
			break;
		end
		str = "0"..str;
	end
	return str;
end

function Lua_CommonUtil.checkUnitDistance(sx,sy,sz,tx,ty,tz,dis)
	--判断两个点的距离是否在 检测距离之内
	local dx = sx - tx;
	local dy = sy - ty;
	local dz = sz - tz;
	local cDis = dx * dx + dy * dy + dz * dz;
	return dis > cDis and true or false;
end

function Lua_CommonUtil.transMoney(val)
	local str = tostring(val);
	local strList = {};
	local index = 1;
	while true do
		local len = string.len(str);
		if len > 3 then
			strList[index] = string.sub(str, -3, -1);
			str = string.sub(str, 1, -4);
		else
			strList[index] = str;
			break;
		end
		index = index + 1;
	end
	local len = #strList;
	local rs = "";
	for i,v in ipairs(strList) do
		if i ~= 1 then
			rs = v..",".. rs;
		else
			rs = v;
		end
	end
	return rs;
end

function Lua_CommonUtil.checkIsInvalid(str, checkCount, preStr)
	if not strIsNullOrEmpty(str) then
		preStr = preStr == nil and "" or preStr
		-- 含有敏感字符
		if LanUtil.isIncludeInvalidName(str) or LanUtil.isIncludeInvalidWord(str) then
			OperateAlert.show(preStr..LanUtil.getValue(123655));
			return true;
		end

		-- 链接
		local si, ei = string.find(str, "\\");
		if si ~= nil and si > 0 then
			OperateAlert.show(preStr..LanUtil.getValue(123655));
			return true;
		end

		-- 数字或者字母太多
		if checkCount ~= false and (LanUtil.checkStrNumberCount(str) >= 7 or LanUtil.checkStrEngAlphabetCount(str) >= 5) then
			OperateAlert.show(preStr..LanUtil.getValue(123658));
			return true;
		end
	end

	return false;
end