-- 弹框管理
Lua_PopupWinManager = class("Lua_PopupWinManager");
function Lua_PopupWinManager:ctor()
	self:superCall("ctor");
	self:dispose();
end

function Lua_PopupWinManager:dispose()
	self.popupWinDataDic = {}; -- 弹窗控制数据
	self.popupWinDataArr = {}; -- 弹窗控制数据
	self.popupWinNameDic = {}; -- 当前弹窗窗口
end

function Lua_PopupWinManager:checkPopupWin()
	if not DataCenter.loginData.isInGameWorld then
		return false;
	end

	-- 扭蛋动画
	if DataCenter.gashaponData.isDrawTween then
		return false;
	end

	-- 百发百中动画
	if DataCenter.crackShotData.isCrackShotTween then
		return false;
	end

	-- print("checkPopupWin", DataCenter.fightData.isFighting);
	if DataCenter.fightData.isFighting then
		return false;
	end

	if not DataCenter.guideData.bInitServerInfo then
		return false;
	end

	local vo;
	local bPopup = false;
	local guideState = DataCenter.guideData.isGuiding;
	for popupType, arr in pairs(self.popupWinDataDic) do
		if self.popupWinNameDic[popupType] == nil then
			local len = arr:getSize();
			for i = 1, len do
				local winName = arr[i];
				local list = self.popupWinDataArr[winName];
				if list ~= nil then
					for j = 1, list:getSize() do
						vo = list[j];
						if vo ~= nil and vo:checkCanPopup(guideState) then -- 引导允许弹框的界面
							bPopup = true;
							break;
						end
					end
				end

				if bPopup then
					break;
				end
			end

			if bPopup and vo ~= nil then
				if not Lua_WindowUtil.isOpenWin(vo.winName) then
					self.popupWinNameDic[popupType] = vo
					Lua_WindowUtil.openWin(vo.winName, Lua_WinParam.ACTION_OPEN, vo.data, true);
					-- print("openWin1", vo.winName);
					-- print("checkPopupWin show popup win", vo.winName, getTraceback())
				end
				break;
			end
		else
			vo = self.popupWinNameDic[popupType];
			if not strIsNullOrEmpty(vo.winName) and not Lua_WindowUtil.isOpenWin(vo.winName) then -- 非正常关闭的
				bPopup = true;
				Lua_WindowUtil.openWin(vo.winName, Lua_WinParam.ACTION_OPEN, vo.data, true);
				-- print("22222checkPopupWin show popup win", vo.winName, getTraceback())
				break;
			end
		end
	end
	return bPopup;
end

function Lua_PopupWinManager:checkPopupWinData(winName, act, data)
	if ConfigManager.getInstance().firstConfigLoaded ~= 1 then
		print("未初始化配置checkPopupWinData winName", winName);
        return false;
    end

	if act == Lua_WinParam.ACTION_CLOSE then
		-- self:removeDataByName(winName)
		-- print("checkPopupWinData -- self:removeDataByName(winName)", winName);
		return false
	end

	local configData = Lua_WinConfigData.getConfigData(winName)
	if configData and configData.popupType > 0 then
		local vo;
		local bAddData = false;
		if not DataCenter.loginData.isInGameWorld then -- 未进入游戏场景
			bAddData = true;
		elseif DataCenter.fightData.isFighting then -- 战斗中
			bAddData = true;
		elseif DataCenter.gashaponData.isDrawTween then -- 扭蛋动画
			bAddData = true;
		elseif DataCenter.crackShotData.isCrackShotTween then -- 百发百中动画
			bAddData = true;
		elseif not DataCenter.guideData.bInitServerInfo then -- 引导还未初始化
			bAddData = true;
		elseif DataCenter.guideData.isGuiding then -- 引导中
			if not DataCenter.guideCommonData:checkGuideUsePopupWin(winName) then
				bAddData = true;
			end
		end

		vo = self:checkData(configData, data)
		if vo == nil then
			vo = self:createPopupVo(winName, act, data, configData);
			self:addData(configData, vo)
		else
			vo:setData(winName, act, data, configData);
			-- print("setData", winName, getTraceback());
		end

		if bAddData then -- 不允许弹框
			return true;
		end

		local curPopupVo = self.popupWinNameDic[configData.popupType];
		if curPopupVo ~= nil then 
			if curPopupVo == vo then -- 当前显示的vo
				-- print("checkPopupWinData show popup win 当前显示的vo", vo.winName, getTraceback())
				return false;
			else
				return true;
			end
		end

		if not vo:checkCanPopup() then
			return true;
		end
		self.popupWinNameDic[configData.popupType] = vo
	end

	-- print("checkPopupWinData show popup win", winName, getTraceback())
	return false
end

function Lua_PopupWinManager:createPopupVo(winName, act, data, configData)
	local popupVo = nil;
	if self.idlePopupVoList ~= nil then
		popupVo = self.idlePopupVoList:remove();
	end
	if popupVo == nil then
		popupVo = Lua_PopupWinInfoVo.new();
	end
	popupVo:setData(winName, act, data, configData);
	return popupVo;
end

function Lua_PopupWinManager:checkData(configData, data)
	local popupOverlap = configData.popupOverlap;
	if popupOverlap and data == nil then
		return nil;
	end

	local winName = configData.winName;
	if self.popupWinDataArr[winName] ~= nil then
		local arr = self.popupWinDataArr[winName];
		for i = 1, arr:getSize() do
			if popupOverlap then -- 可叠加
				if arr[i].data == data then 
					return arr[i];
				end
			else
				return arr[i];
			end
		end
	end
	return nil
end

function Lua_PopupWinManager:addData(configData, popupWinVo)
	local winName = configData.winName;
	if self.popupWinDataDic[configData.popupType] == nil then
		self.popupWinDataDic[configData.popupType] = Array();
	end
	local arr = self.popupWinDataDic[configData.popupType];
	if arr:indexOf(winName) == -1 then
		arr:add(winName);

		-- 排序
		if arr:getSize() > 1 then
	        local a;
	        local b;
			local len = arr:getSize();
	        for i = 1, len do
	            for j = i + 1, len do
	            	a = Lua_WinConfigData.getConfigData(arr[i]);
	            	b = Lua_WinConfigData.getConfigData(arr[j]);
	                if a.popupSort > b.popupSort then
	                    local temp = arr[i];
	                    arr[i] = arr[j];
	                    arr[j] = temp;
	                end
	            end
	        end
	    end
	end

	if self.popupWinDataArr[winName] == nil then
		self.popupWinDataArr[winName] = Array();
	end
	self.popupWinDataArr[winName]:add(popupWinVo);
	-- print("addData", popupWinVo.winName, getTraceback());
end

function Lua_PopupWinManager:removeData(configData)
	if configData == nil or configData.popupType <= 0 then
		return;
	end

	local curPopupVo = self.popupWinNameDic[configData.popupType];
	if curPopupVo ~= nil and curPopupVo.winName == configData.winName then
		local winName = curPopupVo.winName;
		local list = self.popupWinDataArr[winName];
		if list ~= nil and list:removeItem(curPopupVo) and list:getSize() <= 0 then
			local arr = self.popupWinDataDic[configData.popupType];
			if arr ~= nil then arr:removeItem(winName); end
		end

		curPopupVo:dispose();
		self.popupWinNameDic[configData.popupType] = nil;
		-- print("removeData", configData.popupType, configData.winName, self.popupWinDataDic[configData.popupType]:getSize(), getTraceback());
		if self.idlePopupVoList == nil then
			self.idlePopupVoList = Array();
		end
		if self.idlePopupVoList:getSize() < 5 then
			self.idlePopupVoList:add(curPopupVo);
		end
	end
end

function Lua_PopupWinManager:removeDataByName(winName)
	local configData = Lua_WinConfigData.getConfigData(winName)
	self:removeData(configData)
end

function Lua_PopupWinManager:generatePopupWinsStr()
	local resultStr = "";
	local hasPopupWin = false;
	for popupType, arr in pairs(self.popupWinDataDic) do
		for i = 1, arr:getSize() do
			local winName = arr[i];
			if winName ~= nil then
				if hasPopupWin then
					resultStr = resultStr .. "、" .. winName;
				else
					hasPopupWin = true;
					resultStr = resultStr .. winName;
				end
			end
		end
	end

	if hasPopupWin then
		return resultStr;
	end
	return "暂无";
end





















-- 弹窗界面数据
Lua_PopupWinInfoVo = class("Lua_PopupWinInfoVo");
function Lua_PopupWinInfoVo:ctor()
	self:superCall("ctor");
end

function Lua_PopupWinInfoVo:setData(winName, act, data, configData)
	self.winName = winName;
	self.data = data;
	self.act = act == nil and Lua_WinParam.ACTION_AUTO or act;
	self.configData = configData;
end

function Lua_PopupWinInfoVo:dispose()
	self.winName = nil;
	self.data = nil;
	self.act = nil;
	self.configData = nil;
end

function Lua_PopupWinInfoVo.Get:sort()
	return self.configData.popupSort;
end

-- 检测当前该弹框是否可以弹，已排除战斗中情况
function Lua_PopupWinInfoVo:checkCanPopup(guideState)
	if self.configData == nil then
		-- print("checkCanPopup self.configData == nil", self.winName)
		return false;
	end

	-- 带有跳转的窗口在引导中是否可弹出
	if guideState then
		local popupGuideIds = self.configData.popupGuideIds;
		if popupGuideIds ~= nil then
			if popupGuideIds[1] == -1 then
				return false;
			end
			local canPopup;
			local guideId = DataCenter.guideData.curGuideId;
			for i = 1, #popupGuideIds do
				if popupGuideIds[i] == guideId then
					canPopup = true;
					break;
				end
			end

			if not canPopup then
				-- print("checkCanPopup 引导中", self.winName)
				return false;
			end
		end
	end

	-- 布阵时不允许弹框
	local unPopupLineup = self.configData.unPopupLineup;
	if unPopupLineup ~= nil and unPopupLineup == 1 then
		if DataCenter.heroLineupData.isLineuping then -- 布阵中
			-- print("checkCanPopup 布阵中", self.winName)
			return false;
		end
	end

	local curSceneId = DataCenter.sceneData.curSceneId;
	if curSceneId > 0 then
		-- 允许弹框场景
		local isEmpty = true;
		local allowPopup = false;
		local allowScenes = self.configData.allowScenes;
		if allowScenes ~= nil and #allowScenes > 0 then
			isEmpty = false;
			for i = 1, #allowScenes do
				if allowScenes[i] == curSceneId then
					allowPopup = true;
					break;
				end
			end

			if not allowPopup then
				-- print("checkCanPopup 不在允许弹框场景列表", self.winName)
				return false;
			end
		end

		-- 不允许弹框场景
		if isEmpty or not allowPopup then
			local expectScenes = self.configData.expectScenes;
			if expectScenes ~= nil and #expectScenes > 0 then
				for i = 1, #expectScenes do
					if expectScenes[i] == curSceneId then
						-- print("checkCanPopup 在不允许弹框场景", self.winName)
						return false;
					end
				end
			end
		end
	end

	local unPopupWin = self.configData.unPopupWin;
	if unPopupWin ~= nil then
		for i = 1, #unPopupWin do
			if Lua_WindowUtil.isOpenWin(unPopupWin[i]) then
				-- print("checkCanPopup 互斥弹框", self.winName)
				return false;
			end
		end
	end

	-- 引导中不允许弹的
	if guideState and not DataCenter.guideCommonData:checkGuideUsePopupWin(self.winName) then
		-- print("checkCanPopup 引导中不允许弹的", self.winName)
		return false;
	end

	return true;
end